%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculates below cloud scavengning coefficient as a function of prec.
%intensity (mm), T(K), P(Pa), N_particle (n_conc, m3), Dp, m, nobins(number
%of bins and rho_p(density of particle).
%
%A Marshal-Palmer raindrop distribution is used based on precipitation 
%intensity and prescirbed size range 
%
%
%Created by Peter Tunved, ITM/SU, 2015.
%
%Usage: [scav_dp, units]=Below_cloud_scaveng(p0, temp, press, N, Dp, nobins, rho_p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function scav_dp=Below_cloud_scaveng(p0, temp, press, N, Dp, nobins, rho_p)

if nargin==7



dDrops=0.25;                                                                 %mm
Drops=[0.1:dDrops:6];                                                         %Marshal-Palmer (MP) droplet distribution 
                                                                            %betwen 0.1 and 5mm droplets
                                                                            
N_drops=marshall_palmer(p0, Drops)*dDrops;                                   %MP function  (drops per bin, dN). Function
                                                                            %gives dN/dDp, thus integration over dDp necessary.
                                                                            
Drops=Drops*1e-3;                                                           %from mm to m
                                                                            
rho_w=1000;                                                                 %Density water, kg/m3
                                                                 
[Ut, Re]=terminal_v(temp, press, Drops, rho_w);                             %Terminal velocity rain drop, (m/s)

scav_dp(1:nobins+1)=0;


for i=1:size(Drops, 2)                                                      %RAINDROP BINS
    for j=1:nobins+1
    if (N(j)<0) 
        N(j)=0;
    end
    [E]=scav_eff(temp, press, Dp(j), Drops(i), rho_p, Re(i), Ut(i));        %Scavenging efficiency
    scav_dp(j)=scav_dp(j)+pi/4*(Drops(i))^2*Ut(i).*E.*N_drops(i);           %Scavengning coefficient s-1
    end
end
units='s-1';
else
    error('Check your input...')
end