!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
!
! Contains subroutines and functions to initialize the particle size distribution
!	and to calculate the water and ammonia content of the particles
!
! UHMA - University of Helsinki Multicomponent Aerosol model
! (c) Hannele Korhonen 2003
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

module distribution_dyn

  use global_parameters
  
  implicit none

  private
  public :: init_dist, equilibrium_size

contains


!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
!
! Subroutines INIT_DIST and NUMBER_CONCENTRATION solve the initial distribution and
!	divide the particles to size sections according to input data.
!   The routines give as an output the dry radius and initial number concentration 
!	in each section as well as the volume concentrations of each compound in each section.
!
!	r_min, r_max		=	smallest and largest particle radius in distribution (m)
!	n_mode, sigma, rp	=	number concentration (m^-3), std and mean radius (m) 
!							of each log-normal mode
!	mfrac				=	mass fractions of different compounds in each mode
!	
! Determines values for global variables:
!	rdry				=	dry radius of particle in each section (m)
!	n_conc				=	number conc.			-"-			 (m^-3)
!	vol_conc			=	mass conc. of each compound in each section (um^3/m^3)
!	core				=	volume of dry particles (m^3)
!	uppervol, lowervol	=	upper and lower volumes of section boundaries (m^3)
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine init_dist(r_min, r_max, n_mode, sigma, rp, mfrac)

	implicit none

	real*8, intent(in) :: r_min, r_max
	real*8, dimension(modes), intent(in) :: n_mode, sigma, rp
	real*8, dimension(modes,compo), intent(in) :: mfrac

	integer :: jj, kk
	real*8 :: log_rdry, vol, width
	real*8, dimension(modes) :: n_onemode, dens_part


	! checking that the sum of mass fractions in each mode equals one
	call fracsum(mfrac)

	! density of particles in each log-normal mode (kg/m^3)
	! NOTE: ideal solution assumed
	do jj = 1,modes
		dens_part(jj) = sum(mfrac(jj,:)/density)
	end do
	dens_part = 1./dens_part
	
	width = (log10(r_max)-log10(r_min))/sections	! width of section (log-scale)

	n_conc = 0.
	vol_conc = 0.

	do kk = 1,sections						! initializing one section at a time

		log_rdry = log10(r_min)+width*(kk-1)		
		rdry(kk) = 10**log_rdry				! dry radius of particle in section (m)
		core(kk) = 4.*pi/3.*rdry(kk)**3		! dry volume -"- (m^3)
		midvol(kk) = core(kk)


		! needed for moving center approach
		uppervol(kk) = 4.*pi/3.*(10**(log_rdry + 0.5*width))**3	! highest possible volumes in sections
		lowervol(kk) = 4.*pi/3.*(10**(log_rdry - 0.5*width))**3	! lowest possible volumes in sections

		! number concentration contribution of each individual mode to size section (m^-3) 
		call number_concentration(n_mode, sigma, rp, log_rdry, width, n_onemode)		
 
		n_conc(kk) = sum(n_onemode)		! total number concentration in section (m^-3)

		!if (n_conc(kk) < n_limit) then	! check whether number concentration negligible
		!	n_conc(kk) = 0.
		!	vol_conc(kk,:) = 0.
		!	goto 100
	!	end if

		! volume concentration of each compound in section (um^3/m^3)
		do jj = 1, modes 
			vol_conc(kk,:) = vol_conc(kk,:) + mfrac(jj,:)*dens_part(jj)/density*core(kk)*n_onemode(jj)
		end do
		vol_conc(kk,:) = vol_conc(kk,:)*scale**3		

100		continue
	end do

  end subroutine init_dist

!--------------------------------------------------------------------------------
!
!	Calculates the contribution of the input log-normal modes to a certain
!	 size section number concentration (numerical integration to find the area
!	 under the distribution curve dN/d(logDp) )
!
!	n_mode, sigma, rp	=	number concentration (m^-3), std and mean radius (m) 
!							of each log-normal mode
!	rdlog				=	logarithm of size section radius
!	width				=	width of the section
!	n_onemode			=	contribution of modes to size section particle 
!							concentration(m^3)
!
!--------------------------------------------------------------------------------

  subroutine number_concentration(n_mode, sigma, rp, rdlog, width, n_onemode)

	implicit none

	real*8, dimension(modes), intent(in) :: n_mode, sigma, rp
	real*8, intent(in) :: rdlog, width
	real*8, dimension(modes), intent(out) :: n_onemode

	integer :: ii, jj
	real*8 :: dist, expo, low, r_i, conv, steps, stepsize

	conv = 10.					!needed for conversion: log10(step) = log(step)/log(10)

	low = rdlog - 0.5*width		!lower boundary of the bin
	stepsize = 1.d-5			!size and number of integration steps
	steps = nint(width/stepsize)

	n_onemode=0.       

	! the size sections is divided into 'slices' and the particle concentration
	! in that slice solved integrating the log-normal normal distribution function numerically
	do jj = 1, modes
		do ii = 1,steps

			r_i = low + (ii-1)*stepsize	
			r_i = 10.**r_i				!radius of size section slice in consideration 
    		
			expo = -( log(r_i/rp(jj)) )**2 / (2*log(sigma(jj))**2)
			if (expo < -700.) expo = -700.							! can't handle < exp(-700)
			
			! value of log-normal distribution dN/d(lnDp) at r_i 
			dist = n_mode(jj)/( sqrt(2.*pi)*log(sigma(jj)) )*exp(expo) 

			n_onemode(jj) = n_onemode(jj) + stepsize*log(conv)*dist

		end do
	end do

  end subroutine number_concentration


!-----------------------------------------------------------------------
!
! Subroutine FRACSUM checks that the input data for mass fractions is
!	valid i.e. that the sum of mass fractions is
!	equal to one in each mode.
!
!-----------------------------------------------------------------------

  subroutine fracsum(mfractions)

	implicit none

	real*8, dimension(modes, compo), intent(in) :: mfractions
	integer :: ii, jj
	real*8:: total

	do ii = 1,modes

		total = sum(mfractions(ii,:))
	
		if (total < 0.9999 .or. total > 1.0001) then
			write(unit=6,fmt=10) ii
			10 format ("Sum of mass fractions doesn't equal one in mode",3I2)
			stop
		end if
	end do

  end subroutine fracsum

!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
!
!  Subroutines EQUILIBRIUM_SIZE, TERSIZE and BINSIZE equilibrate
!   the sulphuric acid portion of the particles to ambient water 
!	and ammonia vapours with the help of parameterisations to the Khler curves. 
!	The rest of the particle constituent compounds are assumed insoluble.
!
!
!	Determines values for global variables:
!		radius		=	radius of particle in each section (m)
!		mass		=	mass  -"- (kg)	
!
!------------------------------------------------------------------------------
!
!	13.11.2003 HK
!		Added organic component uptake to ternary case; irrespective of the
!		particle size, the growth rate for the organic portion is 1.1.
!		
!------------------------------------------------------------------------------
!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine equilibrium_size()

	implicit none

 

	if (nh3_mix > 0.1) then		! ternary case
		

	call tersize()
	else						! binary case

		call binsize()
	end if

  end subroutine equilibrium_size


!-----------------------------------------------------------------------
!	Parameterizations for wet radius, mole fractions and density
!	 in ternary case
!		(c) Ismo Napari 2003		
!	
!		Division of Atmospheric Sciences
!		Department of Physical Sciences
!		P.O. Box 64
!		FIN-00014 University of Helsinki
!		Finland
!
!		(ismo.napari@helsinki.fi)
!-----------------------------------------------------------------------

  subroutine tersize

	implicit none

	integer :: ii
	real*8 :: nacid, nwater, nammo, norg, vol_sa, vol_org, vol_tot
	real*8 :: wet_rad, r_dorg, r_worg, wet_mass
	real*8 :: lrw,lrh,ln1,ln3
   

	call tervalidity()

	do ii = 1,sections
   
  ! if ((ISNAN(vol_conc(ii,1)) .eq. .TRUE.) .or. (vol_conc(ii,1) .gt. 1.d200)) then
  !  print*, 'number of sulphuric acid molecules in particles out of bounds'
   !         vol_conc(ii,1)=1.d-12
   !         n_conc(ii)=n_limit
   !         endif   
           
   !         vol_sa = vol_conc(ii,1)/(n_conc(ii)*scale**3)    ! volume of sulphuric acid content in one particle (m^3)

    !        nacid = vol_sa/molecvol(1)                        ! number of acid molecules -"-               

	!	if (n_conc(ii) .eq. n_limit) then
	!	print*, vol_conc(ii,1), nacid
	
!	print*, n_conc(ii), ii, isnan(n_conc(ii))
	!print*, vol_conc(ii, 1:3)
!		endif
		! if the section not empty and the sulphuric acid & water sol. organic contents are not negligible, 
		! calculate total radius and mass
		if (n_conc(ii) < n_limit) then				
			radius(ii) = rdry(ii)
			mass(ii) = 4.*pi/3.*radius(ii)**3*1000.				! (kg)		
		else if (sum(vol_conc(ii,1:2)) < 0.01*sum(vol_conc(ii,:)) &
				.or. vol_conc(ii,1)/(n_conc(ii)*scale**3*molecvol(1)) < 1.) then				
			radius(ii) = rdry(ii)
			mass(ii) = sum(density(1:compo)*vol_conc(ii,1:compo)/n_conc(ii))/(scale**3) ! (kg)
		else
			vol_sa = vol_conc(ii,1)/(n_conc(ii)*scale**3)	! volume of sulphuric acid content in one particle (m^3)
			nacid = vol_sa/molecvol(1)						! number of acid molecules -"-				

			lrw = -1.22986 + 0.44570*rh - 1.213689*rh**2 + 0.78022*rh**3 + &
				0.0024136*log(nh3_mix) + 0.33194*log(nacid) + &
				0.024503*rh*log(nacid) + 0.050236*rh**2*log(nacid) + &
				0.00018042*log(nacid)**2 - 0.0017155*rh*log(nacid)**2

			wet_rad = exp(lrw)*1.d-9	! radius of the particle's h2so4-h2o-nh3 content (m)

			! for the water soluble organic species
			vol_org = vol_conc(ii,2)/(n_conc(ii)*scale**3)	! volume of water soluble organic content			
			norg = vol_org/molecvol(2)
			r_dorg = (3*vol_org/(4*pi))**(1./3.)		! dry radius of organic content (m)	
			r_worg = 1.1*r_dorg							! wet radius -"-

			vol_tot = 4.*pi/3.*wet_rad**3 +	4.*pi/3.*r_worg**3 +			&
					sum(vol_conc(ii,3:compo))/(n_conc(ii)*scale**3)	! total volume (m^3)
			radius(ii) = (3*vol_tot/(4*pi))**(1./3.)		! total radius (m)	

			! calculation of water and ammonia molecules in the particles because of equilibration
			!	with sulphate content
			ln1 = -7.92524 + 10.3438*rh - 20.2406*rh**2 + &
     			10.6466*rh**3 + 0.00044323*temp + 0.013745*rh*temp + 0.0053998*log(nh3_mix) + &
				2.03629*log(nacid) + 0.32495*rh*log(nacid) - &
				0.00061285*temp*log(nacid) - 0.056351*log(nacid)**2 - &
				0.0087760*rh*log(nacid)**2 + 0.0011431*log(nacid)**3

			ln3 = 0.33976 + 7.63687*rh - 0.87997*rh**2 + 0.099540*rh**3 - 0.0013829*temp - &
				0.025128*rh*temp - 0.66492*log(nh3_mix) + &
				0.18587*rh*log(nh3_mix) + 0.0024368*temp*log(nh3_mix) - &
				0.012224*log(nh3_mix)**2 + 1.12186*log(nacid) - &
				0.075124*rh*log(nacid) - 0.00023663*temp*log(nacid) + &
				0.0023043*log(nh3_mix)*log(nacid) - &
				0.0024878*log(nacid)**2 + &
				0.0026711*rh*log(nacid)**2 + 0.000022432*log(nacid)**3

			nwater = exp(ln1)			! number of water molecules in the particle
			nammo = exp(ln3)			! number of ammonia molecules -"-

			if(nwater < 0.) nwater=0.
			if(nammo < 0.) nammo=0.
 
			! mass of sulphuric acid-water-ammonia content and the whole particle (kg)
			wet_mass = nacid*molecmass(1) + nwater*molec_h2o + nammo*molec_nh3
			wet_mass = wet_mass + 4.*pi/3.*(r_worg**3 - r_dorg**3)*1000.
			mass(ii) = wet_mass + sum(density(2:compo)*vol_conc(ii,2:compo)/n_conc(ii))/(scale**3) ! (kg)

		end if
	end do

  end subroutine tersize



  subroutine tersize_old()

	implicit none

	integer :: ii
	real*8 :: vol_sa, vol_tot, nacid, nwater, nammo, wet_rad, wet_mass, xacid, xammo
	real*8 :: ln2, lrh, rd, lrw,xx, vol_org, norg, r_dorg, r_worg

		
	call tervalidity()		! temperature, rh and ammonia mixing ratio limits of the parameterizations

	do ii = 1,sections

		! if the section not empty and the sulphuric acid & water sol. organic contents are not negligible, 
		! calculate total radius and mass
		if (n_conc(ii) < n_limit) then				
			radius(ii) = rdry(ii)
			mass(ii) = 4.*pi/3.*radius(ii)**3*1000.				! (kg)		
		else if (sum(vol_conc(ii,1:2)) < 0.01*sum(vol_conc(ii,:)) &
				.or. vol_conc(ii,1)/(n_conc(ii)*scale**3*molecvol(1)) < 1.) then				
			radius(ii) = rdry(ii)
			mass(ii) = sum(density(1:compo)*vol_conc(ii,1:compo)/n_conc(ii))/(scale**3) ! (kg)
		else
			vol_sa = vol_conc(ii,1)/(n_conc(ii)*scale**3)	! volume of sulphuric acid content in one particle (m^3)
			nacid = vol_sa/molecvol(1)						! number of acid molecules -"-				

			ln2=log(nacid)
			lrh=log(rh)
			rd = (3*vol_sa/(4*pi))**(1./3.)*1.d9		! radius of sulphuric acid content (nm)	

			! parameterization for 'wet' radius (accuracy: +/- 10%)
			lrw = -1.13581 + 0.354429*ln2 - &
				  0.000447790*ln2*ln2 + 0.122801*lrh + &
				  0.00336133*ln2*lrh + 0.0305115*lrh*lrh
			wet_rad=exp(lrw)*1.d-9			! radius of particle containing h2so4-h2o-nh3 content (m)


			! for the water soluble organic species
			vol_org = vol_conc(ii,2)/(n_conc(ii)*scale**3)	! volume of water soluble organic content			
			norg = vol_org/molecvol(2)
			r_dorg = (3*vol_org/(4*pi))**(1./3.)		! dry radius of organic content (m)	
			r_worg = 1.1*r_dorg							! wet radius -"-

			vol_tot = 4.*pi/3.*wet_rad**3 +	4.*pi/3.*r_worg**3 +			&
					sum(vol_conc(ii,3:compo))/(n_conc(ii)*scale**3)	! total volume (m^3)
			radius(ii) = (3*vol_tot/(4*pi))**(1./3.)		! total radius (m)	

			! parameterizations for sulphuric acid and ammonia mole fractions
			xacid = 0.159189 - 0.00153754/(rd*rd) + &
					0.00944658/rd - 0.509025*rh - &
					0.0764054*rh/(rd*rd) + 0.413094*rh/rd + &
					0.00106949*temp
			xammo = 0.906012 - 0.00157851/(rd*rd) - &
					0.0127086/rd - 0.594510*rh - &
					0.0888652*rh/(rd*rd) + 0.485237*rh/rd - & 
					0.00114448*temp
			if (xacid + xammo > 0.9999) then
				xx=(xacid+xammo-0.9999)/2.
				xacid=xacid-xx
				xammo=xammo-xx
			end if

			nammo = nacid/xacid*xammo				! number of ammonia molecules in particle
			nwater = nacid/xacid*(1-xacid-xammo)	! number of water molecules in particles

			! mass of sulphuric acid-water-ammonia content and the whole particle (kg)
			wet_mass = nacid*molecmass(1) + nwater*molec_h2o + nammo*molec_nh3
			wet_mass = wet_mass + 4.*pi/3.*(r_worg**3 - r_dorg**3)*1000.
			mass(ii) = wet_mass + sum(density(2:compo)*vol_conc(ii,2:compo)/n_conc(ii))/(scale**3) ! (kg)

		end if				
	end do
  
  end subroutine tersize_old


!---------------------------------------------------------------------------
!	Parameterizations for wet radius and acid number concentration
!	 in binary case
!		(c) Hanna Vehkamki 2003	
!
!		Division of Atmospheric Sciences
!		Department of Physical Sciences
!		P.O. Box 64
!		FIN-00014 University of Helsinki
!		Finland
!
!		(hanna.vehkamaki@helsinki.fi)
!-----------------------------------------------------------------------

  subroutine binsize()

	implicit none

	integer :: ii
	real*8 :: vol_sa, vol_tot, nacid, nwater, wet_rad, wet_mass, rd, z 

	call binvalidity()		! temperature and rh limits of the parameterizations

	do ii = 1,sections

	   
   if ((ISNAN(vol_conc(ii,1)) .eq. .TRUE.) .or. (vol_conc(ii,1) .gt. 1.d200)) then
    print*, 'number of sulphuric acid molecules in particles out of bounds'
            vol_conc(ii,1)=1.d-12
            n_conc(ii)=n_limit
            endif   
           
            vol_sa = vol_conc(ii,1)/(n_conc(ii)*scale**3)    ! volume of sulphuric acid content in one particle (m^3)

            nacid = vol_sa/molecvol(1)                        ! number of acid molecules -"-               

			if (n_conc(ii) .eq. n_limit) then
			print*, vol_conc(ii,1), nacid
			
			endif

		! if the section not empty and the sulphuric acid content not negligible, 
		! calculate 'wet' radius and mass
		if (n_conc(ii) < n_limit) then				
			radius(ii) = rdry(ii)
			mass(ii) = 4.*pi/3.*radius(ii)**3*1000.				! (kg)		
		else if (vol_conc(ii,1) < 0.005*sum(vol_conc(ii,:))) then
			radius(ii) = rdry(ii)
			mass(ii) = sum(density(2:compo)*vol_conc(ii,2:compo)/n_conc(ii))/(scale**3) ! (kg)		
		else
			vol_sa = vol_conc(ii,1)/(n_conc(ii)*scale**3)	! volume of sulphuric acid content in one particle (m^3)
			nacid = vol_sa/molecvol(1)						! number of acid molecules -"-

			if (nacid < 0.1.or.nacid > 2e11) stop &			! validity of parameterizations
			
				'number of sulphuric acid molecules in particles out of bounds'
			
			! parameterization for particle 'wet' radius (accuracy: < +/- 10%)
			rd = (3*vol_sa/(4*pi))**(1./3.)*1.d9		! radius of sulphuric acid content (nm)	

			rd = log(rd)
			z = 0.3571062410312164 + log(rh)*0.1005557226829263 + rd*(1.072418249000919 - rd*0.007225150816512805)
			wet_rad = exp(z)*1.d-9		! radius of particle containing sulphuric acid - water content (m)

			vol_tot = 4.*pi/3.*wet_rad**3 + sum(vol_conc(ii,2:compo))/(n_conc(ii)*scale**3)	! total volume (m^3)
			radius(ii) = (3*vol_tot/(4*pi))**(1./3.)		! total radius (m)	

			! parameterization for number of water molecules in the particle
			! (accuracy: - 15%-+22%; for rh 0.20-0.90: -8%-+18%)
			nwater = exp( 0.7175349331751585 + 3.516681581495114*rh - 7.799949839852581*rh**2 &
					+ 4.399706826114728*rh**3 - 0.003003960532620574*temp &
					+ 0.003600567492582062*rh*temp + 1.168794974365219*log(nacid) + 0.07550157096542515*rh*log(nacid) &
					+ 0.1230059777461879*rh**2*log(nacid) - 0.00004695130053660258*temp*log(nacid) &
					- 0.01003371715122718*log(nacid)**2 - 0.005132252262920538*rh*log(nacid)**2 &
					+ 0.0002242756945069555*log(nacid)**3)

			! mass of sulphuric acid-water content and the whole particle (kg)	
			wet_mass = nacid*molecmass(1) + nwater*molec_h2o		  				
			mass(ii) = wet_mass + sum(density(2:compo)*vol_conc(ii,2:compo)/n_conc(ii))/(scale**3) ! (kg)

		end if
	end do 
 
  end subroutine binsize

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  subroutine binvalidity()

	if (temp < 190.) then
		print *,'Warning: temperature < 190 K, using 190 K'
		temp = 190.
	else if	(temp > 330.) then
		print *,'Warning: temperature > 330 K, using 330 K'
		temp = 330.
	end if		
	if (rh < 0.15) then
		print *,'Warning: rh < 15%, using 15%'
		rh = 0.15
	else if	(rh > 0.94) then
	!	print *,'Warning: rh > 94%, using 94%'
		rh = 0.94
	end if
		
  end subroutine binvalidity

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  subroutine tervalidity()

	if(temp < 240.0) then
	!	print *,'Warning: temperature < 240 K, using 240 K'
		temp=240.0
	else if (temp > 300.0) then
		print *,'Warning: temperature > 300 K, using 300 K'
		temp=300.0
	end if
	if(rh < 0.05) then
		print *,'Warning: relative humidity < 5 %, using 5%'
		rh=0.05
	else if (rh > 0.95) then
	!	print *,'Warning: relative humidity > 95 %, using 95%'
		rh=0.95
	end if
	if(nh3_mix < 0.1) then
		print *,'Warning: ammonia mixing ratio < 0.1 ppt, using 0.1 ppt'
		nh3_mix=0.1
	else if(nh3_mix > 100.0) then
		print *,'Warning: ammonia mixing ratio > 100 ppt, using 100 ppt'
		nh3_mix=100.0
	end if
		
  end subroutine tervalidity

end module distribution_dyn

