!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
!
! Contains subroutines and functions to evaluate the effect of coagulation,
!	condensation and nucleation on particle distribution
!
! UHMA - University of Helsinki Multicomponent Aerosol model
! (c) Hannele Korhonen 2003
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

module dynamic_lib

  use global_parameters

  implicit none

  private

  integer, parameter :: &
			hydmax = 5					

  ! hydrate stuff
  real*8 :: hydpw					
  real*8, dimension(hydmax) :: rhoh, kprod		

  public :: coagulation, condensation, collision_rate, pseubin, pseust, nanokohler, nucleation, act_nucleation, kin_nucleation, barless_nucleation, dry_deposition_aerosols

contains

!ooooooooooooooooooo  GENERAL STUFF  ooooooooooooooooooooooooooooooooooo

!-----------------------------------------------------------
! Calculate water and sulphuric acid saturation vapour 
!	pressures in ambient temperature (Pa)
!-----------------------------------------------------------

  function waterps()

	implicit none
	real*8 :: waterps

	waterps=exp(77.344913 - 7235.4247/temp - 8.2*log(temp) + 0.0057113*temp)

  end function waterps


  function acidps()

	implicit none
	real*8 :: lpar, acidps

	lpar= -11.695+log(pstand) ! Zeleznik
	acidps = 1/360.15-1.0/temp + 0.38/545.*(1.0+log(360.15/temp)-360.15/temp )
	acidps = 10156.0*acidps +lpar
	acidps = exp(acidps)
	
  end function acidps


!oooooooooooooooooooooooooooooo  COAGULATION	oooooooooooooooooooooooooooooooooooooooooooooooo
!
! Subroutine COAGULATION and function COAG_COEFF calculate the effect of
!	coagulation on particle population 
!
!	n_coag			=	particle concentration change due to coagulation in each section (#/m^3 s)
!	vol_coag		=	vol conc. change of each compound  - " - (um^3/(m^3 s))
!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine coagulation(n_coag, vol_coag)

	implicit none

	real*8, dimension(sections), intent(out) :: n_coag
	real*8, dimension(sections, compo), intent(out) :: vol_coag 

	integer :: ii, jj, kk,oo,pp
	real*8, dimension(compo) :: vol_new
	real*8 :: core_new, aa, bb, coagul, delta
		
	n_coag = 0.			! number concentration net change rate (#/m^3 s)
	vol_coag = 0.		! volume concentration net change rate (um^3/(m^3 s))

	! particles and volume added to each section by coagulation
	do jj = 1,sections	
		
		if (n_conc(jj) < n_limit) goto 10

		do ii = 1,jj		
			if (n_conc(ii) < n_limit) goto 20

			if (ii.eq.jj) then
				delta = 0.5		!self-coagulation (indistinguishability)
			else
				delta = 1.
			end if

			! volume of the core (m^3) and constituent volumes (um^3) of one
			! particle formed in collision 		
			core_new = core(ii) + core(jj)												
			vol_new = vol_conc(ii,:)/n_conc(ii) + vol_conc(jj,:)/n_conc(jj)		

			do kk = jj, sections-1
		

				! find the sections between which the new particle falls
				if (core_new >= core(kk) .and. core_new < core(kk+1)  ) then			
				pp=kk
				oo=kk+1

				if (n_conc(kk)==0) pp=kk-1
				if(n_conc(kk+1)==0) oo=kk+2
				if (oo>sections) oo=sections

					aa = (core(oo)-core_new)/(core(oo)-core(pp))	! number fraction moved to lower section
					bb = (core_new-core(pp))/(core(oo)-core(pp))		!			-"-			higher section

					coagul = delta*coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj)	! collision frequency (m^-3 s^-1)
					n_coag(kk) = n_coag(kk) + aa*coagul			! (m^-3 s^-1)
					n_coag(kk+1) = n_coag(kk+1) + bb*coagul
					vol_coag(kk,:) = vol_coag(kk,:) + aa*core(kk)/core_new*coagul*vol_new	! (um^3/(m^3 s))
					vol_coag(kk+1,:) = vol_coag(kk+1,:) + bb*core(kk+1)/core_new*coagul*vol_new

					goto 20

				end if
			end do

			! if resulting particle larger than the biggest size section, all particles go to biggest section
			! thus some volume flows out - choose the largest bin large enough!
			if (core_new >= core(sections)) then

				n_coag(sections) = n_coag(sections) + delta*coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj)* &
								core_new/core(sections)
				vol_coag(sections,:) = vol_coag(sections,:) + delta*coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj) &
										*vol_new
			end if

20			continue
		end do
10		continue
	end do

	! number and volume removed from each section by coagulation
	do ii = 1, sections
		if (n_conc(ii) < n_limit) goto 30
	
		do jj = 1, sections		
			if (n_conc(jj) < n_limit) goto 40	
!print*, coag_coeff(ii,jj), n_conc(ii)/1e6, n_conc(jj)/1e6
!pause
			coagul = coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj)		! collision frequency (m^-3 s^-1)
			n_coag(ii) = n_coag(ii) - coagul
			vol_coag(ii,:) = vol_coag(ii,:) - coagul*vol_conc(ii,:)/n_conc(ii)

40			continue
		end do
30		continue
	end do

  end subroutine coagulation

!---------------------------------------------------------------------------
!  
!  Calculates coagulation coefficients for Brownian coagulation (m^3/s)
!
!  Following XXX	
!  
!---------------------------------------------------------------------------

  function coag_coeff(ii,jj)

	implicit none

	integer, intent(in) :: ii, jj
	real*8 :: coag_coeff
	real*8 :: free_path, viscosity, vel_12, rad_12, dif_12
	real*8 :: prov1, prov2, dist, continuum, free_molec
	real*8, dimension(2) :: radii, m_part, knudsen, corr, diff, veloc, omega

	! air mean free path (m) and viscosity (kg/ms) (where from?)
	free_path = (6.73d-8*temp*(1.+110.4/temp))/(296.*pres/pstand*1.373)	
	viscosity = (1.832d-5*406.4*temp**1.5)/(5093*(temp+110.4))
				
	! for both sections	
	radii(1) = radius(ii)	! radii of colliding particles (m)
	radii(2) = radius(jj)
	m_part(1) = mass(ii)	! masses of colliding particles (kg)			
	m_part(2) = mass(jj) 

	knudsen = free_path/radii								! particle Knudsen number	
	corr = 1. + knudsen*(1.142+0.558*exp(-0.999/knudsen))	! Cunninghan correction factor (Allen and Raabe, Aerosol Sci. Tech. 4, 269)
	diff = boltzmann*temp*corr/(6*pi*viscosity*radii)		! particle diffusion coefficient (m^2/s)
	veloc = sqrt((8.*boltzmann*temp)/(pi*m_part))			! mean thermal velocity of a particle (m/s)

	omega = 8.*diff/(pi*veloc)								! ????? (m) 

	vel_12 = sqrt(veloc(1)**2 + veloc(2)**2)	! mean relative thermal velocity
	rad_12 = sum(radii)
	dif_12 = diff(1)+diff(2)					! relative diffusion coefficient 
	continuum = 4.*pi*rad_12*dif_12				! flux in continuum regime					
	free_molec = pi*vel_12*rad_12**2			! flux in free molecular regime

	! flux matching according to Fuchs (1964) (e.g. Seinfeld & Pandis p. 661)
	prov1 = (rad_12+omega(1))**3 - (rad_12**2 + omega(1)**2)**1.5 
	prov1 = prov1/(3.*rad_12*omega(1)) - rad_12
	prov2 = (rad_12+omega(2))**3 - (rad_12**2 + omega(2)**2)**1.5 
	prov2 = prov2/(3.*rad_12*omega(2)) - rad_12

	dist = sqrt(prov1**2 + prov2**2)	! distance at which fluxes are matched

	! coagulation coefficient between particles [m^3/s]
	coag_coeff = continuum / (rad_12/(rad_12+dist) + continuum/free_molec)

  end function coag_coeff


!---------------------------------------------------------------------------------------------
!
!  Calculates the effect of condensation of different vapours on 
!		particle population over a given timestep. The subroutine updates
!		the size distribution but NOT the gas concentrations.
!
!	timestep	=	timestep over which condensation is calculated (s)
!	sink		=	condensable vapours consumed by condensation (#/m^3 s)
!	
!---------------------------------------------------------------------------------------------

  subroutine condensation(timestep, sink)

	implicit none
	
	real*8, intent(in) :: timestep
	real*8, dimension(cond), intent(out) :: sink

	integer :: ii, jj, kk, io

	real*8 :: water_cont, s_org, aa, bb, vol_new2, vol_tol
	real*8, dimension(sections,compo) :: vol_change, vol_cond, vol_cond2
	real*8, dimension(sections) :: tot_change, collision, n_cond, vol_new, kelvin_eff, sat_conc, &
										sat_org, n_temp, n_cond2 

	collision = 0.	! collision rate of molecules (m^3/s)
	sink = 0.		! vapour sink (#/m^3 s)

	n_cond = 0.			! number concentration of particles that is moved (and/or stays) in the bin
	vol_cond = 0.		! volume concentration of each component	- " -

	vol_change = 0.

    do ii = 1,cond		! for each condensable vapour whose concentration is high enough
		if(vap_conc(ii) > vap_limit) then
			
			collision = collision_rate(ii)	! [m^3/s]
			
			call kelvin(ii, kelvin_eff)
			sat_conc = c_sat(ii)*kelvin_eff

			if (ii == 2) then

				do jj = 1,sections
					if (n_conc(jj) > n_limit) then
						call nanokohler(jj,water_cont,s_org, ii)
							sat_conc(jj) = c_sat(ii)*s_org	
								if (sat_conc(jj)<vap_conc(2)) then
						sat_conc(jj+1:sections)=0
					
						goto 13
						end if
					end if
				end do

			end if
	13 continue
			vol_change(:,ii) = collision*molecvol(ii)*(vap_conc(ii) - sat_conc)	! volume change rate of a particle (m^3/s)
			vol_change(:,ii) = vol_change(:,ii)*timestep			

			! not so elegant way to make sure that we don't divide by zero below 
			n_temp = n_conc
			where (n_temp <= 0.)
				n_temp = n_limit
			end where
			
			where (-vol_change(:,ii)*n_conc >= vol_conc(:,ii)/scale**3)		
				vol_change(:,ii) = -vol_conc(:,ii)/(n_temp*scale**3)	! not more than exists in bin evaporates
			end where

			sink(ii) = sum(n_conc*vol_change(:,ii))/(molecvol(ii)*timestep)	! (#/ m^3 s)

		end if
	end do





!irreversible condensation
!	where (vol_change<0)
!	vol_change=0
!	end where


	tot_change = sum(vol_change,2)		! total volume change in each bin (m^3)
	vol_new = core + tot_change			! the volume the particles in each bin grow to (m^3)
!dist_approach='mc'
	! choose the size distribution approach used for solving condensation
	




		! find the section that particles have grown/shrank to	
		do ii = 1,sections
			if (n_conc(ii) >= n_limit) then
			
				if (vol_new(ii) >= core(ii)) then !condensation, or evaporation within bin					
					do jj = ii,sections-1
						if (core(jj) <= vol_new(ii) .and. vol_new(ii) < core(jj+1)) then
							n_cond(jj) = n_cond(jj) + n_conc(ii)

							vol_cond(jj,:) = vol_cond(jj,:) + vol_conc(ii,:)	! condensation/evap. not taken into account yet				
							
					
							vol_cond(jj,:) = vol_cond(jj,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
							goto 200
						end if
					end do
				else	! evaporation to a lower bin
					do jj = ii-1,1,-1	! starting from bin next to ii
						if (core(jj) <= vol_new(ii) .and. vol_new(ii) < core(jj+1)) then
							n_cond(jj) = n_cond(jj) + n_conc(ii)
							vol_cond(jj,:) = vol_cond(jj,:) + vol_conc(ii,:)	! condensation/evap. not taken into account yet				
							vol_cond(jj,:) = vol_cond(jj,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
														
							goto 200
						end if
					end do
				end if
			end if


			if (vol_new(ii) >= core(sections)) then	
				n_cond(sections) = n_cond(sections) + n_conc(ii)				! (#/m^3)
				vol_cond(sections,:) = vol_cond(sections,:) + vol_conc(ii,:)	! no cond./evap taken into account yet
				vol_cond(sections,:) = vol_cond(sections,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
				goto 200
			end if	
	
			! particles that have evaporated/grown out of simulated size range
			! treated normally here and checked again when particles allocated to new bins
			if (vol_new(ii) < core(1)) then		
				n_cond(1) = n_cond(1) + n_conc(ii)					! (#/m^3)
				vol_cond(1,:) = vol_cond(1,:) + vol_conc(ii,:)		! no cond./evap taken into account yet
				vol_cond(1,:) = vol_cond(1,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
			end if 
			
200			continue

		end do

		where (vol_cond < 0.)
			vol_cond = 0.
		end where

		n_cond2 = 0.
		vol_cond2 = 0.

		! particles allocated to their new bins
		do ii = 2,sections-1
				
			if (n_cond(ii) > n_limit) then

				vol_new2 = sum(vol_cond(ii,:))/(n_cond(ii)*scale**3)	! (m^3)
				aa = (core(ii+1)-vol_new2)/(core(ii+1)-core(ii))
				bb = 1.-aa			

				n_cond2(ii) = n_cond2(ii)+aa*n_cond(ii)
				n_cond2(ii+1) = n_cond2(ii+1) + bb*n_cond(ii)					
				! effect of condensation/evaporation
				vol_cond2(ii,:) = vol_cond2(ii,:) + aa*core(ii)/vol_new2*vol_cond(ii,:) ! (um^3/m^3)					
				vol_cond2(ii+1,:) = vol_cond2(ii+1,:) + bb*core(ii+1)/vol_new2*vol_cond(ii,:)   ! (um^3/m^3)					

			end if
		end do

	


		if (n_cond(1) > n_limit) then
			
			vol_new2 = sum(vol_cond(1,:))/(n_cond(1)*scale**3)	! (m^3)
			if (vol_new2 >= core(1)) then
				aa = (core(2)-vol_new2)/(core(2)-core(1))
				bb = 1.-aa			

				n_cond2(1) = n_cond2(1)+aa*n_cond(1)
				n_cond2(2) = n_cond2(2) + bb*n_cond(1)					
				! effect of condensation/evaporation
				vol_cond2(1,:) = vol_cond2(1,:) + aa*core(1)/vol_new2*vol_cond(1,:) ! (um^3/m^3)					
				vol_cond2(2,:) = vol_cond2(2,:) + bb*core(2)/vol_new2*vol_cond(1,:)   ! (um^3/m^3)					
			else
				n_cond2(1) = n_cond2(1) + vol_new2*n_cond(1)/core(1)
				vol_cond2(1,:) = vol_cond2(1,:) + vol_cond(1,:)
			end if
		end if



		if (n_cond(sections) > n_limit) then

			vol_new2 = sum(vol_cond(sections,:))/(n_cond(sections)*scale**3)	! (m^3)
			if (vol_new2 >= core(sections)) then						
				n_cond2(sections) = n_cond2(sections) + vol_new2*n_cond(sections)/core(sections)
				vol_cond2(sections,:) = vol_cond2(sections,:) + vol_cond(sections,:)
			else
				write(*,*) 'Something wrong here!'
			end if
		end if



	n_conc = n_cond2
		vol_conc = vol_cond2


  end subroutine condensation

!------------------------------------------------

  subroutine kelvin(ii, kelvinEffect)

	implicit none

	integer, intent(in) :: ii
	real*8, dimension(sections), intent(out) :: kelvinEffect
	real*8, dimension(sections) :: exponento
	
	exponento=2.*surf_ten(ii)*molecvol(ii)
	exponento=exponento/(boltzmann*temp*radius)
	kelvinEffect=dexp(exponento)
	
	print*, dexp(exponento(20)) ,exp(exponento(20))
	pause

  end subroutine kelvin


!----------------------------------------------------------------------------
!----------------------------------------------------------------------------

  subroutine nanokohler(ii,water_cont, sat_org, kk)

	implicit none

	integer, intent(in) :: ii, kk
	real*8, intent(out) :: water_cont, sat_org

	integer :: jj
	real*8 :: vol_org, vol_as, n_org, n_as, surften_org, dens_as, dens_h2o, r
	real*8 :: aa, cc, dd, mass_as, mass_h2o, vol_h2o, dens_org, nu_org, nu_as
	real*8 :: nwmin, nwmax, n_h2o, nwold, mass_org, rad, x2, fx, xo, surt, pjw
	real*8 :: gamw, gamo, gapb, gambi, xbw, f, xbo, rw, v_as, v_org

	v_org = vol_conc(ii,kk)/(n_conc(ii)*scale**3)	! volume of water soluble organic content			
	n_org = v_org/molecvol(kk)						! number of organic molecules in particle

	v_as = vol_conc(ii,1)/(n_conc(ii)*scale**3)	! volume of sulphate content in one particle (m^3)
	n_as = v_as/molecvol(1)						! number of sulphate molecules in particle		

	if (n_org <= 0.) n_org = 0.05*n_as

    if (n_as <= 0.) then
		write(*,*) 'number of sulphate molecules is smaller than zero'
		write(*,*) ii, vol_conc(ii,:), n_conc(ii), rdry(ii)
		Write(*,*) 'adding one molecule sulfuric acid'
		n_as=1.
		v_as=molecvol(1)*n_as
		vol_conc(ii, 1)=v_as*(n_conc(ii)*scale**3)
		
	end if

! fitted thermodynamic parameters
    aa = -190.
    cc = 0.3
    dd = 0.8

    surften_org = 0.03		! (N/m)

    dens_as = 1780.			! density of ammonium bisulphate (kg/m^3)
    dens_h2o = 1000.		! -"- of water (kg/m^3)
    mass_as = 115.11		! molar mass of ammonium bisulphate (g/mol)
    mass_h2o = 18.02		! -"- water

	mass_org = 186.
	dens_org = 1107.

    vol_as = mass_as*1.d-3/(avogadro*dens_as)
    vol_h2o = mass_h2o*1.d-3/(avogadro*dens_h2o)
	vol_org= mass_org*1.d-3/(avogadro*dens_org)	! remember this: v_org = 2.25d-28 m^3         

	nu_org= 1.				! dissociation factor
    nu_as = 2.

! r= dry radius of a particle:
    r =  (3./pi/4.*(n_as*vol_as + n_org*vol_org))**(1./3.)

    nwmin = 0.
    nwmax= (4.*(pi/3.)*((2.5*r)**3.))/vol_h2o

    n_h2o = 0.
    do jj=1,20           
		nwold= n_h2o
        n_h2o = (nwmin+nwmax)/2.
        rad= ((3./4.)*(n_as*vol_as+n_h2o*vol_h2o+n_org*vol_org)/pi)**(1./3.)
        call xbulk(n_h2o,n_org,n_as,rad,x2,fx)
        xo = x2
        if (xo .eq. 0. ) then
			xo= (1.d-10)*n_as/(n_as+n_h2o)           
        endif
        call pseust(xo,n_h2o,n_as,surt,pjw)
        call pseubin(xo,n_h2o,n_as,gamw,gamo,gapb,gambi,n_org) 
        xbw = 1./(1./(1.-xo) +2.*n_as/n_h2o)
! r instead of rad in original:
        f = rh/(dexp(2.*surt*vol_h2o/boltzmann/temp/rad)*xbw*gamw)-1.
 
        if(f.lt.0.) then
			nwmax = n_h2o
        else
            nwmin = n_h2o
        endif
	end do          
    xbo = xo*xbw/(1.-xo)
! r instead of rad in original:
    sat_org = gamo*xbo*dexp(2.*surt*vol_org/(boltzmann*temp*rad))
  
    rw = rad        
    water_cont= (n_h2o/avogadro)*mass_h2o     
	
  end subroutine nanokohler


  subroutine xbulk(n1,n2,n3,rad,x2,fb)

	implicit none

	real*8, intent(in) :: n1, n2, n3, rad
	real*8, intent(out) :: x2, fb

	integer :: i
	real*8 :: xl, xu, x, xy, xa
	real*8 :: g1y, g2y, gpby, gabi, g1a, g2a, gpba
	real*8 :: surty, surta, pj, dsurt, dmu1, dmu2

	xl = 0.
    xu = 1.
    if (n2 .eq. 0.) then
		x= 0.
        goto 11
    endif
    do i=1,40
		x = (xl+xu)/2.
        xy = x+.00001
        xa = x-.00001         
        if (xa .lt. 0.) xa= 1.d-9
        if (xy .gt. 1.) xy= 1.        
        call pseubin(xy,n1,n3,g1y,g2y,gpby,gabi,n2)  
        call pseubin(xa,n1,n3,g1a,g2a,gpba,gabi,n2)       
        call pseust(xy,n1,n3,surty,pj)
        call pseust(xa,n1,n3,surta,pj)         
        dsurt = (surty-surta)/(xy-xa)       
        dmu1 = boltzmann*temp*(dlog((1.-xy)*gpby)-dlog((1.-xa)*gpba)) &
              /(xy-xa)
        dmu2 = boltzmann*temp*(dlog(xy*g2y)-dlog(xa*g2a))/(xy-xa)
        fb = n1*dmu1 + n2*dmu2 + 4.*pi*rad**2*dsurt
        if(fb.gt.0.) then
			xl = x
        else
			xu = x
        endif
	end do        
11	x2 = x

  end subroutine


! pseudobinary surface tension for a system with n1 water, n2 organic 
! and n3 ammonium sulfate molecules
  subroutine pseust(xbu,n1,n3,stb,stwa)

	implicit none

	real*8, intent(in) :: xbu, n1, n3
	real*8, intent(out) :: stb, stwa 

	real*8 :: m3, m1, st1, xb1, w, xbw, xbo, xbws, surften_org, cc, dd


	cc = 0.3 
	dd = 0.8
	surften_org = 0.02	!(N/m)
    m3 = 115.11
    m1 = 18.02
    st1 = (93.6635+.009133*temp-.000275*temp**2)/1000.
    stwa=st1

    xb1 = 1.-xbu
    w = n3*m3/(n1*m1+n3*m3)

! water-ammonium bisulfate surface tension

    st1 = st1 + 0.01467*w

! water-bisulfate-organic surface tension model

    xbw = 1./(1./(1.-xbu) +2.*n3/n1)
    xbo = xbu*xbw/(1.-xbu)
    xbws = 1.-xbo
  
    stb = st1-(st1-surften_org)*(1.+cc*xb1/(1.-dd*xb1))*xbu

  end subroutine


! pseudobinary activity coefficients for a system with n1 water, n2 organic 
! and n3 ammonium sulfate molecules
  subroutine pseubin(xb2,n1,n3,gam1,gam2,vl1,acs,n2)

	implicit none

	real*8, intent(in) :: xb2, n1, n3, n2
	real*8, intent(out) :: gam1, gam2, vl1, acs

	real*8 :: rho1, r, m3, rho3, m1, amu, v11, v3, v1, bvl
	real*8 :: xb1, xs1, w, vl2, c1, c2, c3, wf, act, aa


	aa = -190.
	rho1 = 1000.0D0
    r = 8.3143D0
    m3 = 115.11
    rho3 = 1780.
    m1 = 18.02
    amu = 1.67d-27
    v11 = m1*amu/rho1
    v3 = m3*amu/rho3  
    v1 = v11+n3/n1*v3
    bvl= 100.

    xb1=1.-xb2

    xs1 = n1/(n1+2.*n3)
    w = n3*m3/(n1*m1+n3*m3)

! Van Laar coefficients

    vl1 = aa/(1.+xb1/(xb2*bvl))**2/r/temp
    vl1 = dexp(vl1)     
    vl2 = aa*bvl/(1.+bvl*xb2/xb1)**2/r/temp
    vl2 = dexp(vl2)

! water activity coefficient in amm. bisulf. solution

    c1 = -3.05d-3
    c2 = -2.94d-5
    c3 = -4.43d-7
    wf=w*100.
    act = 1+c1*wf+c2*wf**2+c3*wf**3
    acs = act/xs1        

    gam1 = acs*vl1
    gam2 = vl2

  end subroutine


!----------------------------------------------------------------------------
!----------------------------------------------------------------------------
! 
!  Calculates the collision rate (m^3/s) of condensing 
!	vapour molecules with particles in each size section
!	
!	(for the 'unconventional' way to describe free-molecular condensation
!	 see Lehtinen & Kulmala: A model for particle formation and growth
!			in the atmosphere with molecular resolution in size;
!			Atmos. Chem. Phys. Discuss., 2, 1-17, 2002)
!				
!---------------------------------------------------------------------------

  function collision_rate(jj)	

	implicit none

	integer, intent(in) :: jj							! the 'number' of condensing vapour
	real*8, dimension(sections) :: collision_rate

	integer :: ii
	real*8 :: air_free_path, dif_vap, r_vap, n1, viscosity, r_hyd, r_h2o
	real*8, dimension(sections) :: knudsen, corr, dif_part, mean_path, rate 

	! air mean free path (m) and viscosity (kg s^-1 m^-1) (where from?)
	air_free_path = (6.73d-8*temp*(1.+110.4/temp))/(296.*pres/pstand*1.373)	
	viscosity = (1.832d-5*406.4*temp**1.5)/(5093*(temp+110.4))

	r_h2o = (3*molec_h2o/1000./(4*pi))**(1./3.)	! radius of water molecule
	r_vap = (molecvol(jj)*3./(4.*pi))**(1./3.)	! radius of condensable molecules (m)

	! for particles in each size section
	knudsen = air_free_path/radius							! particle Knudsen number
	corr = 1. + knudsen*(1.142+0.558*exp(-0.999/knudsen))	! Cunninghan correction factor (Allen and Raabe, Aerosol Sci. Tech. 4, 269)



	! molecular diffusion coefficient (m^2/s) for condensing vapour and
	! mean free path of condensation process (m)
	dif_vap = diffusion(molarmass(jj), diff_vol(jj), zero)
	mean_path = 3*(dif_vap + dif_part)/sqrt(8*boltzmann*temp/pi*(1./molecmass(jj) + 1./mass))

	knudsen = mean_path/(radius + r_vap)
	corr = knudsen + 1.			! transitional correction factor (Fuchs and Sutugin, 1971)
	corr = corr/(0.377*knudsen + 1 + 4./(3.*alpha(jj))*(knudsen+knudsen**2))

!condensation_sink = 2.*pi*dif_vap*(corr*2.*radius*n_conc)
!condensation_sink_integral = sum(condensation_sink)

	! no hydrates
	if (jj .ne. 1) then
		collision_rate = 4*pi*(radius + r_vap)*(dif_vap + dif_part)*corr
		print*, collision_rate(20), rdry(20)
		pause		! (m^3/s)
		goto 100
	end if

	! only for h2so4 (if hydrates taken into account) 
	call hydrates()

	collision_rate = (radius+r_vap)*(dif_vap/hydpw + dif_part)*corr	! (m^3/s)

	! hydrate correction
	do ii = 1, hydmax
		n1 = real(ii)
		dif_vap = diffusion(molarmass(jj), diff_vol(jj), n1)
		mean_path = 3.*(dif_vap+dif_part)
		mean_path = mean_path/sqrt((8*boltzmann*temp)/pi*(1./(molecmass(jj)+n1*molec_h2o) + 1./mass))
		r_hyd = (r_vap**3+n1*r_h2o**3)**(1./3.)
		knudsen = mean_path/(radius + r_hyd)
		corr = knudsen + 1.	
		corr = corr/(0.377*knudsen + 1 + 4./(3.*alpha(jj))*(knudsen+knudsen**2))
		collision_rate = collision_rate + corr*(dif_vap*rhoh(ii) + dif_part)*(radius+r_hyd) 
	end do

	collision_rate = 4.*pi*collision_rate !(m^3/s)

100 continue 

  end function collision_rate

!-----------------------------------------------------------------
! 
!  Evaluates molecular diffusion coefficient (m^2/s) 
!	for condensing vapour (see Reid et al. Properties of 
!		Gases and Liquids (1987))
!
!-----------------------------------------------------------------

  function diffusion(molar_gas, d_gas, hydrates)

	implicit none

	real*8, intent(in) :: molar_gas, d_gas, hydrates
	real*8 :: d_air, d_h2o, diffusion

	d_air = 19.70	! diffusion volume of air (???)
	d_h2o = 13.10	!		- " -		water

	diffusion = 1.d-7*temp**1.75*sqrt(1./molar_air + 1./(molar_gas+hydrates*molar_h2o))
	diffusion = diffusion/( (pres/pstand)*(d_air**(1./3.)+(d_gas+hydrates*d_h2o)**(1./3.))**2 )

  end function diffusion

!--------------------------------------------------------
!  Evaluates hydrate equilibrium coefficients and  
!    hydrate fractions in gas phase for sulphuric acid 						  
!
!	(c) Hanna Vehkamki 2002	
!
!		Division of Atmospheric Sciences
!		Department of Physical Sciences
!		P.O. Box 64
!		FIN-00014 University of Helsinki
!		Finland
!
!		(hanna.vehkamaki@helsinki.fi)
!--------------------------------------------------------

  subroutine hydrates()

	implicit none

	integer :: ii, jj
	real*8, dimension(hydmax) :: kh
	real*8 :: pres_h2o, VOL_TOL

	pres_h2o = rh*waterps()

	! Zeleznik equilibrium constant for reactions
	kh(1)= exp(6159.1796/temp-14.3343)	! water+h2so4 =1-hydrate
	kh(2)= exp(5813.8776/temp-15.5140)	! i-hydrate+water= i+1-hydrate
	kh(3)= exp(5246.0/temp-14.90)		! i-hydrate+water= i+1-hydrate
	kh(4)= exp(4934.0/temp-14.47)		! i-hydrate+water= i+1-hydrate
	kh(5)= exp(4763.0/temp-14.21)		! i-hydrate+water= i+1-hydrate

	do ii=1,hydmax
		kprod(ii)=1. !product of the equilibrium constants k1*k2*k3*..*ki1
		do jj = 1,ii
			kprod(ii)= kh(jj)*kprod(ii)
		end do
	end do

	! Jaecker-Voirol et al., J. Chem. Phys., vol. 87, 4849-4852 (1987):
	! hydpw = pa(total)/pa(free) = 1+k1(pw/p0)+..+k1*k2*..*ki1(pw/p0)^ii+..+
	hydpw = 1.0
	do ii = 1,hydmax
		hydpw = hydpw + kprod(ii)*(pres_h2o/pstand)**ii
	end do 

	!concentration of the hydrates (divided by total concentration)
	!rho(ii)/rho(tot)= k1*k2*..*ki1(pw/p0)^ii/hydpw
	do ii = 1,hydmax
		rhoh(ii) = kprod(ii)*(pres_h2o/pstand)**ii/hydpw 
	end do

  end subroutine hydrates

subroutine nucleation(n_coeff)

	implicit none

	real*8, intent(in) :: n_coeff

	
	integer :: ii
	real*8 :: r_crit, n_h2o, n_nh3, core_nuc, aa


	n_nuc = 0.		! number concentration net change rate (#/m^3 s)
	vol_nuc = 0.		! volume concentration net change rate (um^3/m^3 s)

	select case(nuc_approach)
	
	case('Act')  ! activation nucleation

		if (n_coeff < 1.0e-8 .or. n_coeff > 1.0e-4) &
			stop 'The value of activation nucleation coefficient not in correct range'

		call act_nucleation(n_coeff)

	case('Kin')  ! kinetic nucleation with a determined coefficient
		
		if (vap_conc(1) > nh3_mix*2.4d13) &
			stop ' [NH3] < [H2SO4]: kinetic nucleation assumptions not valid'
		if (n_coeff < 1.0e-14 .or. n_coeff > 1.0e-10) &
			stop 'The value of kinetic nucleation coefficient not in correct range'
		
		call kin_nucleation(n_coeff)

	case('Bar')  ! barrierless kinetic nucleation
	
	!	if (vap_conc(1) > nh3_mix*2.4d13) &
		!	stop ' [NH3] < [H2SO4]: kinetic nucleation assumptions not valid'

		call barless_nucleation()

	
	end select
	
	core_nuc = n_sa*molecvol(1)	! volume of sulphuric acid core of nucleated particles

	! find the bins to which the new particle is divided
	if (core_nuc <= core(1)) then
		n_nuc(1) = nuc_rate*core_nuc/core(1)		! here we conserve number, volume overestimated
		vol_nuc(1,1) = core_nuc*nuc_rate
	else
		do ii = 1,sections-1
			if (core_nuc > core(ii) .and. core_nuc <= core(ii+1)) then
				aa = (core(ii+1)-core_nuc)/(core(ii+1)-core(ii))
				n_nuc(ii) = aa*nuc_rate
				n_nuc(ii+1) = (1-aa)*nuc_rate
				vol_nuc(ii,1) = aa*core(ii)*nuc_rate
				vol_nuc(ii+1,1) = (1-aa)*core(ii+1)*nuc_rate
				goto 100
			end if
		end do
	end if

	if (core_nuc > core(sections)) then
		n_nuc(sections) = nuc_rate*core_nuc/core(sections)	! here we conserve number, volume underestimated
		vol_nuc(sections,1) = nuc_rate*core_nuc
	end if

100	continue

	vol_nuc = vol_nuc*scale**3	! (um^3/m^3)

  end subroutine nucleation

  
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
! Subroutine ACT_NUCLEATION evaluates the "activation" nucleation rate
! for sulphuric acid clusters using activation coefficient given as input parameter.
!
!        act_coeff  =   activation coefficient (1/s), given as an input parameter
!        nuc_rate   =   nucleation rate (#/m^3 s)
!        n_sa       =   number of sulphuric acid molecules in a critical cluster
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine act_nucleation(act_coeff)

        implicit none

	real*8, intent(in) :: act_coeff
 

        nuc_rate = act_coeff*vap_conc(1)	! [#/m^3 s]
        n_sa = 1.0

  end subroutine act_nucleation
  
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
! Subroutine KIN_NUCLEATION evaluates the "kinetic type" nucleation rate
! for ammonium bisulphate (NH4HSO4) molecules using kinetic coefficient given as input parameter.
!
!        kin_coeff  =   kinetic collision frequency (cm^3/s), given as an input parameter
!        nuc_rate   =   nucleation rate (#/m^3 s)
!        n_sa       =   number of sulphuric acid molecules in a critical cluster
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine kin_nucleation(kin_coeff)

        implicit none

	real*8, intent(in) :: kin_coeff
     

        nuc_rate = kin_coeff*1e-6*(vap_conc(1))**2	! [#/m^3 s]
        n_sa = 2.0

  end subroutine kin_nucleation
  
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
! Subroutine BARLESS_NUCLEATION evaluates the barrierless kinetic nucleation rate
! for ammonium bisulphate molecules (NH4HSO4 + NH4HSO4 -> 2 NH4HSO4).
!
!        nuc_rate   =   nucleation rate (#/m^3 s)
!        n_sa       =   number of sulphuric acid molecules in a critical cluster
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine barless_nucleation()

        implicit none
	
  

        real*8 :: r_init, dens_init, coag_coeff
        
	r_init = 0.315d-9 ! radius (m) of the initial NH4HSO4 molecule
        dens_init = 1465. ! density (kg/m^3) of the initial NH4HSO4 molecule

	! collision frequency from kinetic theory
        coag_coeff = 8.*sqrt(3.*boltzmann*temp*r_init/dens_init)

        nuc_rate = 0.5*coag_coeff*(vap_conc(1))**2	! [#/m^3 s]
        n_sa = 2.0

  end subroutine barless_nucleation
  
  subroutine  dry_deposition_aerosols()

    implicit none
    integer, parameter :: LUC = 17
	integer :: ii,nLUCchosen, kk

	real*8 :: dry_dep_1



	
	real*8 :: nLUC(LUC)=0.0, E_Constant_Alpha(LUC)=0.0, E_Constant_Beta(LUC)=0.0, Z0m(LUC)=0.0, LAI(LUC)=0.0, Rac0(LUC)=0.0, Charac_Radius(LUC)=0.0 
	real*8 :: Vd(LUC)=0.0
  real*8 :: Hminusd(LUC)=0.0, Zminusd(LUC)=0.0, Ustar_Zhang2003_DAY(LUC)=0.0,Ustar_Zhang2003_NIGHT(LUC)=0.0
	real*8 :: z02, g, kinematic_air_viscosity, charnock_constant, d, Vg, Vgh, Ab, Kn, Gcunn, Bd, Sc, Vis, Eb, Eim, k, Rb_water, Rb1, Rb2
	real*8 :: air_mean_path, air_viscosity, air_density,  Uref, diameter
	real*8 :: St_water, Eb_Water, Eim_Water, Eb_Veg, small_collector_width, large_collector_width
	real*8 :: a_constant, b_constant, fitting_constant1, fitting_constant2, e0, St_Veg, Eim_Veg, F, Ein, roff, E, Uh, Rs
	real*8 :: Eb_Nonveg, Rb_Nonveg, Eim_Urban, Eim_Nonveg, Rb_Urban, St_Nonveg
	real*8 :: set_vel,rb_ex, rb_new, diff_part, schmidt, dens, radie_98, diameter_98, S, A, B, Bdh

	real*8 :: wind_velocity_grad, L_energy, L_momentum, Beta_energy, Beta_momentum
	real*8 :: Gamma_energy, Gamma_momentum, Gamma_momentum_function_Zr, Gamma_energy_function_Zr
	real*8 ::	Gm=0.0, Gh=0.0, Ustar_Bulk=0.0, Thetastar_Bulk=0.0
	real*8 :: Pr, Prt, sigma_h_integral, Rs_lowveg
	real*8 :: Z0h=0.0, Gamma_momentum_function_Z0m=0.0, sigma_m_integral=0.0, Gamma_energy_function_Z0h=0.0, sigma_e_integral=0.0
	real*8 :: Gamma_momentum_function_Hminusd, sigma_m_integral_Hminusd,radie,test, temp_75m, press_75


	real*8 :: uv_r_grad, theta_grad, uv_r, u10m_r, RI_bulk, Theta_diff, pot_temp_intep_surf, u_comp_intep_75, v_comp_intep_75, L_bulk, &
					theta_av, sigma_m_integral_bulk, downward_sw, temp_2m, percent_total_cc, accum_precip, rainfall_rate, surface_pressure, temp850, &
					sigma_e_integral_bulk

	integer :: i=0



	open(unit=15,file='Dep-Gas_CanopyHeight.dat')
	do i = 1,LUC
	read(15,*) Hminusd(i), Zminusd(i)
	enddo
	close(15)

	open(unit=16,file='Dep_Aerosol_Summer.dat')
	open(unit=17,file='Dep_Aerosol_Autumn.dat')
	open(unit=18,file='Dep_Aerosol_Winter.dat')
	open(unit=19,file='Dep_Aerosol_Spring.dat')
month=8
	do i = 1,LUC
	if ((month.eq.6) .or. (month.eq.7) .or. (month.eq.8))   then
	read(16,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
	elseif ((month.eq.9) .or. (month.eq.10) .or. (month.eq.11))   then
	read(17,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
    elseif ((month.eq.12) .or. (month.eq.1) .or. (month.eq.2))   then
	read(18,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
    elseif ((month.eq.3) .or. (month.eq.4) .or. (month.eq.5))   then
	read(19,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
	endif
	enddo
	close(16)
	close(17)
	close(18)
	close(19)

!FOR BATCH FILE

	open(unit=20,file='Ustar_Zhang2003.dat')
	do i = 1,LUC
	read(20,*) Ustar_Zhang2003_DAY(i), Ustar_Zhang2003_NIGHT(i)
	enddo
	close(20)

!FIX replace by cluster_dom_class
!nLUCchosen=dom_class
nLUCchosen=dom_class

!************************TEST***************************
!	if (nLUCchosen.gt.1) then
!	dom_class=8
!	endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!   dom_class=2
!cluster_dom_class=1
!nLUCchosen=dom_class

	n_depo=0
	vol_depo=0
	
	
	do kk = 1,sections

if (n_conc(kk)<n_limit) goto 3
radie=radius(kk)
dens=mass(kk)/(4.*pi*radie**3./3.)
if (kk==35) then


end if

n_depo(kk) = -dry_dep_1/boundary*n_conc(kk)	![#/m^3 s]
vol_depo(kk,:) = -dry_dep_1/boundary*vol_conc(kk,:)	![um^3/m^3 s]
!depo_vel= -n_depo(kk)*boundary/n_conc(kk)

g=9.81

!Von Karman constant (assume value)
k = 0.4
!Prandtl number
Pr=0.72
!Turbulent Prandtl number
Prt = 0.97
charnock_constant = 0.016

wind_velocity_grad = UV_R_grad
potential_temp_grad= Theta_grad
Uref = abs(UV_R)
U10m_r=5.

!Uref needs to be at the same height as reference height

!FIX have at 75m using adiabatic lapse rate
temp_2m=temp+altitude*0.0065
press_75=pres
temp_75m=temp_2m+(6.7e-3*75)
	air_viscosity = (1.832d-5*406.4*temp_75m**1.5)/(5093*(temp_75m+110.4))
	air_density = 1.2929*273.15*Press_75/(temp_75m*pstand)	    
    kinematic_air_viscosity = air_viscosity/air_density
	air_mean_path = (6.73d-8*temp_75m*(1.+110.4/temp_75m))/(296.*Press_75/pstand*1.373)	
!***********************************************************************************	
!Z0m over water, follow Zhangs code
if (nLUCchosen.eq.1) then
Z0m(nLUCchosen) = 0.000002*(U10m_R**2.5)
endif


!Z0m take from look up table (Zhang et al, 2002)
!Page 531 Jacobson, use IF statements for different LUC
Z0h = Z0m(nLUCchosen)/7.3891  !(exp(2))

!Water
if (nLUCchosen.eq.1) then
Z0h = Z0m(nLUCchosen)/(exp(k*((13.6*(Pr**0.66667))-12)))
endif


!Non-iterative aerodynamic resistance- Jacobson

Theta_diff=-3
pot_temp_INTEP_Surf=288
u_comp_INTEP_75=10
v_comp_INTEP_75=5
UV_R=wind_speed
Uref=UV_R
Theta_Av=286.5
UV_R_grad=0.5
Theta_grad=0.2
wind_velocity_grad = 0.5
potential_temp_grad= 0.2

!Bulk Ri 
!Potential virtual temperature? Should be very close to potential temp over Sweden
!Assume Z0h close to the surface

Ri_Bulk= (g*(Theta_diff)*((Zminusd(nLUCchosen)-Z0m(nLUCchosen))**2))/((pot_temp_INTEP_Surf)*(u_comp_INTEP_75**2+v_comp_INTEP_75**2)*(Zminusd(nLUCchosen)-Z0h))

IF (Ri_Bulk .le. 0.0) then

Gm = 1 - (9.4*Ri_Bulk/(1+ (70*(k**2)*((abs(Ri_Bulk)*Zminusd(nLUCchosen)/Z0m(nLUCchosen))**0.5)/(Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))**2))))

Gh = 1 - (9.4*Ri_Bulk/(1+ (50*(k**2)*((abs(Ri_Bulk)*Zminusd(nLUCchosen)/Z0m(nLUCchosen))**0.5)/(Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))**2))))

else 

Gm = 1/((1+(4.7*Ri_Bulk))**2)

Gh = Gm

ENDIF

Ustar_Bulk = ((k*abs(UV_R))/log((Zminusd(nLUCchosen)/Z0m(nLUCchosen))))*(Gm**0.5)

IF ((Ustar_Bulk .lt. 0.1) .and. (nLUCchosen.gt.1)) then
Ustar_Bulk = 0.1
Endif

Thetastar_Bulk = ((k**2*abs(UV_R)*Theta_diff)/(Ustar_Bulk*Prt*(Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))**2)))*Gh

L_Bulk = (Ustar_Bulk**2*Theta_Av)/(k*g*Thetastar_Bulk)



!print*, 'Ri_Bulk'
!print*, Ri_Bulk

!print*, 'Gm'
!print*, Gm

!print*, 'Gh'
!print*, Gh

!print*, 'Ustar_Bulk'
!print*, Ustar_Bulk

!print*, 'Thetastar_Bulk'
!print*, Thetastar_Bulk

!print*, 'L_Bulk'
!print*, L_Bulk
!pause
!**************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Aerodynamic resistance
!Just used to get the module working and as a comparison.
!Taken from Lindroth, 1993. Zo values from Zhang et al, 2002.

!Z0m for water surfaces (follow approach by Garratt, 1999)
!If (nLUCchosen.eq.1) then
!Z0m(nLUCchosen) = (charnock_constant*(Ustar(nLUCchosen)**2))/(9.81+((0.11*kinematic_air_viscosity)/Ustar(nLUCchosen)))
!endif

!Ra = (Log(((Z-d)/Z0m(nLUCchosen))**2))/((k**2)*Uref)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Ra Aerodynamic resistance between a reference height and the quasi-laminar sub-layer 
!above the surface.
!Method: Rearranging and substituting Richardson formulae (Golder, 1972) into approach by
!Jacobson, 1999 for aerodynamic resistance based on the Monin-Obukhov length stability functions
!and therefore bypassing the need of iterations as Ustar is not initially used as a function of Z/L.
!Do not need to use Bulk Richardson approach if have gradient for both potential temperature and wind velocity.

!potential_temp_grad=-0.3
!wind_velocity_grad=1.118
potential_temp_grad=0.02 
Ri = (9.81/Temp)*(potential_temp_grad/(wind_velocity_grad**2))

!FIX
Ri=Ri_Bulk

!print*, 'Ri'
!print*, Ri

!Z/L = stability
Beta_energy = 7.8
Beta_momentum = 6.0  

!Upper limit of 0.2 !Check-Annica's book
IF (Ri .gt. 0.2) then
Ri = 0.2
ENDIF

!Stable !Upper limit of 0.2 !Check-Annica's book
IF (Ri .gt. 0.0) then
L_energy = Zminusd(nLUCchosen)/Ri
L_momentum = Zminusd(nLUCchosen)/Ri
!Unstable
ELSEIF (Ri .le. 0.0) then
L_energy = (Zminusd(nLUCchosen)*(1-(Beta_energy*Ri)))/Ri
L_momentum = (Zminusd(nLUCchosen)*(1-(Beta_momentum*Ri)))/Ri
ENDIF
 
!Integrals for sigma_m (momentum for calculation of Ustar) 
!Need to compare to lookup table for LUC for Ustar (Zhang et al, 2002) when complete to see if values reasonable. 
!Also need to remember to compare final Ra to method by Lindroth that does not consider stability.
!Still keeps values of Ra specific for each LUC as values of Z0m are obtained from a LUC lookup table.
!Need to compare with other similar approaches other than this (my own method) to check that final Ra results are reasonable.

!Also note: Gamma approximation
Gamma_energy = 11.6
Gamma_momentum = 19.3

!Stable
!If (Ri .gt. 0.0) then
If (L_Bulk .gt. 0.0) then
sigma_m_integral_Bulk = Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))+((Beta_momentum/L_Bulk)*(Zminusd(nLUCchosen)-Z0m(nLUCchosen)))
!Unstable
!elseif (Ri .lt. 0.0) then
elseif (L_Bulk .lt. 0.0) then
Gamma_momentum_function_Zr = (((1-((Gamma_momentum*Zminusd(nLUCchosen))/L_Bulk))**0.25)-1)/(((1-((Gamma_momentum*Zminusd(nLUCchosen))/L_Bulk))**0.25)+1)
Gamma_momentum_function_Z0m = (((1-((Gamma_momentum*Z0m(nLUCchosen))/L_Bulk))**0.25)-1)/(((1-((Gamma_momentum*Z0m(nLUCchosen))/L_Bulk))**0.25)+1)
sigma_m_integral_Bulk = (Log(Gamma_momentum_function_Zr))-(Log(Gamma_momentum_function_Z0m))+(2*atan((1-(Gamma_momentum*Zminusd(nLUCchosen)/L_Bulk))**0.25))-(2*atan((1-(Gamma_momentum*(Z0m(nLUCchosen)/L_Bulk)))**0.25))
!Neutral
!elseif (Ri .eq. 0.0) then !Make it less sensitive (i.e allow to vary slightly around 0)
elseif (L_Bulk .eq. 0.0) then
sigma_m_integral_Bulk = Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))
Endif

!print*, 'L_momentum'
!print*, L_momentum

!Could change back Ri=Ri_Bulk
!print*, 'L_energy'
!print*, L_energy


!print*, 'L_Bulk'
!print*, L_Bulk

!print*, 'sigma_m_integral'
!print*, sigma_m_integral_Bulk

!Friction Velocity: Ustar
Ustar_Bulk2 = k*Uref*(1/sigma_m_integral_Bulk)

!print*, 'Ustar_Bulk2'
!print*, Ustar_Bulk2


IF ((Ustar_Bulk2 .lt. 0.1) .and. (nLUCchosen.gt.1)) then
Ustar_Bulk2 = 0.1
Endif

!KEEP?
IF ((Ustar_Bulk2 .lt. 0.08) .and. (nLUCchosen.eq.1)) then
Ustar_Bulk2 = 0.08
Endif

!Have replaced Ri and L_energy by L_Bulk
!Integrals for sigma_h (energy for calculation of Ra) 
!Stable

!If (Ri .gt. 0.0) then
If (L_Bulk .gt. 0.0) then
sigma_e_integral_Bulk = (Prt*(Log(Zminusd(nLUCchosen)/Z0h)))+((Beta_energy/L_Bulk)*(Zminusd(nLUCchosen)-Z0h))

!Unstable
!elseif (Ri .lt. 0.0) then
elseif (L_Bulk .lt. 0.0) then
Gamma_energy_function_Zr = (((1-((Gamma_energy*Zminusd(nLUCchosen))/L_Bulk))**0.5)-1)/(((1-((Gamma_energy*Zminusd(nLUCchosen))/L_Bulk))**0.5)+1)
Gamma_energy_function_Z0h = (((1-((Gamma_energy*Z0h)/L_Bulk))**0.5)-1)/(((1-((Gamma_energy*Z0h)/L_Bulk))**0.5)+1)
sigma_e_integral_Bulk = Prt*((Log(Gamma_energy_function_Zr))-(Log(Gamma_energy_function_Z0h)))

!Neutral
!elseif (Ri .eq. 0.0) then
elseif (L_Bulk .eq. 0.0) then
sigma_e_integral_Bulk = Prt*(Log(Zminusd(nLUCchosen)/Z0h))

Endif






!Aerodynamic Resistance: Ra
!Ra = (1/(k*Ustar))*sigma_e_integral
Ra_Bulk = (1/(k*Ustar_Bulk2))*sigma_e_integral_Bulk

IF (Ra_Bulk.lt.5) then
Ra_Bulk=5
elseif ((nLUCchosen.eq.1) .and. (Ra_Bulk.gt.2000)) then
Ra_Bulk=2000
elseif ((nLUCchosen.gt.1) .and. (Ra_Bulk.gt.1000)) then
Ra_Bulk=1000
ENDIF
!**********************************************

!print*, 'Ra_Bulk'
!print*, Ra_Bulk
!print*, 'Ustar_Bulk2'
!print*, Ustar_Bulk2

!*********************************************

Ra=Ra_Bulk
Ustar=Ustar_Bulk2

!print*, 'Ra'
!print*, Ra
!print*, 'Ustar'
!print*, Ustar
IF (zenith_angle .gt. 90) then

Ustar=Ustar_Zhang2003_NIGHT(nLUCchosen)
!FIX just for testing for Peter/Radek Amazon data
!This is reasonable based on papers. E.g. Shuttleworth (1988) 
!paramaterises Ra=34.2/Uref (sm-1). Also average Ra daytime found to be 
!10sm-1 by Wallace and !McJannet (2006). Others also confirm this is 
!reasonable.
Ra = (Uref)/Ustar_Zhang2003_NIGHT(nLUCchosen)**2
elseif ((zenith_angle .ge. 0) .and. (zenith_angle .le. 90)) then
Ustar=Ustar_Zhang2003_DAY(nLUCchosen)
!FIX just for testing for Peter/Radek Amazon data
Ra = (Uref)/Ustar_Zhang2003_DAY(nLUCchosen)**2
ENDIF




!FIX just for testing 
!Ra = (Uref)/Ustar**2

!*******************************************************
diameter=radie*2.
!Knudsen number
Kn = (2*air_mean_path)/(2.*radie)
!Don't use anywhere. Maybe better to use Gcunn by Jacobson
!Gcunn2 = 1+(Kn*(1.249+(0.42*exp(-0.87/Kn))))

! Cunningham correction factor (based on Allen and Raabe (1982))
Gcunn = 1. + air_mean_path/(2.*radie)*(2.514 + 0.8*exp(-0.55*(2.*radie/air_mean_path)))

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Vg: Gravitational sedimentation velocity
Vg = ((diameter**2.)*(dens-air_density)*g*Gcunn)/(18.*air_viscosity)
!(Atkinsson et al and Jacobson uses the same approach as Slinn whereas Eiden ignored air_density. Follow Slinn approach here). 

!Vgh: Gravitational sedimentation velocity in the humid quasi-lamina layer near the air water interface.
!Fitzgerald, 1975
S=0.98
A=1.2*exp((0.066*S)/(1.058-S))
B=exp((0.00077*S)/(1.009-S))
radie_98=A*(rdry(kk)**B)
diameter_98=radie_98*2.
!p2052 Pryor, 1999
Vgh= ((diameter_98**2.)*(dens-air_density)*gravitation*Gcunn)/(18.*air_viscosity)

!Rb: Quasi-laminar layer resistance
!Consists of the resistance over the smooth surface Rb1 and resistance over the broken one Rb2.
!Fraction of surface area broken due to the wind force (Wu, 1979).
Ab = 1.7d-6*(U10m_R**3.75)

!Rb1: Quasi-laminar layer resistance over smooth surface
!Determined mostly by the Brownian diffusion and impaction (Slinn and Slinn, 1980)
!Brownian diffusion coefficient in the humid layer
Bd = (boltzmann*Temp*Gcunn)/(3.*pi*air_viscosity*(radie*2.))
Bdh = (boltzmann*Temp*Gcunn)/(3.*pi*air_viscosity*(radie_98*2.))

!Schmidt number (Slinn and Slinn, 1999)
Sc = kinematic_air_viscosity/Bd
!Should be Bdh

!Stokes number for water surfaces
St_water = Ustar**2.*Vgh/(gravitation*kinematic_air_viscosity)

!Collection Efficiencys: Consituents from Brownian diffusion, impaction
Eb_Water = Sc**-0.5
Eim_Water = 10**(-3./St_water)

!Rb1: Quasi-laminar layer resistance over smooth surface
!Determined mostly by the Brownian diffusion and impaction (Slinn and Slinn, 1980)
Rb1 = (k*Uref/(Ustar**2))*(1/(Eb_Water+Eim_Water))

!Rb2: Quasi-laminar layer resistance over broken surface
!Tentative value of 10s/m is used-lack of reliable measurements (Williams, 1982)
Rb2 = 10

!FIX Check dots 

Rb_Water = (((1-Ab)/Rb1)+(Ab/Rb2))**-1

!*************************************************  
!Vegetated Surface (Based on Slinn, 1982. Similar approach: Zhang et al, 2001)
!Canopy resistance
!E: Collection efficiency

!Diffusion term
Eb_Veg = Sc**-0.666667

!Impaction term (Peters and Eiden, 1992)
!Stokes Number for vegetated surfaces
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
small_collector_width = 1d-5 !(m)
!From Zhang lookup table (Changed from 1d-3)
large_collector_width = Charac_Radius(nLUCchosen)/1000 !(m)

!Forests
If ((nLUCchosen.le.6) .and. (nLUCchosen.ge.2)) then
e0 = 1.4
!Low Vegetaion
else 
e0 = 0.22
endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!From Zhang lookup table
!Prev was 1
a_constant = E_Constant_Alpha(nLUCchosen)
!Prev was 0.5
b_constant = E_Constant_Beta(nLUCchosen)

fitting_constant1 = 2
fitting_constant2 = 0.25
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
St_Veg = Ustar*Vg/(g*large_collector_width)
Eim_Veg = (St_Veg/(a_constant+St_Veg))**b_constant

!Interception term Ein
!The most uncertain part of the collection efficiency. Sensitivity analysis has shown this term
!to be insignificant compared to the other terms
F = 0.01
Ein = (F*((radie*2)/((radie*2)+small_collector_width)))+((1-F)*((radie*2)/((radie*2)+large_collector_width)))

!Bounce-off correction factor
!Assumed to take place from dry surfaces only (No Precipitation during previous 6hrs period for grass
!and current/previous periods for forest)
roff = exp(-(fitting_constant1)*St_Veg**fitting_constant2)

E = e0*(Eb_Veg+Ein+Eim_Veg)*roff

!Uh wind velocity at the canopy height H (Stability function required)
Gamma_momentum_function_Hminusd = (((1-((Gamma_momentum*Hminusd(nLUCchosen))/L_momentum))**0.25)-1)/(((1-((Gamma_momentum*Hminusd(nLUCchosen))/L_momentum))**0.25)+1)

!Stable
If (Ri .gt. 0.0) then
sigma_m_integral_Hminusd = Log(Hminusd(nLUCchosen)/Z0m(nLUCchosen))+((Beta_momentum/L_momentum)*(Hminusd(nLUCchosen)-Z0m(nLUCchosen)))
!Unstable
elseif (Ri .lt. 0.0) then
sigma_m_integral_Hminusd = (Log(Gamma_momentum_function_Hminusd))-(Log(Gamma_momentum_function_Z0m))+(2*atand((1-(Gamma_momentum*Hminusd(nLUCchosen)/L_momentum))**0.25))-(2*atand((1-(Gamma_momentum*(Z0m(nLUCchosen)/L_momentum)))**0.25))
!Neutral
elseif (Ri .eq. 0.0) then !Make it less sensitive (i.e allow to vary slightly around 0)
sigma_m_integral_Hminusd = Log(Hminusd(nLUCchosen)/Z0m(nLUCchosen))
Endif

Uh = (Ustar/k)*sigma_m_integral_Hminusd

IF (Uh .lt. 0.5) then
Uh=0.5
ENDIF

!Resistance of the interfacial sub-layer (Canopy Resistance)
Rs = Uh/(E*(Ustar**2))

!In order to calculate for low vegetation more accurate to follow approach by Wesely et al (1985).
!For L>=0 is the same though

if (L_momentum .ge. 0.0) then
Rs_lowveg = Uh/(E*(Ustar**2))
else if (L_momentum .lt. 0.0) then
Rs_lowveg = (Uh/(E*(Ustar**2)))*((1+((-100/L_momentum)**(2./3.)))**-1)
endif

!Zhang uses different formula which neglets canopy wind speed
!Could fix sensitivity issue. Replaces above Wesely correction.
Rs_lowveg = 1/(E*Ustar)

!Need to try, and also add new look up table for fitting constants

!********************************************************************!
!Non-vegetated surfaces (glaciers-Ice)
Eb_Nonveg = Eb_Veg

St_Nonveg = Ustar**2.*Vg/(gravitation*kinematic_air_viscosity)

Eim_Nonveg = 10**(-3./St_Nonveg)

Rb_Nonveg = ((k*Uref)/Ustar**2)*((Eb_Nonveg+Eim_Nonveg)**-1)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Urban areas (Giorgi, 1986)
Eim_Urban = St_Nonveg**2/(400+St_Nonveg**2)
Rb_Urban = ((k*Uref)/Ustar**2)*((Eb_Nonveg+Eim_Urban)**-1)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Dry Deposition Velocities of different surfaces
! Check http://www.emep.int/UniDoc/node11.html

!******************************************
!Turbulence test
!Ra=Ra*2

!******************************************



!Water

if (nLUCchosen.eq.1) then
Vd(nLUCchosen) = ((1+(Ra*Vg))*(1+(Rb_Water*Vgh)))/((Ra+Rb_Water)+(Ra*Rb_Water*Vgh))
!Vegetated FORESTS
elseif ((nLUCchosen.le.6) .and. (nLUCchosen.ge.2)) then
Vd(nLUCchosen) = (1/(Ra+Rs))+Vg
!Vegetated Low Vegetation
elseif (((nLUCchosen.le.13) .and. (nLUCchosen.ge.7)).or. (nLUCchosen.eq.15)) then
Vd(nLUCchosen) = (1/(Ra+Rs_lowveg))+Vg
!Non-Vegetated
elseif ((nLUCchosen.eq.16) .or. (nLUCchosen.eq.17)) then
Vd(nLUCchosen) = (1/(Ra+Rb_Nonveg+(Ra*Rb_Nonveg*Vg)))+Vg
!Urban
elseif (nLUCchosen.eq.14) then
Vd(nLUCchosen) = (1/(Ra+Rb_Urban+(Ra*Rb_Urban*Vg)))+Vg
endif

!print *, 'Vd'
!write(6,*) Vd(nLUCchosen)      
!pause
 
DRY_DEP_1=vd(NLUCCHOSEN)


!print *,DRY_DEP_1
!write(6,*) DRY_DEP_1


!print *,'Ra'
!write(6,*) Ra

!print *,'Rs'
!write(6,*) Rs

!print *,'Ri'
!write(6,*) Ri

!print*, dom_class

if ((dry_dep_1>0.1) .or. (dry_dep_1<0))  then

if (dry_dep_1<0) then
dry_dep_1=1d-15
end if

end if
3 continue
end do
end subroutine dry_deposition_aerosols

end module dynamic_lib
