!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
!
! Contains subroutines and functions to evaluate the effect of coagulation,
!	condensation and nucleation on particle distribution
!
! UHMA - University of Helsinki Multicomponent Aerosol model
! (c) Hannele Korhonen 2003
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

module dynamics_dyn

  use global_parameters

  implicit none

  private

  integer, parameter :: &
			hydmax = 5					

  ! hydrate stuff
  real*8 :: hydpw					
  real*8, dimension(hydmax) :: rhoh, kprod		

  public :: coagulation, condensation, nucleation, dry_deposition, kin_nucleation, dry_deposition_aerosols, dry_dep_gases, qual_check

contains

!ooooooooooooooooooo  GENERAL STUFF  ooooooooooooooooooooooooooooooooooo

!-----------------------------------------------------------
! Calculate water and sulphuric acid saturation vapour 
!	pressures in ambient temperature (Pa)
!-----------------------------------------------------------

  function waterps()

	implicit none
	real*8 :: waterps

	waterps=exp(77.344913 - 7235.4247/temp - 8.2*log(temp) + 0.0057113*temp)

  end function waterps


  function acidps()

	implicit none
	real*8 :: lpar, acidps

	lpar= -11.695+log(pstand) ! Zeleznik
	acidps = 1/360.15-1.0/temp + 0.38/545.*(1.0+log(360.15/temp)-360.15/temp )
	acidps = 10156.0*acidps +lpar
	acidps = exp(acidps)
	
  end function acidps


!oooooooooooooooooooooooooooooo  COAGULATION	oooooooooooooooooooooooooooooooooooooooooooooooo
!
! Subroutine COAGULATION and function COAG_COEFF calculate the effect of
!	coagulation on particle population 
!
!	n_coag			=	particle concentration change due to coagulation in each section (#/m^3 s)
!	vol_coag		=	vol conc. change of each compound  - " - (um^3/(m^3 s))
!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine coagulation(n_coag, vol_coag)

	implicit none

	real*8, dimension(sections), intent(out) :: n_coag
	real*8, dimension(sections, compo), intent(out) :: vol_coag 

	integer :: ii, jj, kk,oo,pp
	real*8, dimension(compo) :: vol_new
	real*8 :: core_new, aa, bb, coagul, delta
		
	n_coag = 0.			! number concentration net change rate (#/m^3 s)
	vol_coag = 0.		! volume concentration net change rate (um^3/(m^3 s))

	! particles and volume added to each section by coagulation
	do jj = 1,sections	
		
		if (n_conc(jj) < n_limit) goto 10

		do ii = 1,jj		
			if (n_conc(ii) < n_limit) goto 20

			if (ii.eq.jj) then
				delta = 0.5		!self-coagulation (indistinguishability)
			else
				delta = 1.
			end if

			! volume of the core (m^3) and constituent volumes (um^3) of one
			! particle formed in collision 		
			core_new = core(ii) + core(jj)												
			vol_new = vol_conc(ii,:)/n_conc(ii) + vol_conc(jj,:)/n_conc(jj)		

			do kk = jj, sections-1
		

				! find the sections between which the new particle falls
				if (core_new >= core(kk) .and. core_new < core(kk+1)  ) then			
				pp=kk
				oo=kk+1

				if (n_conc(kk)==0) pp=kk-1
				if(n_conc(kk+1)==0) oo=kk+2
				if (oo>sections) oo=sections

					aa = (core(oo)-core_new)/(core(oo)-core(pp))	! number fraction moved to lower section
					bb = (core_new-core(pp))/(core(oo)-core(pp))		!			-"-			higher section

					coagul = delta*coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj)	! collision frequency (m^-3 s^-1)
					n_coag(kk) = n_coag(kk) + aa*coagul			! (m^-3 s^-1)
					n_coag(kk+1) = n_coag(kk+1) + bb*coagul
					vol_coag(kk,:) = vol_coag(kk,:) + aa*core(kk)/core_new*coagul*vol_new	! (um^3/(m^3 s))
					vol_coag(kk+1,:) = vol_coag(kk+1,:) + bb*core(kk+1)/core_new*coagul*vol_new

					goto 20

				end if
			end do

			! if resulting particle larger than the biggest size section, all particles go to biggest section
			! thus some volume flows out - choose the largest bin large enough!
			if (core_new >= core(sections)) then

				n_coag(sections) = n_coag(sections) + delta*coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj)* &
								core_new/core(sections)
				vol_coag(sections,:) = vol_coag(sections,:) + delta*coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj) &
										*vol_new
			end if

20			continue
		end do
10		continue
	end do

	! number and volume removed from each section by coagulation
	do ii = 1, sections
		if (n_conc(ii) < n_limit) goto 30
	
		do jj = 1, sections		
			if (n_conc(jj) < n_limit) goto 40	

			coagul = coag_coeff(ii,jj)*n_conc(ii)*n_conc(jj)		! collision frequency (m^-3 s^-1)
			n_coag(ii) = n_coag(ii) - coagul
			vol_coag(ii,:) = vol_coag(ii,:) - coagul*vol_conc(ii,:)/n_conc(ii)

40			continue
		end do
30		continue
	end do

  end subroutine coagulation

!---------------------------------------------------------------------------
!  
!  Calculates coagulation coefficients for Brownian coagulation (m^3/s)
!
!  Following XXX	
!  
!---------------------------------------------------------------------------

  function coag_coeff(ii,jj)

	implicit none

	integer, intent(in) :: ii, jj
	real*8 :: coag_coeff
	real*8 :: free_path, viscosity, vel_12, rad_12, dif_12
	real*8 :: prov1, prov2, dist, continuum, free_molec
	real*8, dimension(2) :: radii, m_part, knudsen, corr, diff, veloc, omega

	! air mean free path (m) and viscosity (kg/ms) (where from?)
	free_path = (6.73d-8*temp*(1.+110.4/temp))/(296.*pres/pstand*1.373)	
	viscosity = (1.832d-5*406.4*temp**1.5)/(5093*(temp+110.4))
				
	! for both sections	
	radii(1) = radius(ii)	! radii of colliding particles (m)
	radii(2) = radius(jj)
	m_part(1) = mass(ii)	! masses of colliding particles (kg)			
	m_part(2) = mass(jj) 

	knudsen = free_path/radii								! particle Knudsen number	
	corr = 1. + knudsen*(1.142+0.558*exp(-0.999/knudsen))	! Cunninghan correction factor (Allen and Raabe, Aerosol Sci. Tech. 4, 269)
	diff = boltzmann*temp*corr/(6*pi*viscosity*radii)		! particle diffusion coefficient (m^2/s)
	veloc = sqrt((8.*boltzmann*temp)/(pi*m_part))			! mean thermal velocity of a particle (m/s)

	omega = 8.*diff/(pi*veloc)								! ????? (m) 

	vel_12 = sqrt(veloc(1)**2 + veloc(2)**2)	! mean relative thermal velocity
	rad_12 = sum(radii)
	dif_12 = diff(1)+diff(2)					! relative diffusion coefficient 
	continuum = 4.*pi*rad_12*dif_12				! flux in continuum regime					
	free_molec = pi*vel_12*rad_12**2			! flux in free molecular regime

	! flux matching according to Fuchs (1964) (e.g. Seinfeld & Pandis p. 661)
	prov1 = (rad_12+omega(1))**3 - (rad_12**2 + omega(1)**2)**1.5 
	prov1 = prov1/(3.*rad_12*omega(1)) - rad_12
	prov2 = (rad_12+omega(2))**3 - (rad_12**2 + omega(2)**2)**1.5 
	prov2 = prov2/(3.*rad_12*omega(2)) - rad_12

	dist = sqrt(prov1**2 + prov2**2)	! distance at which fluxes are matched

	! coagulation coefficient between particles [m^3/s]
	coag_coeff = continuum / (rad_12/(rad_12+dist) + continuum/free_molec)

  end function coag_coeff


!oooooooooooooooooooooooooo  DEPOSITION  ooooooooooooooooooooooooooooooooooooooooooooooo
!
!  Calculates dry deposition velocities for particles -> loss of number and volume
!	concentration due to dry deposition
!
!  The parameterization and the parameter values used here are based on: 
!	Rannik et al, Interpretation of aerosol particle fluxes over a pine forest: 
!	Dry deposition and random errors, JGR, 2003.
!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine dry_deposition(boundaryheight, n_depo, vol_depo, land)
	implicit none
	
	real*8, intent(in) :: boundaryheight, land
	real*8, dimension(sections), intent(out) :: n_depo
	real*8, dimension(sections, compo), intent(out) :: vol_depo

	integer :: ii

	real*8, dimension(sections) ::  corr_coeff, settling_veloc, r_c, diff_part
	real*8, dimension(sections) :: schmidt, brownian, empirical, dep_veloc, eps,r_b, xx1

	real*8 :: fric_veloc, exponent, wind_canopy, gamma, collection, dmin
	real*8 :: air_mean_path, air_viscosity, air_density
	real*8 :: r_a, prefactor, diff_dmin, schmidt_dmin, deriv_coeff, deriv_schmidt, coeff_dmin, adep, bdep, z0

! the following parameter values are based on Hyytil measurements
! they are not (necessarily) applicable to other sites!!!

		! median wind speed [m/s]
	fric_veloc = 0.4		! median friction velocity [m/s]
	exponent = 2.5		
	wind_canopy = 1.5		! wind speed at canopy height [m/s]
	gamma = 1.9				! coefficient of exponental decrease of wind speed inside canopy
	collection = 2.9		! collection efficiency factor (from fig. 7; TRFE < 15%)
	dmin = 14.6d-8			! diameter of minimum in collection efficiency (from fig. 7; TRFE < 15%)

! ambient air properties
	air_mean_path = (6.73d-8*temp*(1.+110.4/temp))/(296.*pres/pstand*1.373)	
	air_viscosity = (1.832d-5*406.4*temp**1.5)/(5093*(temp+110.4))
	air_density = 1.2929*273.15*pres/(temp*pstand)		! [kg/m^3]     

! Cunningham correction factor (based on Allen and Raabe (1982))
	corr_coeff = 1. + air_mean_path/(2.*radius)*(2.514 + 0.8*exp(-0.55*(2.*radius/air_mean_path)))

! particle gravitational settling velocity [m/s]
	settling_veloc = corr_coeff*mass*gravitation/(6.*pi*air_viscosity*radius)
 
! aerodynamic resistance [s/m]  
	r_a = (wind_speed - wind_canopy)/fric_veloc**2

! collection efficiency of particles due to Brownian diffusion
	diff_part = boltzmann*temp*corr_coeff/(6.*pi*air_viscosity*radius)	! diffusivity of particles [m^2/s]
	schmidt = air_viscosity/(air_density*diff_part)						! particle Schmidt number
	brownian = 1./3.*schmidt**(-2./3.)								

! calculating prefactor (as a function of dmin) for empirical collection efficiency of particles
	coeff_dmin = 1. + air_mean_path/dmin*(2.514 + 0.8*exp(-0.55*dmin/air_mean_path))
	diff_dmin = boltzmann*temp*coeff_dmin/(3.*pi*air_viscosity*dmin)
	schmidt_dmin = air_viscosity/(air_density*diff_dmin)
	deriv_coeff = -2.514*air_mean_path/dmin**2 - &	! derivative of Cunninghan correction factor 
				(air_mean_path/dmin + 0.55)*0.8/dmin*exp(-0.55*dmin/air_mean_path)
	deriv_schmidt = schmidt_dmin*(1./dmin - 1./coeff_dmin*deriv_coeff) ! deriv. of Schmidt number
	prefactor = 2./9.*schmidt_dmin**(-5./3.)* &
				deriv_schmidt/(exponent*dmin**(exponent-1.))

	empirical = 4.*prefactor*radius**exponent		! empirical collection efficiency of particles

	eps = collection*(brownian + empirical)	! total collection efficiency of particles
	eps = sqrt(eps)	

! canopy resistance [s/m]
	r_c = wind_canopy/(fric_veloc**2*eps)*((1. + eps*tanh(gamma*eps))/(eps + tanh(gamma*eps))) 
	
	adep=0.4
	bdep=20
	z0=1.d-2

air_viscosity=(1.832d-5*temp**1.5*406.4)/(5093*(temp+110.4))/1.85d3;
diff_part = boltzmann*temp*corr_coeff/(6*pi*air_viscosity*1.85d3*radius)
schmidt=air_viscosity/diff_part;
xx1=2*radius*(fric_veloc/z0/air_viscosity)**(1.d0/2.d0)*schmidt**(1.d0/3.d0);
r_b=1/(diff_part/2/radius*(adep*xx1+bdep*xx1**3));
! deposition velocity [m/s]
	
	dep_veloc = settling_veloc + 1./(r_a + r_c)


	if (land==0) then
	dep_veloc = (settling_veloc + 1./(r_a + r_b+r_a*r_b*settling_veloc))

end if

! number and volume concentration change rate in each bin 
	do ii = 1,sections
		if (n_conc(ii) > n_limit) then



	
			n_depo(ii) = -dep_veloc(ii)/boundaryheight*n_conc(ii)	![#/m^3 s]
	
			vol_depo(ii,:) = -dep_veloc(ii)/boundaryheight*vol_conc(ii,:)	![um^3/m^3 s]

		else
			n_depo(ii) = 0.
			vol_depo(ii,:) = 0.
		end if
	end do

  end subroutine dry_deposition

!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

!ooooooooooooooooooooooo  GAS-TO-PARTICLE CONVERSION  oooooooooooooooooooooooooooooooooo
!
!	Subroutine HYDRATES evaluates hydrate coefficients for sulphuric acid nucleation
!		and condensation
!
!	Subroutine CONDENSATION (uses subroutine COLLISION_RATE and function DIFFUSION) 
!		calculates the effect of condensation on particle population 
!
!	Subroutine NUCLEATION (uses either TER_NUCLEATION or BIN_NUCLEATION) calculates 
!		the effect of nucleation on particle population
!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

!--------------------------------------------------------
!  Evaluates hydrate equilibrium coefficients and  
!    hydrate fractions in gas phase for sulphuric acid 						  
!
!	(c) Hanna Vehkamki 2002	
!
!		Division of Atmospheric Sciences
!		Department of Physical Sciences
!		P.O. Box 64
!		FIN-00014 University of Helsinki
!		Finland
!
!		(hanna.vehkamaki@helsinki.fi)
!--------------------------------------------------------

  subroutine hydrates()

	implicit none

	integer :: ii, jj
	real*8, dimension(hydmax) :: kh
	real*8 :: pres_h2o, VOL_TOL

	pres_h2o = rh*waterps()

	! Zeleznik equilibrium constant for reactions
	kh(1)= exp(6159.1796/temp-14.3343)	! water+h2so4 =1-hydrate
	kh(2)= exp(5813.8776/temp-15.5140)	! i-hydrate+water= i+1-hydrate
	kh(3)= exp(5246.0/temp-14.90)		! i-hydrate+water= i+1-hydrate
	kh(4)= exp(4934.0/temp-14.47)		! i-hydrate+water= i+1-hydrate
	kh(5)= exp(4763.0/temp-14.21)		! i-hydrate+water= i+1-hydrate

	do ii=1,hydmax
		kprod(ii)=1. !product of the equilibrium constants k1*k2*k3*..*ki1
		do jj = 1,ii
			kprod(ii)= kh(jj)*kprod(ii)
		end do
	end do

	! Jaecker-Voirol et al., J. Chem. Phys., vol. 87, 4849-4852 (1987):
	! hydpw = pa(total)/pa(free) = 1+k1(pw/p0)+..+k1*k2*..*ki1(pw/p0)^ii+..+
	hydpw = 1.0
	do ii = 1,hydmax
		hydpw = hydpw + kprod(ii)*(pres_h2o/pstand)**ii
	end do 

	!concentration of the hydrates (divided by total concentration)
	!rho(ii)/rho(tot)= k1*k2*..*ki1(pw/p0)^ii/hydpw
	do ii = 1,hydmax
		rhoh(ii) = kprod(ii)*(pres_h2o/pstand)**ii/hydpw 
	end do

  end subroutine hydrates


!---------------------------------------------------------------------------------------------
!
!  Calculates the effect of condensation of different vapours on 
!		particle population over a given timestep. The subroutine updates
!		the size distribution but NOT the gas concentrations.
!
!	timestep	=	timestep over which condensation is calculated (s)
!	sink		=	condensable vapours consumed by condensation (#/m^3 s)
!	
!---------------------------------------------------------------------------------------------

  subroutine condensation(timestep, sink)

	implicit none
	
	real*8, intent(in) :: timestep
	real*8, dimension(cond), intent(out) :: sink

	integer :: ii, jj, kk, io

	real*8 :: water_cont, s_org, aa, bb, vol_new2, vol_tol
	real*8, dimension(sections,compo) :: vol_change, vol_cond, vol_cond2
	real*8, dimension(sections) :: tot_change, collision, n_cond, vol_new, kelvin_eff, sat_conc, &
										sat_org, n_temp, n_cond2 

	collision = 0.	! collision rate of molecules (m^3/s)
	sink = 0.		! vapour sink (#/m^3 s)

	n_cond = 0.			! number concentration of particles that is moved (and/or stays) in the bin
	vol_cond = 0.		! volume concentration of each component	- " -

	vol_change = 0.

    do ii = 1,cond		! for each condensable vapour whose concentration is high enough
		if(vap_conc(ii) > vap_limit) then
			
			collision = collision_rate(ii)	! [m^3/s]
			
			call kelvin(ii, kelvin_eff)
			sat_conc = c_sat(ii)*kelvin_eff

			if (ii == 2) then

				do jj = 1,sections
					if (n_conc(jj) > n_limit) then
						call nanokohler(jj,water_cont,s_org, ii)
							sat_conc(jj) = c_sat(ii)*s_org	
								if (sat_conc(jj)<vap_conc(2)) then
						sat_conc(jj+1:sections)=0
					
						goto 13
						end if
					end if
				end do

			end if
	13 continue
			vol_change(:,ii) = collision*molecvol(ii)*(vap_conc(ii) - sat_conc)	! volume change rate of a particle (m^3/s)
			vol_change(:,ii) = vol_change(:,ii)*timestep			

			! not so elegant way to make sure that we don't divide by zero below 
			n_temp = n_conc
			where (n_temp <= 0.)
				n_temp = n_limit
			end where
			
			where (-vol_change(:,ii)*n_conc >= vol_conc(:,ii)/scale**3)		
				vol_change(:,ii) = -vol_conc(:,ii)/(n_temp*scale**3)	! not more than exists in bin evaporates
			end where

			sink(ii) = sum(n_conc*vol_change(:,ii))/(molecvol(ii)*timestep)	! (#/ m^3 s)

		end if
	end do





!irreversible condensation
!	where (vol_change<0)
!	vol_change=0
!	end where


	tot_change = sum(vol_change,2)		! total volume change in each bin (m^3)
	vol_new = core + tot_change			! the volume the particles in each bin grow to (m^3)
!dist_approach='mc'
	! choose the size distribution approach used for solving condensation
	select case(dist_approach)

!------ Moving center approach (Jacobson, Atmos. Environ., 31, 131-144) --------------------
	
	case('mc')


		! find the section that particles have grown/shrank to	
		do ii = 1,sections
		
		
			if (n_conc(ii) > n_limit .and. vol_new(ii) >= lowervol(ii)) then !condensation, or evaporation within bin
					
				do jj = ii,sections
					if (lowervol(jj) <= vol_new(ii) .and. vol_new(ii) < uppervol(jj)) then
						n_cond(jj) = n_cond(jj) + n_conc(ii)
						vol_cond(jj,:) = vol_cond(jj,:) + vol_conc(ii,:)	! condensation/evap. not taken into account yet				
						vol_cond(jj,1:compo) = vol_cond(jj,1:compo) + vol_change(ii,1:compo)*scale**3*n_conc(ii) ! (um^3/m^3)					
						goto 100

					end if
				end do
			else if (n_conc(ii) > n_limit) then 	! evaporation to a lower bin
				do jj = ii-1,1,-1	! starting from bin next to ii
					if (lowervol(jj) <= vol_new(ii) .and. vol_new(ii) < uppervol(jj)) then
						n_cond(jj) = n_cond(jj) + n_conc(ii)
						vol_cond(jj,:) = vol_cond(jj,:) + vol_conc(ii,:)	! condensation/evap. not taken into account yet				
						vol_cond(jj,1:compo) = vol_cond(jj,1:compo) + vol_change(ii,1:compo)*scale**3*n_conc(ii) ! (um^3/m^3)					
						goto 100
					end if
				end do
			end if
	
			! particles that have evaporated/grown out of simulated size range
			! treated normally here and checked again when particles allocated to new bins
			if (vol_new(ii) < lowervol(1)) then		
				n_cond(1) = n_cond(1) + n_conc(ii)					! (#/m^3)
				vol_cond(1,:) = vol_cond(1,:) + vol_conc(ii,:)		! no cond./evap taken into account yet
				vol_cond(1,1:compo) = vol_cond(1,1:compo) + vol_change(ii,1:compo)*scale**3*n_conc(ii) ! (um^3/m^3)					
			end if 
			
			if (vol_new(ii) >= uppervol(sections)) then	
				n_cond(sections) = n_cond(sections) + n_conc(ii)				! (#/m^3)
				vol_cond(sections,:) = vol_cond(sections,:) + vol_conc(ii,:)	! no cond./evap taken into account yet
				do kk = 1,compo										! effect of condensation/evaporation (um^3/m^3)		
					vol_cond(sections,kk) = vol_cond(sections,kk) + vol_change(ii,kk)*scale**3*n_conc(ii)  			
				end do
			end if	

100			continue
		
			do kk = 1,compo
		vol_tol=1e-9
			if (vol_cond(ii, kk)<vol_tol) then
		vol_cond(ii, kk)=0
			end if
				if (vol_cond(ii,kk) < 0.) then

				write(*,*) n_conc(ii), vol_cond(ii, kk), ii, kk
				stop 'Negative volume concentrations!'
				end if
			end do

		end do


		! particles allocated to their new bins
		do ii = 1,sections
		
			if (n_cond(ii) > n_limit) then
				n_conc(ii) = n_cond(ii)									! (m^-3)
				core(ii) = sum(vol_cond(ii,:))/(n_cond(ii)*scale**3)	! (m^3)
				vol_conc(ii,:) = vol_cond(ii,:)							! (um^3/m^3)
				! for smallest and largest bin: volume conserved, thus number over- or underestimated
				if (core(ii) < lowervol(1)) then						
					n_conc(ii) = n_conc(ii)*core(ii)/lowervol(ii)
					core(ii) = lowervol(ii)
				else if (core(ii) >= uppervol(sections)) then
					n_conc(ii) = n_conc(ii)*core(ii)/uppervol(ii)
					core(ii) = uppervol(ii)			
				end if	
				rdry(ii) = (3.*core(ii)/(4.*pi))**(1./3.)				! (m)
			else 
				n_conc(ii) = 0.
				core(ii) = midvol(ii)
				rdry(ii) = (3.*core(ii)/(4.*pi))**(1./3.)
				vol_conc(ii,:) = 0.			
			end if
		end do


!----- by default fixed sectional approach -------------------------------------------------
! 
! when using the code below, make sure that the particles DO NOT grow over a size section
!	within one time step
!
	case default



		! find the section that particles have grown/shrank to	
		do ii = 1,sections
			if (n_conc(ii) >= n_limit) then
			
				if (vol_new(ii) >= core(ii)) then !condensation, or evaporation within bin					
					do jj = ii,sections-1
						if (core(jj) <= vol_new(ii) .and. vol_new(ii) < core(jj+1)) then
							n_cond(jj) = n_cond(jj) + n_conc(ii)

							vol_cond(jj,:) = vol_cond(jj,:) + vol_conc(ii,:)	! condensation/evap. not taken into account yet				
							
					
							vol_cond(jj,:) = vol_cond(jj,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
							goto 200
						end if
					end do
				else	! evaporation to a lower bin
					do jj = ii-1,1,-1	! starting from bin next to ii
						if (core(jj) <= vol_new(ii) .and. vol_new(ii) < core(jj+1)) then
							n_cond(jj) = n_cond(jj) + n_conc(ii)
							vol_cond(jj,:) = vol_cond(jj,:) + vol_conc(ii,:)	! condensation/evap. not taken into account yet				
							vol_cond(jj,:) = vol_cond(jj,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
														
							goto 200
						end if
					end do
				end if
			end if


			if (vol_new(ii) >= core(sections)) then	
				n_cond(sections) = n_cond(sections) + n_conc(ii)				! (#/m^3)
				vol_cond(sections,:) = vol_cond(sections,:) + vol_conc(ii,:)	! no cond./evap taken into account yet
				vol_cond(sections,:) = vol_cond(sections,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
				goto 200
			end if	
	
			! particles that have evaporated/grown out of simulated size range
			! treated normally here and checked again when particles allocated to new bins
			if (vol_new(ii) < core(1)) then		
				n_cond(1) = n_cond(1) + n_conc(ii)					! (#/m^3)
				vol_cond(1,:) = vol_cond(1,:) + vol_conc(ii,:)		! no cond./evap taken into account yet
				vol_cond(1,:) = vol_cond(1,:) + vol_change(ii,:)*scale**3*n_conc(ii) ! (um^3/m^3)					
			end if 
			
200			continue

		end do

		where (vol_cond < 0.)
			vol_cond = 0.
		end where

		n_cond2 = 0.
		vol_cond2 = 0.

		! particles allocated to their new bins
		do ii = 2,sections-1
				
			if (n_cond(ii) > n_limit) then

				vol_new2 = sum(vol_cond(ii,:))/(n_cond(ii)*scale**3)	! (m^3)
				aa = (core(ii+1)-vol_new2)/(core(ii+1)-core(ii))
				bb = 1.-aa			

				n_cond2(ii) = n_cond2(ii)+aa*n_cond(ii)
				n_cond2(ii+1) = n_cond2(ii+1) + bb*n_cond(ii)					
				! effect of condensation/evaporation
				vol_cond2(ii,:) = vol_cond2(ii,:) + aa*core(ii)/vol_new2*vol_cond(ii,:) ! (um^3/m^3)					
				vol_cond2(ii+1,:) = vol_cond2(ii+1,:) + bb*core(ii+1)/vol_new2*vol_cond(ii,:)   ! (um^3/m^3)					

			end if
		end do

	


		if (n_cond(1) > n_limit) then
			
			vol_new2 = sum(vol_cond(1,:))/(n_cond(1)*scale**3)	! (m^3)
			if (vol_new2 >= core(1)) then
				aa = (core(2)-vol_new2)/(core(2)-core(1))
				bb = 1.-aa			

				n_cond2(1) = n_cond2(1)+aa*n_cond(1)
				n_cond2(2) = n_cond2(2) + bb*n_cond(1)					
				! effect of condensation/evaporation
				vol_cond2(1,:) = vol_cond2(1,:) + aa*core(1)/vol_new2*vol_cond(1,:) ! (um^3/m^3)					
				vol_cond2(2,:) = vol_cond2(2,:) + bb*core(2)/vol_new2*vol_cond(1,:)   ! (um^3/m^3)					
			else
				n_cond2(1) = n_cond2(1) + vol_new2*n_cond(1)/core(1)
				vol_cond2(1,:) = vol_cond2(1,:) + vol_cond(1,:)
			end if
		end if



		if (n_cond(sections) > n_limit) then

			vol_new2 = sum(vol_cond(sections,:))/(n_cond(sections)*scale**3)	! (m^3)
			if (vol_new2 >= core(sections)) then						
				n_cond2(sections) = n_cond2(sections) + vol_new2*n_cond(sections)/core(sections)
				vol_cond2(sections,:) = vol_cond2(sections,:) + vol_cond(sections,:)
			else
				write(*,*) 'Something wrong here!'
			end if
		end if



	n_conc = n_cond2
		vol_conc = vol_cond2
	end select

  end subroutine condensation

!------------------------------------------------

  subroutine kelvin(ii, kelvinEffect)

	implicit none

	integer, intent(in) :: ii
	real*8, dimension(sections), intent(out) :: kelvinEffect
	real*8, dimension(sections) :: exponent
	
	exponent=2.*surf_ten(ii)*molecvol(ii)
	exponent=exponent/(boltzmann*temp*radius)
	kelvinEffect=dexp(exponent)

  end subroutine kelvin


!----------------------------------------------------------------------------
!----------------------------------------------------------------------------

  subroutine nanokohler(ii,water_cont, sat_org, kk)

	implicit none

	integer, intent(in) :: ii, kk
	real*8, intent(out) :: water_cont, sat_org

	integer :: jj
	real*8 :: vol_org, vol_as, n_org, n_as, surften_org, dens_as, dens_h2o, r
	real*8 :: aa, cc, dd, mass_as, mass_h2o, vol_h2o, dens_org, nu_org, nu_as
	real*8 :: nwmin, nwmax, n_h2o, nwold, mass_org, rad, x2, fx, xo, surt, pjw
	real*8 :: gamw, gamo, gapb, gambi, xbw, f, xbo, rw, v_as, v_org

	v_org = vol_conc(ii,kk)/(n_conc(ii)*scale**3)	! volume of water soluble organic content			
	n_org = v_org/molecvol(kk)						! number of organic molecules in particle

	v_as = vol_conc(ii,1)/(n_conc(ii)*scale**3)	! volume of sulphate content in one particle (m^3)
	n_as = v_as/molecvol(1)						! number of sulphate molecules in particle		

	if (n_org <= 0.) n_org = 0.05*n_as

    if (n_as <= 0.) then
		write(*,*) 'number of sulphate molecules is smaller than zero'
		write(*,*) ii, vol_conc(ii,:), n_conc(ii), rdry(ii)
		Write(*,*) 'adding one molecule sulfuric acid'
		n_as=1.
		v_as=molecvol(1)*n_as
		vol_conc(ii, 1)=v_as*(n_conc(ii)*scale**3)
		
	end if

! fitted thermodynamic parameters
    aa = -190.
    cc = 0.3
    dd = 0.8

    surften_org = 0.03		! (N/m)

    dens_as = 1780.			! density of ammonium bisulphate (kg/m^3)
    dens_h2o = 1000.		! -"- of water (kg/m^3)
    mass_as = 115.11		! molar mass of ammonium bisulphate (g/mol)
    mass_h2o = 18.02		! -"- water

	mass_org = 186.
	dens_org = 1107.

    vol_as = mass_as*1.d-3/(avogadro*dens_as)
    vol_h2o = mass_h2o*1.d-3/(avogadro*dens_h2o)
	vol_org= mass_org*1.d-3/(avogadro*dens_org)	! remember this: v_org = 2.25d-28 m^3         

	nu_org= 1.				! dissociation factor
    nu_as = 2.

! r= dry radius of a particle:
    r =  (3./pi/4.*(n_as*vol_as + n_org*vol_org))**(1./3.)

    nwmin = 0.
    nwmax= (4.*(pi/3.)*((2.5*r)**3.))/vol_h2o

    n_h2o = 0.
    do jj=1,20           
		nwold= n_h2o
        n_h2o = (nwmin+nwmax)/2.
        rad= ((3./4.)*(n_as*vol_as+n_h2o*vol_h2o+n_org*vol_org)/pi)**(1./3.)
        call xbulk(n_h2o,n_org,n_as,rad,x2,fx)
        xo = x2
        if (xo .eq. 0. ) then
			xo= (1.d-10)*n_as/(n_as+n_h2o)           
        endif
        call pseust(xo,n_h2o,n_as,surt,pjw)
        call pseubin(xo,n_h2o,n_as,gamw,gamo,gapb,gambi,n_org) 
        xbw = 1./(1./(1.-xo) +2.*n_as/n_h2o)
! r instead of rad in original:
        f = rh/(dexp(2.*surt*vol_h2o/boltzmann/temp/rad)*xbw*gamw)-1.
 
        if(f.lt.0.) then
			nwmax = n_h2o
        else
            nwmin = n_h2o
        endif
	end do          
    xbo = xo*xbw/(1.-xo)
! r instead of rad in original:
    sat_org = gamo*xbo*dexp(2.*surt*vol_org/(boltzmann*temp*rad))
  
    rw = rad        
    water_cont= (n_h2o/avogadro)*mass_h2o     
	
  end subroutine nanokohler



  subroutine xbulk(n1,n2,n3,rad,x2,fb)

	implicit none

	real*8, intent(in) :: n1, n2, n3, rad
	real*8, intent(out) :: x2, fb

	integer :: i
	real*8 :: xl, xu, x, xy, xa
	real*8 :: g1y, g2y, gpby, gabi, g1a, g2a, gpba
	real*8 :: surty, surta, pj, dsurt, dmu1, dmu2

	xl = 0.
    xu = 1.
    if (n2 .eq. 0.) then
		x= 0.
        goto 11
    endif
    do i=1,40
		x = (xl+xu)/2.
        xy = x+.00001
        xa = x-.00001         
        if (xa .lt. 0.) xa= 1.d-9
        if (xy .gt. 1.) xy= 1.        
        call pseubin(xy,n1,n3,g1y,g2y,gpby,gabi,n2)  
        call pseubin(xa,n1,n3,g1a,g2a,gpba,gabi,n2)       
        call pseust(xy,n1,n3,surty,pj)
        call pseust(xa,n1,n3,surta,pj)         
        dsurt = (surty-surta)/(xy-xa)       
        dmu1 = boltzmann*temp*(dlog((1.-xy)*gpby)-dlog((1.-xa)*gpba)) &
              /(xy-xa)
        dmu2 = boltzmann*temp*(dlog(xy*g2y)-dlog(xa*g2a))/(xy-xa)
        fb = n1*dmu1 + n2*dmu2 + 4.*pi*rad**2*dsurt
        if(fb.gt.0.) then
			xl = x
        else
			xu = x
        endif
	end do        
11	x2 = x

  end subroutine


! pseudobinary surface tension for a system with n1 water, n2 organic 
! and n3 ammonium sulfate molecules
  subroutine pseust(xbu,n1,n3,stb,stwa)

	implicit none

	real*8, intent(in) :: xbu, n1, n3
	real*8, intent(out) :: stb, stwa 

	real*8 :: m3, m1, st1, xb1, w, xbw, xbo, xbws, surften_org, cc, dd


	cc = 0.3 
	dd = 0.8
	surften_org = 0.02	!(N/m)
    m3 = 115.11
    m1 = 18.02
    st1 = (93.6635+.009133*temp-.000275*temp**2)/1000.
    stwa=st1

    xb1 = 1.-xbu
    w = n3*m3/(n1*m1+n3*m3)

! water-ammonium bisulfate surface tension

    st1 = st1 + 0.01467*w

! water-bisulfate-organic surface tension model

    xbw = 1./(1./(1.-xbu) +2.*n3/n1)
    xbo = xbu*xbw/(1.-xbu)
    xbws = 1.-xbo
  
    stb = st1-(st1-surften_org)*(1.+cc*xb1/(1.-dd*xb1))*xbu

  end subroutine


! pseudobinary activity coefficients for a system with n1 water, n2 organic 
! and n3 ammonium sulfate molecules
  subroutine pseubin(xb2,n1,n3,gam1,gam2,vl1,acs,n2)

	implicit none

	real*8, intent(in) :: xb2, n1, n3, n2
	real*8, intent(out) :: gam1, gam2, vl1, acs

	real*8 :: rho1, r, m3, rho3, m1, amu, v11, v3, v1, bvl
	real*8 :: xb1, xs1, w, vl2, c1, c2, c3, wf, act, aa


	aa = -190.
	rho1 = 1000.0D0
    r = 8.3143D0
    m3 = 115.11
    rho3 = 1780.
    m1 = 18.02
    amu = 1.67d-27
    v11 = m1*amu/rho1
    v3 = m3*amu/rho3  
    v1 = v11+n3/n1*v3
    bvl= 100.

    xb1=1.-xb2

    xs1 = n1/(n1+2.*n3)
    w = n3*m3/(n1*m1+n3*m3)

! Van Laar coefficients

    vl1 = aa/(1.+xb1/(xb2*bvl))**2/r/temp
    vl1 = dexp(vl1)     
    vl2 = aa*bvl/(1.+bvl*xb2/xb1)**2/r/temp
    vl2 = dexp(vl2)

! water activity coefficient in amm. bisulf. solution

    c1 = -3.05d-3
    c2 = -2.94d-5
    c3 = -4.43d-7
    wf=w*100.
    act = 1+c1*wf+c2*wf**2+c3*wf**3
    acs = act/xs1        

    gam1 = acs*vl1
    gam2 = vl2

  end subroutine


!----------------------------------------------------------------------------
!----------------------------------------------------------------------------
! 
!  Calculates the collision rate (m^3/s) of condensing 
!	vapour molecules with particles in each size section
!	
!	(for the 'unconventional' way to describe free-molecular condensation
!	 see Lehtinen & Kulmala: A model for particle formation and growth
!			in the atmosphere with molecular resolution in size;
!			Atmos. Chem. Phys. Discuss., 2, 1-17, 2002)
!				
!---------------------------------------------------------------------------

  function collision_rate(jj)	

	implicit none

	integer, intent(in) :: jj							! the 'number' of condensing vapour
	real*8, dimension(sections) :: collision_rate

	integer :: ii
	real*8 :: air_free_path, dif_vap, r_vap, n1, viscosity, r_hyd, r_h2o
	real*8, dimension(sections) :: knudsen, corr, dif_part, mean_path, rate 

	! air mean free path (m) and viscosity (kg s^-1 m^-1) (where from?)
	air_free_path = (6.73d-8*temp*(1.+110.4/temp))/(296.*pres/pstand*1.373)	
	viscosity = (1.832d-5*406.4*temp**1.5)/(5093*(temp+110.4))

	r_h2o = (3*molec_h2o/1000./(4*pi))**(1./3.)	! radius of water molecule
	r_vap = (molecvol(jj)*3./(4.*pi))**(1./3.)	! radius of condensable molecules (m)

	! for particles in each size section
	knudsen = air_free_path/radius							! particle Knudsen number
	corr = 1. + knudsen*(1.142+0.558*exp(-0.999/knudsen))	! Cunninghan correction factor (Allen and Raabe, Aerosol Sci. Tech. 4, 269)

	dif_part = boltzmann*temp*corr/(6*pi*viscosity*radius)	! particle diffusion coefficient (m^2/s)

	! molecular diffusion coefficient (m^2/s) for condensing vapour and
	! mean free path of condensation process (m)
	dif_vap = diffusion(molarmass(jj), diff_vol(jj), zero)
	mean_path = 3*(dif_vap + dif_part)/sqrt(8*boltzmann*temp/pi*(1./molecmass(jj) + 1./mass))

	knudsen = mean_path/(radius + r_vap)
	corr = knudsen + 1.			! transitional correction factor (Fuchs and Sutugin, 1971)
	corr = corr/(0.377*knudsen + 1 + 4./(3.*alpha(jj))*(knudsen+knudsen**2))

!condensation_sink = 2.*pi*dif_vap*(corr*2.*radius*n_conc)
!condensation_sink_integral = sum(condensation_sink)

	! no hydrates
	if (jj .ne. 1) then
		collision_rate = 4*pi*(radius + r_vap)*(dif_vap + dif_part)*corr		! (m^3/s)
		goto 100
	end if

	! only for h2so4 (if hydrates taken into account) 
	call hydrates()

	collision_rate = (radius+r_vap)*(dif_vap/hydpw + dif_part)*corr	! (m^3/s)

	! hydrate correction
	do ii = 1, hydmax
		n1 = real(ii)
		dif_vap = diffusion(molarmass(jj), diff_vol(jj), n1)
		mean_path = 3.*(dif_vap+dif_part)
		mean_path = mean_path/sqrt((8*boltzmann*temp)/pi*(1./(molecmass(jj)+n1*molec_h2o) + 1./mass))
		r_hyd = (r_vap**3+n1*r_h2o**3)**(1./3.)
		knudsen = mean_path/(radius + r_hyd)
		corr = knudsen + 1.	
		corr = corr/(0.377*knudsen + 1 + 4./(3.*alpha(jj))*(knudsen+knudsen**2))
		collision_rate = collision_rate + corr*(dif_vap*rhoh(ii) + dif_part)*(radius+r_hyd) 
	end do

	collision_rate = 4.*pi*collision_rate !(m^3/s)

100 continue 

  end function collision_rate


!-----------------------------------------------------------------
! 
!  Evaluates molecular diffusion coefficient (m^2/s) 
!	for condensing vapour (see Reid et al. Properties of 
!		Gases and Liquids (1987))
!
!-----------------------------------------------------------------

  function diffusion(molar_gas, d_gas, hydrates)

	implicit none

	real*8, intent(in) :: molar_gas, d_gas, hydrates
	real*8 :: d_air, d_h2o, diffusion

	d_air = 19.70	! diffusion volume of air (???)
	d_h2o = 13.10	!		- " -		water

	diffusion = 1.d-7*temp**1.75*sqrt(1./molar_air + 1./(molar_gas+hydrates*molar_h2o))
	diffusion = diffusion/( (pres/pstand)*(d_air**(1./3.)+(d_gas+hydrates*d_h2o)**(1./3.))**2 )

  end function diffusion


!--------------------------------------------------------------------------------
!
!  Evaluates the nucleation rate, and the size and composition of nucleated
!	particles
!
!  Only the sulphuric acid portion of the newly formed particle is put into
!	the core volumes; water and ammonia equilibrated in equilibrium_size()
!
!	n_coeff		=	nucleation coefficient (used in activation/kinetic nucleation)
!	n_nuc		=	particle concentration change due to condensation 
!					in each section (#/m^3 s)
!	vol_nuc		=	volume concentration change for each component
!					in each section (um^3/m^3 s)
!	nuc_rate	=	nucleation rate (#/m^3 s)
!	n_sa		=	number of sulphuric acid molecules in a critical cluster
!
!--------------------------------------------------------------------------------

  subroutine nucleation(n_coeff, n_nuc, vol_nuc, nuc_rate, n_sa)

	implicit none

	real*8, intent(in) :: n_coeff
	real*8, dimension(sections), intent(out) :: n_nuc
	real*8, dimension(sections, compo), intent(out) :: vol_nuc
	real*8, intent(out) :: nuc_rate, n_sa
	
	integer :: ii
	real*8 :: r_crit, n_h2o, n_nh3, core_nuc, aa


	n_nuc = 0.		! number concentration net change rate (#/m^3 s)
	vol_nuc = 0.		! volume concentration net change rate (um^3/m^3 s)

	select case(nuc_approach)
	
	case('act')  ! activation nucleation

		if (n_coeff < 1.0e-8 .or. n_coeff > 1.0e-4) &
			stop 'The value of activation nucleation coefficient not in correct range'

		call act_nucleation(n_coeff, nuc_rate, n_sa)
	
	case('kin')  ! kinetic nucleation with a determined coefficient
		
		if (vap_conc(1) > nh3_mix*2.4d13) &
			stop ' [NH3] < [H2SO4]: kinetic nucleation assumptions not valid'
		if (n_coeff < 1.0e-14 .or. n_coeff > 1.0e-10) &
			stop 'The value of kinetic nucleation coefficient not in correct range'
		
		call kin_nucleation(n_coeff, nuc_rate, n_sa)
	
	case('bar')  ! barrierless kinetic nucleation
	
	!	if (vap_conc(1) > nh3_mix*2.4d13) &
		!	stop ' [NH3] < [H2SO4]: kinetic nucleation assumptions not valid'

		call barless_nucleation(nuc_rate, n_sa)

	case default  ! homogeneous nucleation

		if (vap_conc(1) < 1.d10) then	! validity of parameterizations
			nuc_rate = 0.
			n_sa = 0.
			return
		else if (nh3_mix < 10000) then	! if no ammonia present: binary scheme
			call bin_nucleation(vap_conc(1),nuc_rate,n_h2o,n_sa,r_crit)
			n_nh3 = 0.
		else 				! if ammonia present: ternary scheme
			call ter_nucleation(vap_conc(1),nh3_mix,nuc_rate,n_h2o,n_sa,n_nh3,r_crit)
		end if
	
	end select
	
	core_nuc = n_sa*molecvol(1)	! volume of sulphuric acid core of nucleated particles

	! find the bins to which the new particle is divided
	if (core_nuc <= core(1)) then
		n_nuc(1) = nuc_rate*core_nuc/core(1)		! here we conserve number, volume overestimated
		vol_nuc(1,1) = core_nuc*nuc_rate
	else
		do ii = 1,sections-1
			if (core_nuc > core(ii) .and. core_nuc <= core(ii+1)) then
				aa = (core(ii+1)-core_nuc)/(core(ii+1)-core(ii))
				n_nuc(ii) = aa*nuc_rate
				n_nuc(ii+1) = (1-aa)*nuc_rate
				vol_nuc(ii,1) = aa*core(ii)*nuc_rate
				vol_nuc(ii+1,1) = (1-aa)*core(ii+1)*nuc_rate
				goto 100
			end if
		end do
	end if

	if (core_nuc > core(sections)) then
		n_nuc(sections) = nuc_rate*core_nuc/core(sections)	! here we conserve number, volume underestimated
		vol_nuc(sections,1) = nuc_rate*core_nuc
	end if

100	continue

	vol_nuc = vol_nuc*scale**3	! (um^3/m^3)

  end subroutine nucleation

  
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
! Subroutine ACT_NUCLEATION evaluates the "activation" nucleation rate
! for sulphuric acid clusters using activation coefficient given as input parameter.
!
!        act_coeff  =   activation coefficient (1/s), given as an input parameter
!        nuc_rate   =   nucleation rate (#/m^3 s)
!        n_sa       =   number of sulphuric acid molecules in a critical cluster
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine act_nucleation(act_coeff, nuc_rate, n_sa)

        implicit none

	real*8, intent(in) :: act_coeff
        real*8, intent(out) :: nuc_rate, n_sa

        nuc_rate = act_coeff*vap_conc(1)	! [#/m^3 s]
        n_sa = 1.0

  end subroutine act_nucleation
  
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
! Subroutine KIN_NUCLEATION evaluates the "kinetic type" nucleation rate
! for ammonium bisulphate (NH4HSO4) molecules using kinetic coefficient given as input parameter.
!
!        kin_coeff  =   kinetic collision frequency (cm^3/s), given as an input parameter
!        nuc_rate   =   nucleation rate (#/m^3 s)
!        n_sa       =   number of sulphuric acid molecules in a critical cluster
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine kin_nucleation(kin_coeff, nuc_rate, n_sa)

        implicit none

	real*8, intent(in) :: kin_coeff
        real*8, intent(out) :: nuc_rate, n_sa

        nuc_rate = kin_coeff*1e-6*(vap_conc(1))**2	! [#/m^3 s]
        n_sa = 2.0

  end subroutine kin_nucleation
  
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
! Subroutine BARLESS_NUCLEATION evaluates the barrierless kinetic nucleation rate
! for ammonium bisulphate molecules (NH4HSO4 + NH4HSO4 -> 2 NH4HSO4).
!
!        nuc_rate   =   nucleation rate (#/m^3 s)
!        n_sa       =   number of sulphuric acid molecules in a critical cluster
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  subroutine barless_nucleation(nuc_rate, n_sa)

        implicit none
	
        real*8, intent(out) :: nuc_rate, n_sa

        real*8 :: r_init, dens_init, coag_coeff
        
	r_init = 0.315d-9 ! radius (m) of the initial NH4HSO4 molecule
        dens_init = 1465. ! density (kg/m^3) of the initial NH4HSO4 molecule

	! collision frequency from kinetic theory
        coag_coeff = 8.*sqrt(3.*boltzmann*temp*r_init/dens_init)

        nuc_rate = 0.5*coag_coeff*(vap_conc(1))**2	! [#/m^3 s]
        n_sa = 2.0

  end subroutine barless_nucleation


!--------------------------------------------------------------------------------
!
!  Calculates ternary nucleation rate and properties of critical clusters
!	using revised theory, Noppel hydrate correction														
!						
!	(see Napari et al., An improved model for ternary nucleation of
!			sulfuric acid - ammoni- water, J. Chem. Phys., 116, 4221-4227 (2002);
!	     Napari et al., Parameterization of ternary nucleation rates for
!			H2SO4 - NH3 - H2O vapors, J. Geophys. Res., 107(D19), AAC 6-1 (2002))
!																																																						
!	sa_conc		=	sulphuric acid concentration  (m^-3) (10^10-10^15 m^-3)	
!	c_nh3		=	ammonia mixing ratio (ppt)	(0.1-100 ppt)
!	nuc_rate	=	nucleation rate (m^-3 s^-1)	
!	n_h2o		=	number of water molecules in a critical cluster	
!	n_sa		=		- " -	sulphuric acid		- " -  
!	n_nh3		=		- " -	ammonia				- " -
!	r_crit		=	radius of the critical cluster (m)
!
!	(c) Ismo Napari, 2002	
!
!		Division of Atmospheric Sciences
!		Department of Physical Sciences
!		P.O. Box 64
!		FIN-00014 University of Helsinki
!		Finland
!
!		(ismo.napari@helsinki.fi)
!
!--------------------------------------------------------------------------------


  subroutine ter_nucleation(sa_conc,c_nh3,nuc_rate,n_h2o,n_sa,n_nh3,r_crit)     

	implicit none

	real*8, intent(in) :: sa_conc, c_nh3
	real*8, intent(out) :: nuc_rate, n_h2o, n_sa, n_nh3, r_crit

	real*8 :: c_sa, lnj, ntot

	c_sa = sa_conc*1.d-6	! (cm^-3)

	! validity of parameterization : DO NOT REMOVE!
	if(temp < 240.) stop '  INVALID INPUT VALUE (ter. nucleation): temperature < 240 K'
	if(temp > 300.) stop '  INVALID INPUT VALUE (ter. nucleation) temperature > 300 K'
	if(rh < 0.05) stop '  INVALID INPUT VALUE (ter. nucleation) relative humidity < 5 %'
	if(rh > 0.95) stop '  INVALID INPUT VALUE (ter. nucleation) relative humidity > 95 %'
	if(c_sa < 1.e4) stop '  INVALID INPUT VALUE (ter. nucleation) H2SO4 concentration < 10^4 1/cm3'
	if(c_sa > 1.e9) stop '  INVALID INPUT VALUE (ter. nucleation) H2SO4 concentration > 10^9 1/cm3'
	if(c_nh3 < 0.1) stop '  INVALID INPUT VALUE (ter. nucleation) ammonia mixing ratio < 0.1 ppt'
	if(c_nh3 > 100.) stop '  INVALID INPUT VALUE (ter. nucleation) ammonia mixing ratio > 100 ppt'

	lnj = -84.7551114741543 + 0.3117595133628944*rh + 1.640089605712946*rh*temp - &
		0.003438516933381083*rh*temp**2 - 0.00001097530402419113*rh*temp**3 - &
		0.3552967070274677/Log(c_sa) - (0.06651397829765026*rh)/Log(c_sa) - &
		(33.84493989762471*temp)/Log(c_sa) - (7.823815852128623*rh*temp)/Log(c_sa) + &
		(0.3453602302090915*temp**2)/Log(c_sa) + &
        (0.01229375748100015*rh*temp**2)/Log(c_sa) - &
        (0.000824007160514956*temp**3)/Log(c_sa) + &
        (0.00006185539100670249*rh*temp**3)/Log(c_sa) +& 
        3.137345238574998*Log(c_sa) + 3.680240980277051*rh*Log(c_sa) - &
        0.7728606202085936*temp*Log(c_sa) - 0.204098217156962*rh*temp*Log(c_sa) + &
        0.005612037586790018*temp**2*Log(c_sa) + &
        0.001062588391907444*rh*temp**2*Log(c_sa) - &
        9.74575691760229e-6*temp**3*Log(c_sa) - &
        1.265595265137352e-6*rh*temp**3*Log(c_sa) + 19.03593713032114*Log(c_sa)**2 - &
        0.1709570721236754*temp*Log(c_sa)**2 + &
        0.000479808018162089*temp**2*Log(c_sa)**2 - &
        4.146989369117246e-7*temp**3*Log(c_sa)**2 + 1.076046750412183*Log(c_nh3) + &
        0.6587399318567337*rh*Log(c_nh3) + 1.48932164750748*temp*Log(c_nh3) +& 
        0.1905424394695381*rh*temp*Log(c_nh3) - 0.007960522921316015*temp**2*Log(c_nh3) - &
        0.001657184248661241*rh*temp**2*Log(c_nh3) + &
        7.612287245047392e-6*temp**3*Log(c_nh3) + &
        3.417436525881869e-6*rh*temp**3*Log(c_nh3) + & 
        (0.1655358260404061*Log(c_nh3))/Log(c_sa) + & 
        (0.05301667612522116*rh*Log(c_nh3))/Log(c_sa) + &
        (3.26622914116752*temp*Log(c_nh3))/Log(c_sa) - &
        (1.988145079742164*rh*temp*Log(c_nh3))/Log(c_sa) - &
        (0.04897027401984064*temp**2*Log(c_nh3))/Log(c_sa) + &
        (0.01578269253599732*rh*temp**2*Log(c_nh3))/Log(c_sa) + &
        (0.0001469672236351303*temp**3*Log(c_nh3))/Log(c_sa) - &
        (0.00002935642836387197*rh*temp**3*Log(c_nh3))/Log(c_sa) + &
        6.526451177887659*Log(c_sa)*Log(c_nh3) - & 
        0.2580021816722099*temp*Log(c_sa)*Log(c_nh3) + &
        0.001434563104474292*temp**2*Log(c_sa)*Log(c_nh3) - &
        2.020361939304473e-6*temp**3*Log(c_sa)*Log(c_nh3) - &
        0.160335824596627*Log(c_sa)**2*Log(c_nh3) + &
        0.00889880721460806*temp*Log(c_sa)**2*Log(c_nh3) - &
        0.00005395139051155007*temp**2*Log(c_sa)**2*Log(c_nh3) + &
        8.39521718689596e-8*temp**3*Log(c_sa)**2*Log(c_nh3) + &
        6.091597586754857*Log(c_nh3)**2 + 8.5786763679309*rh*Log(c_nh3)**2 - &
        1.253783854872055*temp*Log(c_nh3)**2 - 0.1123577232346848*rh*temp*Log(c_nh3)**2 + &
        0.00939835595219825*temp**2*Log(c_nh3)**2 + &
        0.0004726256283031513*rh*temp**2*Log(c_nh3)**2 - &
        0.00001749269360523252*temp**3*Log(c_nh3)**2 - &
        6.483647863710339e-7*rh*temp**3*Log(c_nh3)**2 + &
        (0.7284285726576598*Log(c_nh3)**2)/Log(c_sa) + &
        (3.647355600846383*temp*Log(c_nh3)**2)/Log(c_sa) - &
        (0.02742195276078021*temp**2*Log(c_nh3)**2)/Log(c_sa) + &
        (0.00004934777934047135*temp**3*Log(c_nh3)**2)/Log(c_sa) + &
        41.30162491567873*Log(c_sa)*Log(c_nh3)**2 - &
        0.357520416800604*temp*Log(c_sa)*Log(c_nh3)**2 + &
        0.000904383005178356*temp**2*Log(c_sa)*Log(c_nh3)**2 - &
        5.737876676408978e-7*temp**3*Log(c_sa)*Log(c_nh3)**2 - &
        2.327363918851818*Log(c_sa)**2*Log(c_nh3)**2 + &
        0.02346464261919324*temp*Log(c_sa)**2*Log(c_nh3)**2 - &
        0.000076518969516405*temp**2*Log(c_sa)**2*Log(c_nh3)**2 + &
        8.04589834836395e-8*temp**3*Log(c_sa)**2*Log(c_nh3)**2 - &
        0.02007379204248076*Log(rh) - 0.7521152446208771*temp*Log(rh) + &
        0.005258130151226247*temp**2*Log(rh) - &
        8.98037634284419e-6*temp**3*Log(rh) + &
        (0.05993213079516759*Log(rh))/Log(c_sa) + &
        (5.964746463184173*temp*Log(rh))/Log(c_sa) - &
        (0.03624322255690942*temp**2*Log(rh))/Log(c_sa) + &
        (0.00004933369382462509*temp**3*Log(rh))/Log(c_sa) - &
        0.7327310805365114*Log(c_nh3)*Log(rh) - &
        0.01841792282958795*temp*Log(c_nh3)*Log(rh) + &
        0.0001471855981005184*temp**2*Log(c_nh3)*Log(rh) - &
        2.377113195631848e-7*temp**3*Log(c_nh3)*Log(rh)

	nuc_rate = exp(lnj)	! (cm^-3 s^-1)

	if(nuc_rate < 1.d-5) then	! validity of parametrization
		nuc_rate = 0.
		n_h2o = 0.
		n_sa = 0.
		n_nh3 = 0.
		r_crit = 0.5d-9
		goto 100
	end if

	! validity of parametrization
	if(nuc_rate > 1.e6) stop '  INVALID OUTPUT VALUE for ternary nucleation: nucleation rate > 10^6 1/cm3s'

	ntot = 79.34842673023872 + 1.738399264246183*lnj + &
        0.007114029786143758*lnj**2 - 0.7449934047779641*temp - &
        0.00820608264651173*lnj*temp + 0.001785497681201935*temp**2

	n_sa = 38.16448247950508 + 0.7741058259731187*lnj + &
        0.002988789927230632*lnj**2 - 0.3576046920535017*temp - &
        0.003663583011953248*lnj*temp + 0.000855300153372776*temp**2

	n_nh3 = 26.89821829677973 + 0.6829050502793024*lnj + &
		0.003575211441884629*lnj**2 - 0.2657482507363107*temp - &
		0.003418950836780133*lnj*temp + 0.000673454337882302*temp**2

	n_h2o = ntot-n_sa-n_nh3

	r_crit = 0.1410271086638381 - 0.001226253898894878*lnj - &
		7.822111731550752e-6*lnj**2 - 0.001567273351921166*temp - &
        0.00003075996088273962*lnj*temp + 0.00001083754117202233*temp**2

    nuc_rate = nuc_rate*1.d6	! (m^-3 s^-1)
	r_crit = r_crit*1.d-9		! (m)

100	continue

  end subroutine ter_nucleation


!-------------------------------------------------------------------------------
!
!  Calculates binary nucleation rate and properties of critical cluster
!	using revised theory, Stauffer + Binder & Stauffer kinetics and 
!	Noppel hydrate correction
!
!	sa_conc		=	sulphuric acid concentration (m^-3) 
!	nuc_rate	=	nucleation rate (m^-3 s^-1) 
!	n_h2o		=	number of water molecules in a critical cluster
!	n_sa		=	- " -  sulphuric acid		- " -
!	r_crit		=	radius of the critical cluster core (m)
!
! (c) Hanna Vehkamki, 2002
!
!		Division of Atmospheric Sciences
!		Department of Physical Sciences
!		P.O. Box 64
!		FIN-00014 University of Helsinki
!		Finland
!
!		(hanna.vehkamaki@helsinki.fi)
!
!-------------------------------------------------------------------------------

  subroutine bin_nucleation(sa_conc,nuc_rate,n_h2o,n_sa,r_crit)

	implicit none

	real*8, intent(in) :: sa_conc
	real*8, intent(out) :: nuc_rate, n_h2o, n_sa, r_crit

	real*8 :: c_sa, x, ntot


	c_sa = sa_conc*1.d-6 !(cm^-3)

	! validity of parametrization : DO NOT REMOVE!
	if(temp < 190.15) stop '  INVALID INPUT VALUE (bin. nucleation): temperature < 190.15 K'
	if(temp > 300.15) stop '  INVALID INPUT VALUE (bin. nucleation): temperature > 300.15 K'
	if(rh < 0.0001) stop '  INVALID INPUT VALUE (bin. nucleation): relative humidity < 0.01 %'
	if(rh > 1.) stop '  INVALID INPUT VALUE (bin. nucleation): relative humidity > 100 %'
	if(c_sa < 1.e4) stop '  INVALID INPUT VALUE (bin. nucleation): H2SO4 concentration < 10^4 1/cm3'
	if(c_sa > 1.e11) stop '  INVALID INPUT VALUE (bin. nucleation): H2SO4 concentration > 10^11 1/cm3'

	x = 0.7409967177282139 - 0.002663785665140117*temp + 0.002010478847383187*Log(rh) - &
		0.0001832894131464668*temp*Log(rh) + 0.001574072538464286*Log(rh)**2 -    &  
		0.00001790589121766952*temp*Log(rh)**2 + 0.0001844027436573778*Log(rh)**3 - & 
		1.503452308794887e-6*temp*Log(rh)**3 - 0.003499978417957668*Log(c_sa) + &
		0.0000504021689382576*temp*Log(c_sa)

	nuc_rate = 0.1430901615568665 + 2.219563673425199*temp - 0.02739106114964264*temp**2 +  &
		0.00007228107239317088*temp**3 + 5.91822263375044/x +  &
		0.1174886643003278*Log(rh) + 0.4625315047693772*temp*Log(rh) -  &
		0.01180591129059253*temp**2*Log(rh) +  &
		0.0000404196487152575*temp**3*Log(rh) + (15.79628615047088*Log(rh))/x -  &
		0.215553951893509*Log(rh)**2 - 0.0810269192332194*temp*Log(rh)**2 +  &
		0.001435808434184642*temp**2*Log(rh)**2 -  &
		4.775796947178588e-6*temp**3*Log(rh)**2 -  &
		(2.912974063702185*Log(rh)**2)/x - 3.588557942822751*Log(rh)**3 +  &
		0.04950795302831703*temp*Log(rh)**3 -  &
		0.0002138195118737068*temp**2*Log(rh)**3 +  &
		3.108005107949533e-7*temp**3*Log(rh)**3 -  &
		(0.02933332747098296*Log(rh)**3)/x +  &
		1.145983818561277*Log(c_sa) -  &
		0.6007956227856778*temp*Log(c_sa) +  &
		0.00864244733283759*temp**2*Log(c_sa) -  &
		0.00002289467254710888*temp**3*Log(c_sa) -  &
		(8.44984513869014*Log(c_sa))/x +  &
		2.158548369286559*Log(rh)*Log(c_sa) +  &
		0.0808121412840917*temp*Log(rh)*Log(c_sa) -  &
		0.0004073815255395214*temp**2*Log(rh)*Log(c_sa) -  &
		4.019572560156515e-7*temp**3*Log(rh)*Log(c_sa) +  &
		(0.7213255852557236*Log(rh)*Log(c_sa))/x +  &
		1.62409850488771*Log(rh)**2*Log(c_sa) -  &
		0.01601062035325362*temp*Log(rh)**2*Log(c_sa) +  &
		0.00003771238979714162*temp**2*Log(rh)**2*Log(c_sa) +  &
		3.217942606371182e-8*temp**3*Log(rh)**2*Log(c_sa) -  &
		(0.01132550810022116*Log(rh)**2*Log(c_sa))/x +  &
		9.71681713056504*Log(c_sa)**2 -  &
		0.1150478558347306*temp*Log(c_sa)**2 +  &
		0.0001570982486038294*temp**2*Log(c_sa)**2 +  &
		4.009144680125015e-7*temp**3*Log(c_sa)**2 +  &
		(0.7118597859976135*Log(c_sa)**2)/x -  &
		1.056105824379897*Log(rh)*Log(c_sa)**2 +  &
		0.00903377584628419*temp*Log(rh)*Log(c_sa)**2 -  &
		0.00001984167387090606*temp**2*Log(rh)*Log(c_sa)**2 +  &
		2.460478196482179e-8*temp**3*Log(rh)*Log(c_sa)**2 -  &
		(0.05790872906645181*Log(rh)*Log(c_sa)**2)/x -  &
		0.1487119673397459*Log(c_sa)**3 +  &
		0.002835082097822667*temp*Log(c_sa)**3 -  &
		9.24618825471694e-6*temp**2*Log(c_sa)**3 +  &
		5.004267665960894e-9*temp**3*Log(c_sa)**3 -  &
		(0.01270805101481648*Log(c_sa)**3)/x

	nuc_rate = exp(nuc_rate)	! (cm^-3 s^-1)

	if(nuc_rate < 1.d-7) then	! validity of parameterization
		nuc_rate = 0.
		n_h2o = 0.
		n_sa = 0.
		r_crit = 0.5d-9
		goto 100
	end if

	if (nuc_rate > 1.d10) stop '  INVALID OUTPUT VALUE (bin. nucleation): nucl. rate > 10^16 1/m^3s'

	ntot = -0.002954125078716302 - 0.0976834264241286*temp + &
		0.001024847927067835*temp**2 - 2.186459697726116e-6*temp**3 -  &
		0.1017165718716887/x - 0.002050640345231486*Log(rh) - &
		0.007585041382707174*temp*Log(rh) +  &
		0.0001926539658089536*temp**2*Log(rh) - 6.70429719683894e-7*temp**3*Log(rh) -  &
        (0.2557744774673163*Log(rh))/x + 0.003223076552477191*Log(rh)**2 + &
		0.000852636632240633*temp*Log(rh)**2 -  &
        0.00001547571354871789*temp**2*Log(rh)**2 + &
		5.666608424980593e-8*temp**3*Log(rh)**2 +  &
        (0.03384437400744206*Log(rh)**2)/x + 0.04743226764572505*Log(rh)**3 -  &
        0.0006251042204583412*temp*Log(rh)**3 + 2.650663328519478e-6*temp**2*Log(rh)**3 -  &
        3.674710848763778e-9*temp**3*Log(rh)**3 - (0.0002672510825259393*Log(rh)**3)/x -  &
        0.01252108546759328*Log(c_sa) + 0.005806550506277202*temp*Log(c_sa) -  &
        0.0001016735312443444*temp**2*Log(c_sa) + 2.881946187214505e-7*temp**3*Log(c_sa) + &
        (0.0942243379396279*Log(c_sa))/x - 0.0385459592773097*Log(rh)*Log(c_sa) -  &
        0.0006723156277391984*temp*Log(rh)*Log(c_sa) + &
		2.602884877659698e-6*temp**2*Log(rh)*Log(c_sa) +  &
        1.194163699688297e-8*temp**3*Log(rh)*Log(c_sa) - &
		(0.00851515345806281*Log(rh)*Log(c_sa))/x -  &
        0.01837488495738111*Log(rh)**2*Log(c_sa) + &
		0.0001720723574407498*temp*Log(rh)**2*Log(c_sa) -  &
        3.717657974086814e-7*temp**2*Log(rh)**2*Log(c_sa) -  &
        5.148746022615196e-10*temp**3*Log(rh)**2*Log(c_sa) +  &
        (0.0002686602132926594*Log(rh)**2*Log(c_sa))/x - 0.06199739728812199*Log(c_sa)**2 +  &
        0.000906958053583576*temp*Log(c_sa)**2 - 9.11727926129757e-7*temp**2*Log(c_sa)**2 -  &
        5.367963396508457e-9*temp**3*Log(c_sa)**2 - (0.007742343393937707*Log(c_sa)**2)/x +  &
        0.0121827103101659*Log(rh)*Log(c_sa)**2 - &
		0.0001066499571188091*temp*Log(rh)*Log(c_sa)**2 +  &
        2.534598655067518e-7*temp**2*Log(rh)*Log(c_sa)**2 -  &
        3.635186504599571e-10*temp**3*Log(rh)*Log(c_sa)**2 +  &
        (0.0006100650851863252*Log(rh)*Log(c_sa)**2)/x + &
		0.0003201836700403512*Log(c_sa)**3 -  &
        0.0000174761713262546*temp*Log(c_sa)**3 + &
		6.065037668052182e-8*temp**2*Log(c_sa)**3 -  &
        1.421771723004557e-11*temp**3*Log(c_sa)**3 + &
		(0.0001357509859501723*Log(c_sa)**3)/x
     
	ntot=exp(ntot)
	if (ntot < 4.) stop '  INVALID OUTPUT VALUE (bin. nucleation): ntot < 4'

	n_sa = x*ntot
	n_h2o = (1.-x)*ntot

	r_crit = exp(-1.6524245+0.42316402*x+0.33466487*log(ntot)) ! (nm)
	r_crit = r_crit*1.e-9		! (m)

	nuc_rate = nuc_rate*1.d6	! (m^-3 s^-1)
	
	return

100 continue

  end subroutine bin_nucleation


!oooooooooooooooooooooooooooooooooooooDEPOSITIONoooooooooooooooooooooooooooooooooooooooo!
!  Daniel Partridge ITM																	!
!																						!
!  Calculates dry deposition velocities for aerosol -> loss of number and volume		!
!  concentration due to dry deposition													!
!																						!
!																						!
!																						!
!																						!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo!
	subroutine  dry_deposition_aerosols(n_depo, vol_depo, dry_dep_1)

    implicit none
    integer, parameter :: LUC = 17
	integer :: ii,nLUCchosen, kk

	real*8 :: dry_dep_1

	real*8, dimension(sections) :: n_depo
	real*8, dimension(sections, compo) :: vol_depo

	
	real*8 :: nLUC(LUC)=0.0, E_Constant_Alpha(LUC)=0.0, E_Constant_Beta(LUC)=0.0, Z0m(LUC)=0.0, LAI(LUC)=0.0, Rac0(LUC)=0.0, Charac_Radius(LUC)=0.0 
	real*8 :: Vd(LUC)=0.0
  real*8 :: Hminusd(LUC)=0.0, Zminusd(LUC)=0.0, Ustar_Zhang2003_DAY(LUC)=0.0,Ustar_Zhang2003_NIGHT(LUC)=0.0
	real*8 :: z02, g, kinematic_air_viscosity, charnock_constant, d, Vg, Vgh, Ab, Kn, Gcunn, Bd, Sc, Vis, Eb, Eim, k, Rb_water, Rb1, Rb2
	real*8 :: air_mean_path, air_viscosity, air_density,  Uref, diameter
	real*8 :: St_water, Eb_Water, Eim_Water, Eb_Veg, small_collector_width, large_collector_width
	real*8 :: a_constant, b_constant, fitting_constant1, fitting_constant2, e0, St_Veg, Eim_Veg, F, Ein, roff, E, Uh, Rs
	real*8 :: Eb_Nonveg, Rb_Nonveg, Eim_Urban, Eim_Nonveg, Rb_Urban, St_Nonveg
	real*8 :: set_vel,rb_ex, rb_new, diff_part, schmidt, dens, radie_98, diameter_98, S, A, B, Bdh

	real*8 :: wind_velocity_grad, L_energy, L_momentum, Beta_energy, Beta_momentum
	real*8 :: Gamma_energy, Gamma_momentum, Gamma_momentum_function_Zr, Gamma_energy_function_Zr
	real*8 ::	Gm=0.0, Gh=0.0, Ustar_Bulk=0.0, Thetastar_Bulk=0.0
	real*8 :: Pr, Prt, sigma_h_integral, Rs_lowveg
	real*8 :: Z0h=0.0, Gamma_momentum_function_Z0m=0.0, sigma_m_integral=0.0, Gamma_energy_function_Z0h=0.0, sigma_e_integral=0.0
	real*8 :: Gamma_momentum_function_Hminusd, sigma_m_integral_Hminusd,radie,test, temp_75m, press_75


	real*8 :: uv_r_grad, theta_grad, uv_r, u10m_r, RI_bulk, Theta_diff, pot_temp_intep_surf, u_comp_intep_75, v_comp_intep_75, L_bulk, &
					theta_av, sigma_m_integral_bulk, downward_sw, temp_2m, percent_total_cc, accum_precip, rainfall_rate, surface_pressure, temp850, &
					sigma_e_integral_bulk

	integer :: i=0



	open(unit=15,file='Dep-Gas_CanopyHeight.dat')
	do i = 1,LUC
	read(15,*) Hminusd(i), Zminusd(i)
	enddo
	close(15)

	open(unit=16,file='Dep_Aerosol_Summer.dat')
	open(unit=17,file='Dep_Aerosol_Autumn.dat')
	open(unit=18,file='Dep_Aerosol_Winter.dat')
	open(unit=19,file='Dep_Aerosol_Spring.dat')

	do i = 1,LUC
	if ((month.eq.6) .or. (month.eq.7) .or. (month.eq.8))   then
	read(16,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
	elseif ((month.eq.9) .or. (month.eq.10) .or. (month.eq.11))   then
	read(17,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
    elseif ((month.eq.12) .or. (month.eq.1) .or. (month.eq.2))   then
	read(18,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
    elseif ((month.eq.3) .or. (month.eq.4) .or. (month.eq.5))   then
	read(19,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
	endif
	enddo
	close(16)
	close(17)
	close(18)
	close(19)

!FOR BATCH FILE

	open(unit=20,file='Ustar_Zhang2003.dat')
	do i = 1,LUC
	read(20,*) Ustar_Zhang2003_DAY(i), Ustar_Zhang2003_NIGHT(i)
	enddo
	close(20)

!FIX replace by cluster_dom_class
!nLUCchosen=dom_class
nLUCchosen=dom_class

!************************TEST***************************
!	if (nLUCchosen.gt.1) then
!	dom_class=8
!	endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!   dom_class=2
!cluster_dom_class=1
!nLUCchosen=dom_class

	n_depo=0
	vol_depo=0
	
	
	do kk = 1,sections

if (n_conc(kk)<n_limit) goto 3
radie=radius(kk)
dens=mass(kk)/(4.*pi*radie**3./3.)
if (kk==35) then


end if

n_depo(kk) = -dry_dep_1/boundary*n_conc(kk)	![#/m^3 s]
vol_depo(kk,:) = -dry_dep_1/boundary*vol_conc(kk,:)	![um^3/m^3 s]
!depo_vel= -n_depo(kk)*boundary/n_conc(kk)

g=9.81

!Von Karman constant (assume value)
k = 0.4
!Prandtl number
Pr=0.72
!Turbulent Prandtl number
Prt = 0.97
charnock_constant = 0.016

wind_velocity_grad = UV_R_grad
potential_temp_grad= Theta_grad
Uref = abs(UV_R)
U10m_r=5.

!Uref needs to be at the same height as reference height

!FIX have at 75m using adiabatic lapse rate
temp_2m=temp+altitude*0.0065
press_75=pres
temp_75m=temp_2m+(6.7e-3*75)
	air_viscosity = (1.832d-5*406.4*temp_75m**1.5)/(5093*(temp_75m+110.4))
	air_density = 1.2929*273.15*Press_75/(temp_75m*pstand)	    
    kinematic_air_viscosity = air_viscosity/air_density
	air_mean_path = (6.73d-8*temp_75m*(1.+110.4/temp_75m))/(296.*Press_75/pstand*1.373)	
!***********************************************************************************	
!Z0m over water, follow Zhangs code
if (nLUCchosen.eq.1) then
Z0m(nLUCchosen) = 0.000002*(U10m_R**2.5)
endif


!Z0m take from look up table (Zhang et al, 2002)
!Page 531 Jacobson, use IF statements for different LUC
Z0h = Z0m(nLUCchosen)/7.3891  !(exp(2))

!Water
if (nLUCchosen.eq.1) then
Z0h = Z0m(nLUCchosen)/(exp(k*((13.6*(Pr**0.66667))-12)))
endif


!Non-iterative aerodynamic resistance- Jacobson

Theta_diff=-3
pot_temp_INTEP_Surf=288
u_comp_INTEP_75=10
v_comp_INTEP_75=5
UV_R=wind_speed
Uref=UV_R
Theta_Av=286.5
UV_R_grad=0.5
Theta_grad=0.2
wind_velocity_grad = 0.5
potential_temp_grad= 0.2

!Bulk Ri 
!Potential virtual temperature? Should be very close to potential temp over Sweden
!Assume Z0h close to the surface

Ri_Bulk= (g*(Theta_diff)*((Zminusd(nLUCchosen)-Z0m(nLUCchosen))**2))/((pot_temp_INTEP_Surf)*(u_comp_INTEP_75**2+v_comp_INTEP_75**2)*(Zminusd(nLUCchosen)-Z0h))

IF (Ri_Bulk .le. 0.0) then

Gm = 1 - (9.4*Ri_Bulk/(1+ (70*(k**2)*((abs(Ri_Bulk)*Zminusd(nLUCchosen)/Z0m(nLUCchosen))**0.5)/(Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))**2))))

Gh = 1 - (9.4*Ri_Bulk/(1+ (50*(k**2)*((abs(Ri_Bulk)*Zminusd(nLUCchosen)/Z0m(nLUCchosen))**0.5)/(Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))**2))))

else 

Gm = 1/((1+(4.7*Ri_Bulk))**2)

Gh = Gm

ENDIF

Ustar_Bulk = ((k*abs(UV_R))/log((Zminusd(nLUCchosen)/Z0m(nLUCchosen))))*(Gm**0.5)

IF ((Ustar_Bulk .lt. 0.1) .and. (nLUCchosen.gt.1)) then
Ustar_Bulk = 0.1
Endif

Thetastar_Bulk = ((k**2*abs(UV_R)*Theta_diff)/(Ustar_Bulk*Prt*(Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))**2)))*Gh

L_Bulk = (Ustar_Bulk**2*Theta_Av)/(k*g*Thetastar_Bulk)

!print*, 'Ri_Bulk'
!print*, Ri_Bulk

!print*, 'Gm'
!print*, Gm

!print*, 'Gh'
!print*, Gh

!print*, 'Ustar_Bulk'
!print*, Ustar_Bulk

!print*, 'Thetastar_Bulk'
!print*, Thetastar_Bulk

!print*, 'L_Bulk'
!print*, L_Bulk
!pause
!**************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Aerodynamic resistance
!Just used to get the module working and as a comparison.
!Taken from Lindroth, 1993. Zo values from Zhang et al, 2002.

!Z0m for water surfaces (follow approach by Garratt, 1999)
!If (nLUCchosen.eq.1) then
!Z0m(nLUCchosen) = (charnock_constant*(Ustar(nLUCchosen)**2))/(9.81+((0.11*kinematic_air_viscosity)/Ustar(nLUCchosen)))
!endif

!Ra = (Log(((Z-d)/Z0m(nLUCchosen))**2))/((k**2)*Uref)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Ra Aerodynamic resistance between a reference height and the quasi-laminar sub-layer 
!above the surface.
!Method: Rearranging and substituting Richardson formulae (Golder, 1972) into approach by
!Jacobson, 1999 for aerodynamic resistance based on the Monin-Obukhov length stability functions
!and therefore bypassing the need of iterations as Ustar is not initially used as a function of Z/L.
!Do not need to use Bulk Richardson approach if have gradient for both potential temperature and wind velocity.

!potential_temp_grad=-0.3
!wind_velocity_grad=1.118
Ri = (9.81/Temp)*(potential_temp_grad/(wind_velocity_grad**2))

!FIX
Ri=Ri_Bulk

!print*, 'Ri'
!print*, Ri

!Z/L = stability
Beta_energy = 7.8
Beta_momentum = 6.0  

!Upper limit of 0.2 !Check-Annica's book
IF (Ri .gt. 0.2) then
Ri = 0.2
ENDIF

!Stable !Upper limit of 0.2 !Check-Annica's book
IF (Ri .gt. 0.0) then
L_energy = Zminusd(nLUCchosen)/Ri
L_momentum = Zminusd(nLUCchosen)/Ri
!Unstable
ELSEIF (Ri .le. 0.0) then
L_energy = (Zminusd(nLUCchosen)*(1-(Beta_energy*Ri)))/Ri
L_momentum = (Zminusd(nLUCchosen)*(1-(Beta_momentum*Ri)))/Ri
ENDIF
 
!Integrals for sigma_m (momentum for calculation of Ustar) 
!Need to compare to lookup table for LUC for Ustar (Zhang et al, 2002) when complete to see if values reasonable. 
!Also need to remember to compare final Ra to method by Lindroth that does not consider stability.
!Still keeps values of Ra specific for each LUC as values of Z0m are obtained from a LUC lookup table.
!Need to compare with other similar approaches other than this (my own method) to check that final Ra results are reasonable.

!Also note: Gamma approximation
Gamma_energy = 11.6
Gamma_momentum = 19.3

!Stable
!If (Ri .gt. 0.0) then
If (L_Bulk .gt. 0.0) then
sigma_m_integral_Bulk = Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))+((Beta_momentum/L_Bulk)*(Zminusd(nLUCchosen)-Z0m(nLUCchosen)))
!Unstable
!elseif (Ri .lt. 0.0) then
elseif (L_Bulk .lt. 0.0) then
Gamma_momentum_function_Zr = (((1-((Gamma_momentum*Zminusd(nLUCchosen))/L_Bulk))**0.25)-1)/(((1-((Gamma_momentum*Zminusd(nLUCchosen))/L_Bulk))**0.25)+1)
Gamma_momentum_function_Z0m = (((1-((Gamma_momentum*Z0m(nLUCchosen))/L_Bulk))**0.25)-1)/(((1-((Gamma_momentum*Z0m(nLUCchosen))/L_Bulk))**0.25)+1)
sigma_m_integral_Bulk = (Log(Gamma_momentum_function_Zr))-(Log(Gamma_momentum_function_Z0m))+(2*atan((1-(Gamma_momentum*Zminusd(nLUCchosen)/L_Bulk))**0.25))-(2*atan((1-(Gamma_momentum*(Z0m(nLUCchosen)/L_Bulk)))**0.25))
!Neutral
!elseif (Ri .eq. 0.0) then !Make it less sensitive (i.e allow to vary slightly around 0)
elseif (L_Bulk .eq. 0.0) then
sigma_m_integral_Bulk = Log(Zminusd(nLUCchosen)/Z0m(nLUCchosen))
Endif

!print*, 'L_momentum'
!print*, L_momentum

!Could change back Ri=Ri_Bulk
!print*, 'L_energy'
!print*, L_energy


!print*, 'L_Bulk'
!print*, L_Bulk

!print*, 'sigma_m_integral'
!print*, sigma_m_integral_Bulk

!Friction Velocity: Ustar
Ustar_Bulk2 = k*Uref*(1/sigma_m_integral_Bulk)

!print*, 'Ustar_Bulk2'
!print*, Ustar_Bulk2


IF ((Ustar_Bulk2 .lt. 0.1) .and. (nLUCchosen.gt.1)) then
Ustar_Bulk2 = 0.1
Endif

!KEEP?
IF ((Ustar_Bulk2 .lt. 0.08) .and. (nLUCchosen.eq.1)) then
Ustar_Bulk2 = 0.08
Endif

!Have replaced Ri and L_energy by L_Bulk
!Integrals for sigma_h (energy for calculation of Ra) 
!Stable

!If (Ri .gt. 0.0) then
If (L_Bulk .gt. 0.0) then
sigma_e_integral_Bulk = (Prt*(Log(Zminusd(nLUCchosen)/Z0h)))+((Beta_energy/L_Bulk)*(Zminusd(nLUCchosen)-Z0h))

!Unstable
!elseif (Ri .lt. 0.0) then
elseif (L_Bulk .lt. 0.0) then
Gamma_energy_function_Zr = (((1-((Gamma_energy*Zminusd(nLUCchosen))/L_Bulk))**0.5)-1)/(((1-((Gamma_energy*Zminusd(nLUCchosen))/L_Bulk))**0.5)+1)
Gamma_energy_function_Z0h = (((1-((Gamma_energy*Z0h)/L_Bulk))**0.5)-1)/(((1-((Gamma_energy*Z0h)/L_Bulk))**0.5)+1)
sigma_e_integral_Bulk = Prt*((Log(Gamma_energy_function_Zr))-(Log(Gamma_energy_function_Z0h)))

!Neutral
!elseif (Ri .eq. 0.0) then
elseif (L_Bulk .eq. 0.0) then
sigma_e_integral_Bulk = Prt*(Log(Zminusd(nLUCchosen)/Z0h))

Endif






!Aerodynamic Resistance: Ra
!Ra = (1/(k*Ustar))*sigma_e_integral
Ra_Bulk = (1/(k*Ustar_Bulk2))*sigma_e_integral_Bulk

IF (Ra_Bulk.lt.5) then
Ra_Bulk=5
elseif ((nLUCchosen.eq.1) .and. (Ra_Bulk.gt.2000)) then
Ra_Bulk=2000
elseif ((nLUCchosen.gt.1) .and. (Ra_Bulk.gt.1000)) then
Ra_Bulk=1000
ENDIF
!**********************************************

!print*, 'Ra_Bulk'
!print*, Ra_Bulk
!print*, 'Ustar_Bulk2'
!print*, Ustar_Bulk2

!*********************************************

Ra=Ra_Bulk
Ustar=Ustar_Bulk2

!print*, 'Ra'
!print*, Ra
!print*, 'Ustar'
!print*, Ustar
IF (zenith_angle .gt. 90) then

Ustar=Ustar_Zhang2003_NIGHT(nLUCchosen)
!FIX just for testing for Peter/Radek Amazon data
!This is reasonable based on papers. E.g. Shuttleworth (1988) 
!paramaterises Ra=34.2/Uref (sm-1). Also average Ra daytime found to be 
!10sm-1 by Wallace and !McJannet (2006). Others also confirm this is 
!reasonable.
Ra = (Uref)/Ustar_Zhang2003_NIGHT(nLUCchosen)**2
elseif ((zenith_angle .ge. 0) .and. (zenith_angle .le. 90)) then
Ustar=Ustar_Zhang2003_DAY(nLUCchosen)
!FIX just for testing for Peter/Radek Amazon data
Ra = (Uref)/Ustar_Zhang2003_DAY(nLUCchosen)**2
ENDIF




!FIX just for testing 
!Ra = (Uref)/Ustar**2

!*******************************************************
diameter=radie*2.
!Knudsen number
Kn = (2*air_mean_path)/(2.*radie)
!Don't use anywhere. Maybe better to use Gcunn by Jacobson
!Gcunn2 = 1+(Kn*(1.249+(0.42*exp(-0.87/Kn))))

! Cunningham correction factor (based on Allen and Raabe (1982))
Gcunn = 1. + air_mean_path/(2.*radie)*(2.514 + 0.8*exp(-0.55*(2.*radie/air_mean_path)))

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Vg: Gravitational sedimentation velocity
Vg = ((diameter**2.)*(dens-air_density)*g*Gcunn)/(18.*air_viscosity)
!(Atkinsson et al and Jacobson uses the same approach as Slinn whereas Eiden ignored air_density. Follow Slinn approach here). 

!Vgh: Gravitational sedimentation velocity in the humid quasi-lamina layer near the air water interface.
!Fitzgerald, 1975
S=0.98
A=1.2*exp((0.066*S)/(1.058-S))
B=exp((0.00077*S)/(1.009-S))
radie_98=A*(rdry(kk)**B)
diameter_98=radie_98*2.
!p2052 Pryor, 1999
Vgh= ((diameter_98**2.)*(dens-air_density)*gravitation*Gcunn)/(18.*air_viscosity)

!Rb: Quasi-laminar layer resistance
!Consists of the resistance over the smooth surface Rb1 and resistance over the broken one Rb2.
!Fraction of surface area broken due to the wind force (Wu, 1979).
Ab = 1.7d-6*(U10m_R**3.75)

!Rb1: Quasi-laminar layer resistance over smooth surface
!Determined mostly by the Brownian diffusion and impaction (Slinn and Slinn, 1980)
!Brownian diffusion coefficient in the humid layer
Bd = (boltzmann*Temp*Gcunn)/(3.*pi*air_viscosity*(radie*2.))
Bdh = (boltzmann*Temp*Gcunn)/(3.*pi*air_viscosity*(radie_98*2.))

!Schmidt number (Slinn and Slinn, 1999)
Sc = kinematic_air_viscosity/Bd
!Should be Bdh

!Stokes number for water surfaces
St_water = Ustar**2.*Vgh/(gravitation*kinematic_air_viscosity)

!Collection Efficiencys: Consituents from Brownian diffusion, impaction
Eb_Water = Sc**-0.5
Eim_Water = 10**(-3./St_water)

!Rb1: Quasi-laminar layer resistance over smooth surface
!Determined mostly by the Brownian diffusion and impaction (Slinn and Slinn, 1980)
Rb1 = (k*Uref/(Ustar**2))*(1/(Eb_Water+Eim_Water))

!Rb2: Quasi-laminar layer resistance over broken surface
!Tentative value of 10s/m is used-lack of reliable measurements (Williams, 1982)
Rb2 = 10

!FIX Check dots 

Rb_Water = (((1-Ab)/Rb1)+(Ab/Rb2))**-1

!*************************************************  
!Vegetated Surface (Based on Slinn, 1982. Similar approach: Zhang et al, 2001)
!Canopy resistance
!E: Collection efficiency

!Diffusion term
Eb_Veg = Sc**-0.666667

!Impaction term (Peters and Eiden, 1992)
!Stokes Number for vegetated surfaces
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
small_collector_width = 1d-5 !(m)
!From Zhang lookup table (Changed from 1d-3)
large_collector_width = Charac_Radius(nLUCchosen)/1000 !(m)

!Forests
If ((nLUCchosen.le.6) .and. (nLUCchosen.ge.2)) then
e0 = 1.4
!Low Vegetaion
else 
e0 = 0.22
endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!From Zhang lookup table
!Prev was 1
a_constant = E_Constant_Alpha(nLUCchosen)
!Prev was 0.5
b_constant = E_Constant_Beta(nLUCchosen)

fitting_constant1 = 2
fitting_constant2 = 0.25
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
St_Veg = Ustar*Vg/(g*large_collector_width)
Eim_Veg = (St_Veg/(a_constant+St_Veg))**b_constant

!Interception term Ein
!The most uncertain part of the collection efficiency. Sensitivity analysis has shown this term
!to be insignificant compared to the other terms
F = 0.01
Ein = (F*((radie*2)/((radie*2)+small_collector_width)))+((1-F)*((radie*2)/((radie*2)+large_collector_width)))

!Bounce-off correction factor
!Assumed to take place from dry surfaces only (No Precipitation during previous 6hrs period for grass
!and current/previous periods for forest)
roff = exp(-(fitting_constant1)*St_Veg**fitting_constant2)

E = e0*(Eb_Veg+Ein+Eim_Veg)*roff

!Uh wind velocity at the canopy height H (Stability function required)
Gamma_momentum_function_Hminusd = (((1-((Gamma_momentum*Hminusd(nLUCchosen))/L_momentum))**0.25)-1)/(((1-((Gamma_momentum*Hminusd(nLUCchosen))/L_momentum))**0.25)+1)

!Stable
If (Ri .gt. 0.0) then
sigma_m_integral_Hminusd = Log(Hminusd(nLUCchosen)/Z0m(nLUCchosen))+((Beta_momentum/L_momentum)*(Hminusd(nLUCchosen)-Z0m(nLUCchosen)))
!Unstable
elseif (Ri .lt. 0.0) then
sigma_m_integral_Hminusd = (Log(Gamma_momentum_function_Hminusd))-(Log(Gamma_momentum_function_Z0m))+(2*atand((1-(Gamma_momentum*Hminusd(nLUCchosen)/L_momentum))**0.25))-(2*atand((1-(Gamma_momentum*(Z0m(nLUCchosen)/L_momentum)))**0.25))
!Neutral
elseif (Ri .eq. 0.0) then !Make it less sensitive (i.e allow to vary slightly around 0)
sigma_m_integral_Hminusd = Log(Hminusd(nLUCchosen)/Z0m(nLUCchosen))
Endif

Uh = (Ustar/k)*sigma_m_integral_Hminusd

IF (Uh .lt. 0.5) then
Uh=0.5
ENDIF

!Resistance of the interfacial sub-layer (Canopy Resistance)
Rs = Uh/(E*(Ustar**2))

!In order to calculate for low vegetation more accurate to follow approach by Wesely et al (1985).
!For L>=0 is the same though

if (L_momentum .ge. 0.0) then
Rs_lowveg = Uh/(E*(Ustar**2))
else if (L_momentum .lt. 0.0) then
Rs_lowveg = (Uh/(E*(Ustar**2)))*((1+((-100/L_momentum)**(2./3.)))**-1)
endif

!Zhang uses different formula which neglets canopy wind speed
!Could fix sensitivity issue. Replaces above Wesely correction.
Rs_lowveg = 1/(E*Ustar)
!Need to try, and also add new look up table for fitting constants

!********************************************************************!
!Non-vegetated surfaces (glaciers-Ice)
Eb_Nonveg = Eb_Veg

St_Nonveg = Ustar**2.*Vg/(gravitation*kinematic_air_viscosity)

Eim_Nonveg = 10**(-3./St_Nonveg)

Rb_Nonveg = ((k*Uref)/Ustar**2)*((Eb_Nonveg+Eim_Nonveg)**-1)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Urban areas (Giorgi, 1986)
Eim_Urban = St_Nonveg**2/(400+St_Nonveg**2)
Rb_Urban = ((k*Uref)/Ustar**2)*((Eb_Nonveg+Eim_Urban)**-1)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Dry Deposition Velocities of different surfaces
! Check http://www.emep.int/UniDoc/node11.html

!******************************************
!Turbulence test
!Ra=Ra*2

!******************************************
!Water
if (nLUCchosen.eq.1) then
Vd(nLUCchosen) = ((1+(Ra*Vg))*(1+(Rb_Water*Vgh)))/((Ra+Rb_Water)+(Ra*Rb_Water*Vgh))
!Vegetated FORESTS
elseif ((nLUCchosen.le.6) .and. (nLUCchosen.ge.2)) then
Vd(nLUCchosen) = (1/(Ra+Rs))+Vg
!Vegetated Low Vegetation
elseif (((nLUCchosen.le.13) .and. (nLUCchosen.ge.7)).or. (nLUCchosen.eq.15)) then
Vd(nLUCchosen) = (1/(Ra+Rs_lowveg))+Vg
!Non-Vegetated
elseif ((nLUCchosen.eq.16) .or. (nLUCchosen.eq.17)) then
Vd(nLUCchosen) = (1/(Ra+Rb_Nonveg+(Ra*Rb_Nonveg*Vg)))+Vg
!Urban
elseif (nLUCchosen.eq.14) then
Vd(nLUCchosen) = (1/(Ra+Rb_Urban+(Ra*Rb_Urban*Vg)))+Vg
endif

!print *, 'Vd'
!write(6,*) Vd(nLUCchosen)      
!pause
 
DRY_DEP_1=vd(NLUCCHOSEN)


!print *,DRY_DEP_1
!write(6,*) DRY_DEP_1


!print *,'Ra'
!write(6,*) Ra

!print *,'Rs'
!write(6,*) Rs

!print *,'Ri'
!write(6,*) Ri

!print*, dom_class

if ((dry_dep_1>0.1) .or. (dry_dep_1<0))  then

if (dry_dep_1<0) then
dry_dep_1=1d-15
end if

end if
3 continue
end do
end subroutine dry_deposition_aerosols


!oooooooooooooooooooooooooooooooooooooDEPOSITIONoooooooooooooooooooooooooooooooooooooooo!
!  Daniel Partridge ITM																	!
!																						!
!  Calculates dry deposition velocities for gases -> loss of number and volume		    !
!  concentration due to dry deposition													!
!																						!
!																						!
!																						!
!																						!
!ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo!

subroutine dry_dep_gases()
  
  implicit none

	integer, parameter :: LUC = 17, GAS=19
    integer :: ii,nLUCchosen, kk, j

 

  real*8 :: nLUC(LUC)=0.0, E_Constant_Alpha(LUC)=0.0, E_Constant_Beta(LUC)=0.0, Z0m(LUC)=0.0, LAI(LUC)=0.0, Rac0(LUC)=0.0, Charac_Radius(LUC)=0.0 
  real*8 :: precip,snowice,exponent,SR,Wst,cos_leaf_Zen,k,Pr,c,z,air_viscosity,air_density,kinematic_air_viscosity
  real*8 :: m,SVP_T,WVP_A,D,VRdir, VRdiff, WW, PRdir, PRdiff, V_R, P_R, R_Ratio, SV, FV, PARDIR,PARDIFF
  real*8 :: SO2VegDry(LUC)=0.0, Rcutd0O3(LUC)=0.0, Rcutd0SO2(LUC)=0.0, Rcutw0O3(LUC)=0.0, Rcutw0SO2(LUC)=0.0
  real*8 :: rstmin(LUC)=0.0, brs(LUC)=0.0, Tmin(LUC)=0.0, Tmax(LUC)=0.0, Topt(LUC)=0.0, bvpd(LUC)=0.0, psic1(LUC)=0.0, psic2(LUC)=0.0, sd_max(LUC)=0.0
  real*8 :: alpha(GAS)=0.0, beta(GAS)=0.0, Rm(GAS)=0.0, Di(GAS)=0.0, ScGAS(GAS)=0.0
  real*8 :: Fsun=0.0, Fshade=0.0,PARshade=0.0,PARsun=0.0,rst_PARshade=0.0,rst_PARsun=0.0,Gs_PAR=0.0,Rst=0.0
  real*8 :: bt=0.0,f_T=0.0,f_D=0.0,psi=0.0,f_psi=0.0
  real*8 :: RgO3=0.0,RgSO2=0.0,RcutO3=0.0,RcutSO2=0.0,Rac=0.0
  real*8 :: RS(GAS)=0.0,Rg(GAS)=0.0,Rcut(GAS)=0.0,Rns(GAS)=0.0,Rc(GAS)=0.0, Rb(GAS)=0.0, Vd(GAS)=0.0


  real*8 :: charnock_constant, wind_velocity_grad, Uref, Ri, L_energy, L_momentum, Beta_energy, Beta_momentum
  real*8 :: Gamma_energy, Gamma_momentum, Gamma_momentum_function_Zr, Gamma_energy_function_Zr
  real*8 :: Prt, sigma_h_integral
  real*8 :: Z0h=0.0, Gamma_momentum_function_Z0m=0.0, sigma_m_integral=0.0, Gamma_energy_function_Z0h=0.0, sigma_e_integral=0.0 
  real*8 :: potential_temp_grad, radie, sd, f_snow, Temp_f_T
 
  real*8 :: precipitation, Cp, ThetaStar, EqCC, RHcloud, CCF1, CCF2, ES_Surf_KPa, E_Surf_KPa, Td_Surf_C, Tw_GAMMA, Tw_DELTA, Tw_Surf_C, SNOW   
  real*8 :: ES_Surf_hPa, DQ, Coedew, USMIN, dew, precip_rate, ice, vegetation, non_veg
  real*8 :: dair,dh2o,DVh2o
  real*8 :: dgas(GAS)=0.0,DIG(GAS),Ratio(GAS)=0.0,MW(GAS)=0.0


real*8 :: uv_r_grad, theta_grad, uv_r, u10m_r, RI_bulk, Theta_diff, pot_temp_intep_surf, u_comp_intep_75, v_comp_intep_75, L_bulk, &
					theta_av, sigma_m_integral_bulk, downward_sw, temp_2m, percent_total_cc, accum_precip, rainfall_rate, surface_pressure, temp850, &
					sigma_e_integral_bulk, categorial_snow, categorial_ice



	integer :: i=0

	open(unit=16,file='Dep_Aerosol_Summer.dat')
	open(unit=17,file='Dep_Aerosol_Autumn.dat')
	open(unit=18,file='Dep_Aerosol_Winter.dat')
	open(unit=19,file='Dep_Aerosol_Spring.dat')

	do i = 1,LUC
	if ((month.eq.6) .or. (month.eq.7) .or. (month.eq.8))   then
	read(16,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
	elseif ((month.eq.9) .or. (month.eq.10) .or. (month.eq.11))   then
	read(17,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
    elseif ((month.eq.12) .or. (month.eq.1) .or. (month.eq.2))   then
	read(18,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
    elseif ((month.eq.3) .or. (month.eq.4) .or. (month.eq.5))   then
	read(19,*) nLUC(i), E_Constant_Alpha(i), E_Constant_Beta(i), Z0m(i), LAI(i), Rac0(i), Charac_Radius(i) 
	endif
	enddo
	close(16)
	close(17)
	close(18)
	close(19)

  open(unit=9,file='Dep_Gas_Zhang.dat')
  do i = 1,LUC
  read(9,*) nLUC(i), Rcutd0O3(i), Rcutw0O3(i), Rcutd0SO2(i), SO2VegDry(i),  rstmin(i), brs(i), Tmin(i), Tmax(i), Topt(i), bvpd(i), psic1(i), psic2(i), sd_max(i) 
  enddo
  close(9)

  open(unit=10,file='Gas_Zhang.dat')
  do j = 1,GAS
  read(10,*) alpha(j), beta(j), Rm(j), MW(j)
  enddo
  close(10)

nLUCchosen=dom_class

U10m_R=5

!************************TEST***************************
!	if (nLUCchosen.gt.1) then
!	dom_class=3
!	endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!		dom_class=2

		!!SEARCH FOR FIX

nLUCchosen=dom_class
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!FIX INPUTS
!Averaged over year, not the best!
SR = Downward_SW

!temp=293
temp_2m=temp+altitude*0.0065

wind_velocity_grad = 0.2
potential_temp_grad= 0.2
Uref = abs(UV_R)

!Von Karman constant
k=0.4
!Prandtl number
Pr=0.72
!Turbulent Prandtl number
Prt = 0.97
charnock_constant = 0.016

! ambient air properties
!FIX have at 2m?
	air_viscosity = (1.832d-5*406.4*temp**1.5)/(5093*(temp+110.4))
	air_density = 1.2929*273.15*press/(temp*pstand)	    
    kinematic_air_viscosity = air_viscosity/air_density

!Z0m over water, follow Zhangs code

if (nLUCchosen.eq.1) then
Z0m(nLUCchosen) = 0.000002*(U10m_R**2.5)
endif

!*************************************************************************
!Calc diffusivity for each gas species

     dair=0.369*29.+6.29  
     dh2o=0.369*18.+6.29

do j = 1,GAS
dgas(j) = 0.369*MW(j)+6.29
DIG(j)=0.001*Temp_2m**1.75*SQRT((29.+MW(J))/MW(J)/29.)      
DIG(j)=DIG(j)/1.0/(dair**0.3333+dgas(j)**0.3333)**2    
enddo

!Calculate the ratio of  diffusity of water vapor to the gas species

DVh2o=0.001*Temp_2m**1.75*SQRT((29.+18.)/29./18.)
DVh2o=DVh2o/(dair**0.3333+dh2o**0.3333)**2
do j = 1,GAS
Ratio(j)=DVh2o/DIG(j)
enddo

!**********************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!leaf_Zen : Angle between leaf and sun. 60 degrees. For canopy assumed to have a spherical leaf angle distribution
cos_leaf_Zen=0.5

!Only calculate stomatal resistance if there is solar radiation, LAI is not 0, and within reasonable temp range.
!(During night when stomata closed Rst has an infinite value)


IF ((SR .ge. 0.1) .and. (Temp_2m .lt. (Tmax(nLUCchosen)+273.15)).and. (Temp_2m .gt. (Tmin(nLUCchosen)+273.15)) .and. (cos(pi*zenith_angle/180) .gt. 0.001) .and. ((nLUCchosen) .ne. 1) .and. ((nLUCchosen) .ne. 16) .and. ((nLUCchosen) .ne. 17)) then

!**********************************************
!Weiss and Norman (1985)
!Downward visible radiation fluxes above the canopy from diffuse/direct beam radiation
!Calculate direct and diffuse PAR from solar radiation and solar zenith angle
!Force by clouds and substitute 600 with a parameter!

!FIX
!****PETER: Can change this
SR=600

m=1/cos(pi*zenith_angle/180)
	  
	  VRdir = 600*(exp(-0.185*(press/pstand)*m))*(cos(pi*zenith_angle/180))
	  VRdiff = 0.4*(600-VRdir)*(cos(pi*zenith_angle/180))
!Water absorption in the near infrared for 10mm of precipitable water
	  WW=-LOG(1/m)/2.302585  
      WW=-1.195+0.4459*WW-0.0345*WW**2
      WW=1320*10**WW
!Potential direct beam nr_IR radiation on a horizontal surface
!Potential diffuse beam nr_IR radiation
	  PRdir=((720.*EXP(-0.06*(press/pstand)*m))-WW)*cos(pi*zenith_angle/180)
	  PRdiff=0.6*(720-PRdir-WW)*cos(pi*zenith_angle/180)

!Estimated total visible radiation from measurement of total incoming radiation
V_R=VRdir+VRdiff
	IF (V_R .lt. 0.1) then
	V_R=0.1
	ENDIF
P_R=VRdiff+VRdiff
	IF (P_R .lt. 0.01) then
	P_R=0.01
	ENDIF
!Ratio
R_Ratio=SR/(V_R+P_R)
	IF (R_Ratio .gt. 0.9) then
	R_Ratio=0.9
	ENDIF
!TOTAL PAR
SV=R_Ratio*V_R
!Fraction of PAR in the direct beam
FV=(0.9-R_Ratio)/0.7
	IF (FV .gt. 0.99) then
	FV=0.99
	ENDIF
FV=VRdir/V_R*(1-FV**0.6667)
	IF (FV .lt. 0.01) then
	FV=0.01
	ENDIF
!PAR from Direct Radiation
PARDIR=FV*SV
!PAR from Diffuse Radiation
PARDIFF=SV-PARDIR

!Gs_PAR : Unstressed leaf stomatal conductance as a function of 
!photosynthetically active radiation (PAR) (Zhang et al., 2002a)
!Calculated as the weighted sum of conductances for sunlit and shaded leaves
!Calculate sunlit and shaded leaf area, PAR for sunlit and shaded leaves

!Photosynthetically active radiaiton (PAR)
if ((LAI(nLUCchosen).gt.2.5) .and. (SR.gt.200)) then
PARshade = (PARDIFF*exp(-0.5*(LAI(nLUCchosen)**0.8)))+((0.07*PARDIR)*((1.1-(0.1*LAI(nLUCchosen)))*exp(Cos(-pi*zenith_angle/180))))
PARsun = (((PARDIR**0.8)*(cos_leaf_Zen))/cos(pi*zenith_angle/180))+PARshade
else
PARshade = (PARDIFF*exp(-0.5*(LAI(nLUCchosen)**0.7)))+((0.07*PARDIR)*((1.1-(0.1*LAI(nLUCchosen)))*exp(Cos(-pi*zenith_angle/180))))
PARsun = ((PARDIR*(cos_leaf_Zen))/cos(pi*zenith_angle/180))+PARshade
end if

!Unstressed leaf stomatal resistance
rst_PARshade=rstmin(nLUCchosen)*(1+(brs(nLUCchosen)/PARshade)) 
rst_PARsun=rstmin(nLUCchosen)*(1+(brs(nLUCchosen)/PARsun))

!Fsun,Fshade (Norman, 1982)
Fsun = 2*cos(pi*zenith_angle/180)*(1-(exp((-0.5*LAI(nLUCchosen))/cos(pi*zenith_angle/180))))
Fshade = LAI(nLUCchosen)-Fsun

!Stomatal conductance before including effects of temperature, vapour pressure deficit and water stress.
Gs_PAR = (Fsun/rst_PARsun)+(Fshade/rst_PARshade)

Temp_f_T=Temp_2m-273.15
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Conductance-reducing effects of air temperature, f_T
bt = (Tmax(nLUCchosen)-Topt(nLUCchosen))/(Topt(nLUCchosen)-Tmin(nLUCchosen))
f_T = ((Temp_f_T-Tmin(nLUCchosen))/(Topt(nLUCchosen)-Tmin(nLUCchosen)))*(((Tmax(nLUCchosen)-Temp_f_T)/(Tmax(nLUCchosen)-Topt(nLUCchosen)))**bt)

!Conductance-reducing effects of water vapour pressure deficit, f_D
!FIX are units correct? Should be for surface temp. And in celsius? And RH (1-RH?)
SVP_T = 0.611*(exp((17.27*(Temp_2m-273.15))/((Temp_2m-273.15)+237.3)))
WVP_A = RH*SVP_T
D = SVP_T-WVP_A
f_D = 1-(bvpd(nLUCchosen)*D)

IF (f_D .gt. 1) then
f_D = 1
ENDIF

IF (f_D .lt. 0.1) then
f_D = 0.1
ENDIF

!Conductance-reducing effects of water stress (leaf water potential, f_psi)
psi = -0.72-(0.0013*SR)
f_psi = (psi-(psic2(nLUCchosen)))/((psic1(nLUCchosen))-(psic2(nLUCchosen)))

IF (f_psi .gt. 1) then
f_psi = 1
ENDIF

IF (f_psi .lt. 0.1) then
f_psi = 0.1
ENDIF


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Stomatal Resistance Rst 

Rst = 1/(Gs_PAR*f_T*f_D*f_psi)  
do j = 1,GAS
RS(j) = Rst*Ratio(j)  
enddo

	ELSE

!For water and conditions which don't meet the first IF statement
do j = 1,GAS
RS(j) = 99999.9  
enddo

ENDIF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!In-canopy Resistance  Rac

!For surfaces without canopies, (Water Bodies, Ice, Barren)
if ((nLUCchosen.eq.1) .or. (nLUCchosen.eq.16) .or. (nLUCchosen.eq.17)) then
!Should be 0
Rac=10**-25
else
Rac = (Rac0(nLUCchosen)*(LAI(nLUCchosen)**0.25))/(Ustar**2)
endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Decide if Dew or Rain occurs
!First need to calculate the cloud fraction 

!Test Two methods

! 1.) Ulden and Holstag, 1985: Equivalent Cloud cover
!First need Temperature Scale
!NEEDS MODIFYING TO WORK. See Roger Brode E-mail
!Cp=1005
!ThetaStar=-SHFLX*((air_density*Cp*Ustar)**-1)
!EqCC=((1-(ThetaStar/0.09))/0.5)**0.5

! 2.) Draxler and Hess
!Based on RH profile
!Following a procedure similar to that used in the Nested Grid Model (NGM - Tuccillo, 1988)
!A is determined from the average fractional relative humidity of the three model sigma layers
!about the layer with the maximum RH. The RH is limited to a range of 0.80 to 1.0.
!RHcloud=RH_Max
!IF (RHcloud< 0.8) then
!CCF1=0
!else
!CCF1 = (5*(RHcloud-0.8))**2
!ENDIF

! 3.) Based on GDAS data archive

CCF2=Percent_Total_CC/100


!Precipitation Rate
!Accum_Precip is 6hr accum precip in mm



Precip_Rate=Accum_Precip/6


!Compare to rainfall from trajectory file

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!SNOW AT THE SURFACE
!FIX log or log10? Not so important as not using

!Calculate SVP, ambient vapour pressure, dewpoint temp, wet bulb temp
!explicitly using the methods from Jensen et al., (1990) ASCE Manual No. 70 Pages 176-177.

ES_Surf_KPa= 0.6108*EXP(17.27*(Temp_2m-273.16)/(Temp_2m-35.86)) !Kpa (mb-Kpa,/by 10)
E_Surf_KPa= RH*ES_Surf_KPa  !KPa
Td_Surf_C= (116.9+(237.31*LOG(E_Surf_KPa)))/(16.78-LOG(E_Surf_KPa))  !Celsius
Tw_GAMMA= 0.00066*(surface_pressure*0.001) !Converts Press to KPa 
Tw_DELTA= (4098*E_Surf_KPa)/((Td_Surf_C+237.3)**2)
Tw_Surf_C= ((Tw_GAMMA*(Temp_2m-273.16))+(Tw_DELTA*Td_Surf_C))/(Tw_GAMMA+Tw_DELTA)


!From Avrid Olsen, p28. Rule of thumb for deciding if precipitation snow or rain.
SNOW=0
!IF ((Accum_Precip .ge. 1.2) .and. (temp_2m .lt. 268) .and. (Td_Surf_C .lt. 0) .and. (Temp_2m .lt. 276) .and. (Temp .lt. 274.15)) then
!Could also include IF Wet bulb temp below 2 degrees
!SNOW=1
!ENDIF

!From GDAS Archive
!Snow
IF (Categorial_Snow .eq. 1) then
SNOW=1
ENDIF
!Ice
IF (Categorial_Ice .eq. 1) then
ICE=1
ENDIF

!Snow depth for snow fraction adjustment
!Set depth to certain value. Better to underestimate than overestimate due to limited knowledge.
!Therefore set depth to 30cm
IF (SNOW .eq.1 ) then
sd=30
f_snow=sd/sd_max(nLUCchosen)
ENDIF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!Calculate the saturation vapour pressure at the surface
ES_Surf_hPa=6.108*EXP(17.27*(Temp_2m-273.16)/(Temp_2m-35.86)) !(SVP in hPa)
!Specific humidity deficit
DQ= ES_Surf_hPa*(1.- (RH/100))/10.  !kPa 
DQ= 0.622/1000.*ES_Surf_hPa*(1.- (RH/100))*1000. !g/Kg
IF (DQ .lt. 0.0001) then
DQ= 0.0001
ENDIF



!Calculate USMIN
!Janssen and Rmer, Tellus 1991
!Deposition of dew occurs if there is a downward (negative) flux of latent heat to the dew plate.
!Limit values of Ustar below which LE<0 can be derived depending on cloud cover and specific humidity deficit.

IF (CCF2.lt.0.25) then
Coedew=0.3
elseif (CCF2.ge.0.25 .and. CCF2.lt.0.75) then
Coedew=0.2
else
Coedew=0.1
ENDIF

USMIN=1.5/DQ*Coedew

!Zhang limit 0.2mm/hr therefore 1.2mm/6hr
!IF (Accum_Precip .ge. 1.2 .and. Temp.gt.273.15) then
!!precip=1
!else
!precip=0
!ENDIF

IF (Ustar.lt.USMIN .and. Temp.gt.273.15) then
dew=1
else
dew=0
ENDIF
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!*****************************
!FOR PETER: SETTING PRECIPITATION
!Set dew, snow or precip to 1 to activate
if (prec>0 .and. temp_2m>273) then
precip=0
else 
precip=0
endif



dew=0


if (month>11 .or. month<4 .and. temp_2m<272.1 .or. nlucchosen==16 .or. temp_2m<273 .and. precip==1) then
snow=1
f_snow=0.5
else
SNOW=0
end if


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Ground Resistance Rg

snowice = 70*(275.15-Temp_2m)
if (snowice<= 100) then
snowice = 100
elseif (snowice>= 500) then
snowice = 500
endif

IF (((nLUCchosen.ge.2) .and. (nLUCchosen.lt.12)) .or. (nLUCchosen.eq.13) .or. (nLUCchosen.eq.15) .or. (nLUCchosen.eq.17)) then
Vegetation=1
ENDIF

IF (nLUCchosen.eq.14) then
Non_Veg=1
ENDIF 

!Water
if (nLUCchosen.eq.1) then
RgO3 = 2000
RgSO2 = 20
!Snow/Ice
elseif ((nLUCchosen.eq.16) .or. (SNOW .eq. 1) .or. (ICE .eq. 1)) then
RgO3= 2000
RgSO2= snowice
!Vegetation Dry
elseif ((Vegetation.eq.1) .and. (precip.eq.0)) then
RgO3= 200
RgSO2= SO2VegDry(nLUCchosen)
!Vegetation Wet
elseif (((Vegetation .eq.1) .and. (precip.eq.1)) .or. (nLUCchosen.eq.12) .or. (Non_Veg .eq. 1)) then
RgO3= 500
RgSO2= 50
endif
!Dew
IF (dew .eq. 1) then
RgO3=500
RgSO2=100 
endif

!In winter when temperatures are below -1C Rgd are increased by as much as two times,
!modifying their original value as follows

exponent=exp(0.2*(-1*(-(Temp-273.15))))
if (exponent>= 2) then
exponent = 2
endif

if (Temp<272.15) then

RgSO2 = RgSO2*exponent
endif

!Snow cover fraction adjustmemt. For snow on the ground and leaves both Rg and Rcut are adjusted based on f_snow
IF (SNOW .eq. 1) then
RgO3=1/(((1-(2*f_snow))/RgO3)+((2*f_snow)/2000))
RgSO2=1/(((1-(2*f_snow))/RgSO2)+((2*f_snow)/snowice))
ENDIF
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!The constrasting chemical properties of O3 and SO2 provide a means of estimating the
!surface resistances of other substances by scaling accordng to measures of aqueous
!solubility and oxidizing capacity (Wesely, 1989).

do j = 1,GAS
Rg(j) = 1/((alpha(j)/RgSO2)+(beta(j)/RgO3))
enddo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Canopy Cuticle Resistance Rcut
!For dry/wet conditions seperately according to Zhang et al (2002b).

!Dry
if (precip.eq.0) then
RcutO3 = Rcutd0O3(nLUCchosen)/((exp(3*RH))*(LAI(nLUCchosen)**0.25)*Ustar)
RcutSO2 = Rcutd0SO2(nLUCchosen)/((exp(3*RH))*(LAI(nLUCchosen)**0.25)*Ustar)
	RcutO3 = MAX(RcutO3,100.)
	RcutSO2 = MAX(RcutSO2,100.)

!Wet
elseif (precip.eq.1) then
RcutO3 = Rcutw0O3(nLUCchosen)/((LAI(nLUCchosen)**0.5)*Ustar)
RcutSO2 = 50/((LAI(nLUCchosen)**0.5)*Ustar)
elseif (dew .eq. 1) then
RcutSO2 = 100/((LAI(nLUCchosen)**0.5)*Ustar)
	RcutSO2 = MAX(RcutSO2,20.)
endif

!In winter when temperatures are below -1C Rcutd are increased by as much as two times,
!modifying their original value as follows
if (Temp<272.15) then
RcutO3 = RcutO3*exponent
RcutSO2 = RcutSO2*exponent
	RcutO3 = MAX(RcutO3,100.)
	RcutSO2 = MAX(RcutSO2,100.)
endif

!Snow cover fraction adjustmemt. For snow on the ground and leaves both Rg and Rcut are adjusted based on f_snow
IF (SNOW .eq.1 ) then
RcutO3=1/(((1-f_snow)/RcutO3)+(f_snow/2000))
RcutSO2=1/(((1-f_snow)/RcutSO2)+(f_snow/snowice))
ENDIF

do j = 1,GAS
Rcut(j) = 1/((alpha(j)/RcutSO2)+(beta(j)/RcutO3))
enddo

!For surfaces without canopies, (Water Bodies, Ice, Barren)
if ((nLUCchosen.eq.1) .or. (nLUCchosen.eq.16) .or. (nLUCchosen.eq.17)) then
do j = 1,GAS
Rcut(j)=10**25
enddo
endif
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Mesophyll Resistance
!Values for Rm dependent only on the chemical species, use values specified in Zhang et al, 2002a, in table.

!For surfaces without canopies, (Water Bodies, Ice, Barren)
if ((nLUCchosen.eq.1) .or. (nLUCchosen.eq.16) .or. (nLUCchosen.eq.17)) then
do j = 1,GAS
Rm(j)=10**25
enddo
endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Wst: fraction of stomatal blocking under wet conditions
Wst = 0
IF ((precip.eq.1 .or. dew.eq.1) .and. (SR.gt.200)) then
Wst = (SR-200)/800
elseif  (SR.gt.600) then
Wst = 0.5
endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Calculating 1/Rc for surfaces with canopies. 

!Non Stomatal Resistance Rns 
do j = 1,GAS
Rns(j) = 1/((1/(Rac+Rg(j)))+(1/Rcut(j)))      
enddo

							 
!Overall canopy resistance Rc
do j = 1,GAS
Rc(j) = 1/(((1-Wst)/(RS(j)+Rm(j)))+(1/Rns(j)))         
	Rc(j) = MAX(Rc(j), 10.)
enddo

!*****************************************************************************									
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Expressions for Ra and Rb can be found in many earlier dry deposition studies
!E.g. Erisman et al, 1994; Wesely et al., 2001

!Rb Quasi-laminar sub-layer Resistance
!Is associated with transfer through the quasi-laminar layer in contact with the surface .
!Rb quantifies the way in which pollutant or heat transfer differs from momentum transfer
!in the immediate vicinity of the surface.
!Following Erisman et al, 1994

do j = 1,GAS
ScGAS(j) = kinematic_air_viscosity/DIG(j)
enddo


!For over water check to make LUC dependant and follow Hicks/Liss (1976) based on Z0m
IF (nLUCchosen .eq. 1) then
do j = 1,GAS

Rb(j) = (1/(k*Ustar))*Log((Z0m(nLUCchosen)/DIG(j))*k*Ustar) !log of zero????
enddo
else
!According to Garratt, 1999. For water In(Z0m/Z0h)=k(13.6*(Pr)**0.66667-12)
do j = 1,GAS
Rb(j) = (2/(k*Ustar))*((ScGAS(j)/Pr)**0.666667)
enddo
ENDIF



!NB In Jacobson Rb(j) = In(Z0m/Z0q)*(((ScGAS(j)/Pr)**0.66667)/k*Ustar)
!This is the same formula, the only difference is in the one I use I assume In(Z0m/Z0q)*k=2 as done elsewhere

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!Dry Deposition Velocity
!Resistance analogy E.g. Wesely and Hicks, 2000

do j = 1,GAS
Vd(j) = 1/(Ra+Rb(j)+Rc(j))

!The problem that rb may become as negative that very large and also negative deposition velocities
!appear has to be prevented. Based on physical considerations, the limitation 0 cms-1 < vd < 7 cms-1 
!is imposed to avoid occurrence of large and negative deposition velocities
	   if (Vd(j)>7d-2) then
	   Vd(j)=7d-2
	   end if  
	  
	   if (Vd(j)<0) then
	   Vd(j)=1d-15
	   end if  
enddo


	do j = 1,GAS									  
!	 write(6,*) Vd(j)
	 DRY_DEP_GAS(j)=Vd(j)
    enddo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!1 SO2
!2 H2SO4
!3 NO2
!4 O3
!5 H2o2
!6 HNO3
!7 HONO
!8 HNO4
!9 NH3
!10 HCHO
!11 pald
!12 c4A
!13 C7A
!14 FORM
!15 ACAC
!16 ROOH
!17 ONIT
!18 INIT
!19 PAN 




dry_dep_gas(1) = Vd(1)/boundary
dry_dep_gas(2) = Vd(2)/boundary
dry_dep_gas(3) = Vd(3)/boundary
dry_dep_gas(4) = Vd(4)/boundary
dry_dep_gas(5) = Vd(5)/boundary
dry_dep_gas(6) = Vd(6)/boundary
dry_dep_gas(7) = Vd(7)/boundary
dry_dep_gas(8) = Vd(8)/boundary
dry_dep_gas(9) = Vd(9)/boundary
dry_dep_gas(10) = Vd(10)/boundary
dry_dep_gas(11) = Vd(11)/boundary
dry_dep_gas(12) = Vd(12)/boundary
dry_dep_gas(13) = Vd(13)/boundary
dry_dep_gas(14) = Vd(14)/boundary
dry_dep_gas(15) = Vd(15)/boundary
dry_dep_gas(16) = Vd(16)/boundary
dry_dep_gas(17) = Vd(17)/boundary
dry_dep_gas(18) = Vd(18)/boundary
dry_dep_gas(19) = Vd(19)/boundary

!In Chem code replaced all (17) by (19) now have PAN

!	print*, boundary, dry_dep_gas(1),dry_dep_gas(2),dry_dep_gas(3),dry_dep_gas(4),dry_dep_gas(5)


end subroutine dry_dep_gases

!In order to calculate for other gases could work backwards to get alpha/beta from Vd data.


!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!qual_check ensures quality of data and flags for errors (if minor) or terminates
!
!
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
	

subroutine qual_check(a, b, c)	

integer :: i, j, k, a, c

real*8 :: nan_val,  b

do i=1, sections
nan_val=isnan(n_conc(i))
if (n_conc(i)<0 .or. isnan(n_conc(i))) then


print*, 'Negative number or NaN produced', n_conc(i), i, c, 'numb'

!flag number disagrement
 write(165, '(i5, i5, 1e14.6, i5, 1e14.6, 2i5)') 1, a, b, c, n_conc(i), i,j
!kill bin
n_conc(i)=0
vol_conc(i, 1:compo)=0


end if


	do j=1,compo
	nan_val=isnan(vol_conc(i, j))
		
	if (vol_conc(i, j)<0 .or. isnan(vol_conc(i, j))) then
	

	print*, 'Negative volume or NaN produced', vol_conc(i, j), i, j, c, 'vol'
	! flag volume disagrement
	write(165, '(i5, i5, 1e14.6, i5, 1e14.6, 2i5)') 2, a, b, c, vol_conc(i, j), i, j
	!kill bin
	vol_conc(i, :)=0
	n_conc(i)=0

	end if
	
	 end do
	 end do


do j=1, 2

if (vap_conc(j)<0) then
vap_conc(j)=0
end if

end do

do j=1, 50
do i=1, 2
nan_val=isnan(conc2(j, i))
if (conc2(j, i)<0 .or. nan_val==1) then

print*, 'Negative gas or NaN produced in conc2', j, i
write(165, '(i5, i5, 1e14.6, i5, 1e14.6, 2i5)')  3, a, b, c, conc2(j, i), j, i
conc2(j, i)=0
end if
nan_val=isnan(conc1(j, i))
if (conc1(j, i)<0 .or. nan_val==1) then
print*, 'Negative gas or NaN produced in conc1', j, i

write(165, '(i5, i5, 1e14.6, i5, 1e14.6, 2i5)')  4, a, b, c, conc1(j, i), j, i
conc1(j, i)=0
end if
end do
end do


	 end subroutine qual_check



end module dynamics_dyn
