!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
!
! Contains global variables, model parameters and constants
!
! UHMA - University of Helsinki Multicomponent Aerosol model
! (c) Hannele Korhonen 2003
!
!oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

module global_dyn

  implicit none
  save


  
  
  ! precision of real variables
  integer, parameter :: real_x = selected_real_kind (p=14,r=30)  



!**************************** MODEL PARAMETERS ****************************************

  integer, parameter :: &
			sections = 45, &			! number of size sections
			modes = 3, &				! number of modes
			layers=2, &
			comp=6

  character (len=3) :: &
			nuc_approach				! for reading which nucleation mechanism used
  
  integer, parameter :: &
			cond = 2, &					! number of condensable species
			noncond = 1, &				! number of non-condensable species
			compo = cond + noncond		! number of composition classes

  real*8, parameter :: &
			vap_limit = 1.e2, &			! threshold vapour concentration (#/m^3)
			n_limit = 1.d-5				! threshold number concentration (#/m^3) 
										! (below which bin considered empty)


!******************* PROPERTIES OF PARTICULATE COMPOUNDS *******************************
!
!	compounds in order: (1) sulphuric acid
!						(2) insoluble organic vapour
!						(3) non_condensable insoluble compound (e.g. dust)


  real*8, dimension(compo), parameter ::	&
			density = (/ 1830., 1107., 2000./), &	! density (kg/m^3)
			molarmass = (/ 98.08, 186., 12./)		! molar mass (g/mol) 

  real*8, dimension(cond)	:: &
			c_sat = (/ 0.d0, 3.d12 /), &			! saturation concentration above flat surface (#/m^3)
			surf_ten = (/55.d-3, 30.d-3 /), &		! surface tension (N/m)
			diff_vol = (/ 51.96, 51.96 /), &		! diffusion volume (????)
			alpha = (/ 1., 1. /)					! mass accomodation coefficient
			
real*8, dimension(5), parameter ::	&
	em_fac=(/ 2.65, 2.5, 1.9, 1., 1./)

	real*8, dimension(comp), parameter ::	&
			rhomat = (/ 1900., 787., 2000., 2200., 2000., 1900. /), &	
			molmat = (/ 115.d-3, 200.d-3, 12.d-3, 67.d-3, 70.d-3, 115.d-3 /), &
			solmat = (/ 1., 0.1, 0., 1., 0., 1. /), &
			vantmat = (/ 2.4, 1., 0., 2., 0., 2. /)

!******************************* CONSTANTS *********************************************

 real*8, parameter :: &	
			avogadro = 6.0221d23, &   	! Avogadro number (#/mol)
			boltzmann = 1.3807d-23, &	! Boltzmann constant (J/K)
			gravitation = 9.81,	&		! gravitational acceleration (m/s^2)
			pi = 3.1415927, &
			pstand = 1.01325e5, &		! standard pressure (Pa)
			r_g = 8.314, &				! molar gas constant (J/(mol K))	
			zero = 0.0

  real*8, parameter :: &
			molar_h2o = 18.016,		&	! molar mass of water (g/mol)
			molar_air = 28.965,		&	!	- " -	air
			molar_nh3 = 17.03			!	- " -	ammonia

  real*8, parameter :: &			! mass of one molecule (kg)
			molec_h2o = molar_h2o*1.d-3/avogadro, &
			molec_nh3 = molar_nh3*1.d-3/avogadro

  ! mass and volume of one molecule for condensable compounds													
  real*8, dimension(cond), parameter :: &
			molecmass = molarmass(1:cond)*1.d-3/avogadro, &
			molecvol = molecmass/density(1:cond)		

  real*8, parameter :: &
			scale = 1.d6				! scaling factor (m -> um)

character (len=3), parameter :: manmat(12)=(/"jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec"/) 

!************************ GLOBAL VARIABLES ***********************************************

  character (len=2) :: &
			dist_approach				! for reading which size distribution approach used

  real*8 :: &
						! temperature (K)
			pres,	&					! pressure (Pa)
							! water saturation ratio (rh/100%)
			nh3_mix						! ammonia mixing ratio [ppt]
			
  

  real*8, dimension(sections) :: &
			n_conc,		 &				! number concentration (#/m^3)
			radius,		 &				! actual radii of particles (m)
			rdry,		 &				! dry radii of particles in size sections (m)
			core,		 &				! dry volume of particles (m^3)
			mass,		 &				! mass of particles (m^3)
			uppervol,	 &				! largest possible volume in each section (m^3) 
			lowervol,  	 &				! smallest possible volume in each sections (m^3)
			midvol,		 &
			n_prim,		 &
			numbs,		&
			number, &
			active, &
			radie, &
			Bmat, &
			activerade, &
			drop_radius, &
			n_sootff, &
			n_sootBM, &
			n_orgff, &
			n_orgBM, &
			n_sulf, &
			n_sootBF, &
			n_orgBF, &
			rfitta

  real*8, dimension(sections,compo) :: &
			vol_conc, vol_prim, vol_conc_cloud, vol_ox	, vol_sootff, vol_orgff, &
			vol_sootBM, vol_orgBM, vol_sulf, vol_sootBF, vol_orgBF					! total volume of each compounds in each section (um^3/m^3)


real*8, dimension(12) :: gases

real*8 :: oh,ho2, no3,clavdi, nox, o3,so2, M, no2, no, o2, n2, hcho, ch4, co, nh3,h2o, h2o2_mix, Pso2, Po3, Ph2o2, h2o2 !gas-phase variables

real*8, dimension(layers) :: terp_conc, sesq_conc

real*8 :: temp,temp_surf, press, rh, boundary, TOD, pot_temp, cloud, res_intrus, cloudhojd, cloudbase, duration,  dry_times, prec_times, prec_aver !meteorological variables

 

Real*8 :: lat, longi, altitude, em_AEROCOM, em_unscaled ! geographical coordinates

real*8, dimension(cond) :: vap_conc


real*8, dimension(55, layers):: conc1, conc2 
real*8, dimension(17) :: em_mat
real*8 :: timestep, step, q_parcel, factorenhjj,  zenith_angle, potential_temp_grad, bound_act, cloud_case, tot_time
real*8, dimension(15, 10) :: sol_info
real*8, dimension(56) :: J_values(56)


	real*8, dimension(2) :: terp_conc_mat
	real*8, dimension(2, 2) :: vap_conc_mat
	real*8, dimension(2, sections) :: n_conc_mat
	real :: preci,  prec_int
	real*8, dimension(sections, 12) :: vol_conc_mat
    real*8 t_from_start

integer :: testreadhv, startdate, month, day,  size_level, dom_class, level_V, dissipate, monthen, layer, ml_cloud, cloudclose, vert_layer

	REAL*8, DIMENSION(70) :: Ts_level,qs_level, rTq_level, T_level,RH_level, alt_level, p_level, binnucrate
	real*8, dimension(30) :: dry_dep_gas

 real*8 :: Ustar_Bulk2, Ra_Bulk, Ra, Ustar, ri, SR, uref, ccf2, accum_precip, downward_sw, uv_r, percent_total_cc, surface_pressure, cloud_fract_ML, cloud_fract_RL, disstime, cloudtime, prec, updrft, u10m_r,  sum_prec, prec_allow


		real*8 :: ff_soot, ff_org, BM_soot, BM_org, so2ind, so2off, so2roa, so2dom, so2int, so2pow, BF_bc, BF_org, S_prim, dmsflux

		real*8, dimension(801, 3) :: cumulus_djf
		real*8, dimension(799, 3) :: cumulus_mam
		real*8, dimension(798, 3) :: cumulus_jja
		real*8, dimension(800, 3) :: cumulus_son
		real*8, dimension(801, 3) :: stratus_djf
		real*8, dimension(799, 3) :: stratus_mam
		real*8, dimension(798, 3) :: stratus_jja
		real*8, dimension(800, 3) :: stratus_son

		real*8 :: cum_djf, cum_mam, cum_jja, cum_son, strat_djf, strat_mam, strat_jja, strat_son,small_act, terpene_em, prec_acu, T_base, P_base

		real*8, dimension(720, 360) :: NDVI, NDVIMAX
		real*8, dimension(360) :: lats
		real*8, dimension(720) :: longs
		real*8, dimension(10, 8) :: BIOTABLE
		real*8, dimension(16250, 3) :: grid_all
		
		real*8 :: cloud_base, cloud_top


			real*8, dimension(720, 1440) :: needleleafes, mixedforest,domclass, evergreenbroad, deciduousbroad, decideuousneedle, &
												 openshrubs, closedshrubs, grasslands, permanentwetlands, croplands
		
		
		real*8, dimension(8, 1:sections) :: OC_N_dists, EC_N_dists

		real*8, dimension( 1:sections, compo*3*8) :: OC_vol_dists, EC_vol_dists
		real*8, dimension(30) :: prof_data
		real*8, dimension(23, 10) :: vert_prof, vert_prof_old
		real*8 :: low_cloud, C_C, C_B, new_met, sold, wind_speed
		end module global_dyn 