    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %                                                                     %
    %Function to calculate condensation of H2SO4 over size distribution and%
    %corresponding growth                                                 %
    %                                                                     %
    %                                                                     %
    %Peter Tunved, ITM/SU, 2015                                           %
    %                                                                     %
    %                                                                     %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    function  [N_cond2, CS]=CONDENSATION(Dp, N, vapor_conc, dt, nobins, temp, press)
    
    %%%%%%%%%%%%%%%%%SETS GLOBAL VARIABLES FOR USE IN FUNCTIONS%%%%%%%%%%%%
    global Na Mh2o Mh2so4 molec_h2o molec_h2so4 
    global surf_ten rho_h2so4
    global molec_vol_h2so4 saturation_conc 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%INITIALIZATION OF GLOBAL PARAMETERS USED%%%%%%%%%%%%%%%%%%%%%%%
    Na=6.022e23;                                                            %Avogadros number
    Mh2o=18e-3;                                                             %Molar mass H2O g/mol
    Mh2so4=96*1e-3;                                                         %Molar mass H2SO4 kg/mol
    molec_h2o = Mh2o/Na;                                                    %Mass of one h2o molecule
    molec_h2so4=Mh2so4/Na;                                                  %Mass of one h2So4 molecule
    surf_ten =55.d-3;                                                       %Surface tension for H2SO4, N/m2
    rho_h2so4=1900;                                                         %Density of H2SO4, kg/m3
    molec_vol_h2so4=molec_h2so4/rho_h2so4;                                  %Volume of molecule, m3
    
                                                          %Pressure, Pa
                                                                    	
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
    vol=pi./6*Dp.^3;                                                        %volume particles (m3)
    
 
                                             
 
    %%%%%%%%CALCULATION OF COLLISION RATE; VOLUME CHANGE AND CS%%%%%%%%%%%%
    [coll]=collision(temp, press, Dp);                              %Collision rate for particles/gases (s-1)
    
    kelvin=kelvin_effect(molec_vol_h2so4, temp, Dp/2);                      %Kelvin effect on Sat.Vap over particle of radius R
    
    vol_change = coll.*molec_vol_h2so4.*...                                 %volume change rate of a particle (m^3/s)
        (vapor_conc-kelvin.*saturation_conc)*dt;
  
    CS = nansum(N.*vol_change)/(molec_vol_h2so4)/dt;                                   %Condensation sink(#/ m^3 s)

    
    

    %%%%%%%%%VOLUME CHANGE TAKES EFFECT ON SD BELOW%%%%%%%%%%%%%%%%%%%%%%%%
    
    %If increase in size
    vol_new=vol+vol_change;    
    N_cond(1:nobins+1)=0;
    N_cond2(1:nobins+1)=0;
    vol_cond(1:nobins+1)=0;
    vol_cond2(1:nobins+1)=0;
    
    %%%%%%%%%%%%How much mass is gained per particle
    for ii = 1:nobins+1
    for jj = ii:nobins
    if (vol(jj) <= vol_new(ii) & vol_new(ii) < vol(jj+1)) 						
    N_cond(jj) = N_cond(jj) + N(ii);                                       %Number in bin
    vol_cond(jj) = vol_cond(jj) + vol(ii).*N(ii);                          %Volume in bin			
    vol_cond(jj) = vol_cond(jj) + vol_change(ii)*N(ii);                    %(m3/m^3)										
    end
    end              
    end
    
    %%%%%%%%%%%DETERMINE WHAT BIN THE PARTICLES END UP IN%%%%%%%%%%%%%%%%%%
	
    for ii = 2:nobins-1		
    if (N_cond(ii) > 0) 	
    new_vol = sum(vol_cond(ii))/(N_cond(ii));                               %Volume of new particle in bin
    aa = (vol(ii+1)-new_vol)/(vol(ii+1)-vol(ii));                           %Fraction to keep in bin ii
    bb = 1.-aa;                                                             %Fraction to move to bin ii+1
    N_cond2(ii) = N_cond2(ii)+aa*N_cond(ii);                                %Assign N_cond to bin ii 
    N_cond2(ii+1) = N_cond2(ii+1) + bb*N_cond(ii);                          %Assign N_cond to bin ii+1
    end
	end 
        
    if (N_cond(1) > 0)                                                      %Same as above but for bin ii=1
    vol_new2 = sum(vol_cond(1))/(N_cond(1))	;
    if (vol_new2 >= vol(1)) 
	aa = (vol(2)-vol_new2)/(vol(2)-vol(1));
	bb = 1.-aa			;
	N_cond2(1) = N_cond2(1)+aa*N_cond(1);
	N_cond2(2) = N_cond2(2) + bb*N_cond(1);					
    else
    N_cond2(1) = N_cond2(1) + vol_new2*N_cond(1)/vol(1);
    end 
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
        
   
    
        
        
  
    
     