%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                                                                         %
%  Calculates Dry deposition velocity as function of size, Temperature,   %
%  Pressure, wind speed and land use type according to:                   %
%                                                                         %
%       Vd=settling_velocity+1/(Ra+Rb+Rn) m/s                             %
%                                                                         %5
%  Written by Peter Tunved, ITM/SU, 2015                                  %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function Vd=Dry_dep(temp, press, wind_speed, land_use, Dp, mass)
                        
    
    [air_density]=density_air(temp, press);                                %Density of air, f(P, T), 'kg/m3'
    
    [visc_air]=viscosity_air(temp);                                        %viscosity of air, f(T), 'Pa*s' 
    
    [Kn]=knudsen(temp, press, Dp);                                         %Knudsen number
    
    [corr]=cunningham_correction(Kn);                                      %Cunningham correction   
    
    [diff_Dp]=diffusion_particle(temp, corr, Dp);                          %Diffusion particle (f(T, CC, Visc, Dp)), 'm2*s'
    
    [Sc]=schmidt(air_density, diff_Dp, visc_air);                          %Schmidt number of particle with size Dp
    
    [kinemat_visc]=kinematic_viscosity(air_density, visc_air);             %Kinematic viscosity of air
    
    %therm_veloc=thermal_velocity(temp, mass);                              %thermal velocity of particle
    
    
    %Actual calculation of Ra, Rb and settling velocity below%%%%%%%%%%%%%%
    [sett_veloc]=settling_velocity(Dp, temp, press, mass);                 %Gravitational settling velocity (m/s)
 
    [Ra]=aerodynamical_resistance(wind_speed, land_use);                    %Aerodynamic resistance from WS and Landuse
  
  
    Rb=Quasi_laminar_resistance(Sc, land_use, sett_veloc, kinemat_visc, Dp, temp, press);   %Quasi laminar resistance

    Vd=sett_veloc+1./(Ra+Rb+Ra.*Rb.*sett_veloc);      
    
    
   %loglog(Dp, 100*1./Rb,'-k', Dp, 100./(Ra+Rb), '-pk')
%pause%Dry deposition velocity, m/s.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end
