%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
%Calculates coagulation coefficient for multimodal coagulation as function
%of particle radius1(r1, m) and particle radius2(r2,m), temperature (K) and
%pressure(Pa), mass particle1 and mass particle2.
%Calculation relies on the Fuchs form of the Brownian coagulation
%coefficient K12 (see seinfeld and pandis for details.)
%
%Peter Tunved, 2015
%
%Usage: coag_coeff=coag_coeff(r1, r2, temp, press, m1, m2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function coag_coeff=coag_coeff(r1, r2, temp, press, m1, m2)


pstand=101325;
boltzmann = 1.3807e-23;

radius(1)=r1;                                                               %m
radius(2)=r2 ;                                                              %m
rho=1000  ;  %kg/m3

m_part(1)=m1;                                                               %4/3*pi*radius(1)^3*rho;
m_part(2)=m2;                                                               %4/3*pi*radius(2)^3*rho;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%Knudsen number%%%%%%%%%%%%%%%%%%%%%%

	
	free_path = air_mean_free_path(temp, press);	
	viscosity = viscosity_air(temp);
				


	Kn = free_path./radius;                                                 % particle Knudsen number	
	corr =cunningham_correction(Kn);                                        % Cunninghan correction factor (Allen and Raabe, Aerosol Sci. Tech. 4, 269)
	diff = diffusion_particle(temp, corr, radius*2);                        % particle diffusion coefficient (m^2/s)
	veloc = thermal_velocity(temp, m_part);                                 % mean thermal velocity of a particle (m/s)
	omega = 8.*diff./(pi*veloc)	;                                           % ????? (m) 

	vel_12 = sqrt(veloc(1)^2 + veloc(2)^2);                                 %mean relative thermal velocity
	rad_12 = sum(radius);
	dif_12 = diff(1)+diff(2);                                               %relative diffusion coefficient 
	continuum = 4.*pi*rad_12*dif_12	;                                       %flux in continuum regime					
	free_molec = pi*vel_12*rad_12^2	;                                       %flux in free molecular regime

	% flux matching according to Fuchs (1964) (e.g. Seinfeld & Pandis p. 661)
	prov1 = (rad_12+omega(1))^3 - (rad_12^2 + omega(1)^2)^1.5 ;
	prov1 = prov1/(3.*rad_12*omega(1)) - rad_12;
	prov2 = (rad_12+omega(2))^3 - (rad_12^2 + omega(2)^2)^1.5 ;
	prov2 = prov2/(3.*rad_12*omega(2)) - rad_12;

	dist = sqrt(prov1^2 + prov2^2);	% distance at which fluxes are matched

	% coagulation coefficient between particles [m^3/s]
	coag_coeff = continuum / (rad_12/(rad_12+dist) + continuum/free_molec);
 end
 
