

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Function to calculate the collision rate between (in this case) particles
%and sulfuric acid molecules. The function relies on several other functions
%stored in function library.
%
%Execution requires input of temp, pressure and particle diameter.
%
%Created by Peter Tunved, ITM/SU, 2015
%
%Usage: [collision_rate, units]=collision(temp, press, Dp)

 
function [collision_rate, units]=collision(temp, press, Dp)
   global Na Mh2so4 molec_h2o molec_h2so4 surf_ten
   global molec_vol_h2so4 diff_vol  
   

if nargin==3

    

    
 
    mass=4/3*pi*(Dp/2).^3*1000;                                             %Aerosol mass kg
    Na=6.022e23;                                                            %Avogadros number
    boltzmann = 1.3807e-23;                                                 %Boltzmann constant
    alpha=1;                                                                %mass accomodation coefficient

    diff_vol = 51.96;                                                       %Diffusion volume of condensing gas
    
	[air_mean_path, units]=air_mean_free_path(temp, press);                 %Mean free path of air
	
    [viscosity, units]=viscosity_air(temp) ;                                %Viscosity of air

	r_h2o = (3*molec_h2o/1000./(4*pi))^(1./3.);                             %radius of water molecule
	r_vap = (molec_h2so4*3./(4.*pi))^(1./3.);                               %radius of condensable molecules (m)


	[knud, units]=knudsen(temp, press, Dp);                                 %particle Knudsen number
	[corr, units]=cunningham_correction(knud) ;                             %Cunninghan correction factor (Allen and Raabe, Aerosol Sci. Tech. 4, 269)
 
	dif_part = boltzmann*temp*corr./(6*pi*viscosity.*(Dp/2));               %Particle diffusion coefficient (m^2/s)
 
	dif_vap=diffusivity(Mh2so4, diff_vol, press, temp);                     %molecular diffusion coefficient (m^2/s) for condensing vapour
    
	mean_path = 3*(dif_vap + dif_part)./sqrt(8*boltzmann*temp/pi.*...
        (1./molec_h2so4 + 1./mass));                                        %mean free path of condensation process (m)

	knud = mean_path./(Dp./2 + r_vap);
	corr = knud + 1. ;                                                      %transitional correction factor (Fuchs and Sutugin, 1971)
	
    corr = corr./(0.377*knud + 1. + 4./(3.*alpha).*(knud+knud.^2));
    
	collision_rate = 4*pi*(Dp./2 + r_vap).*(dif_vap + dif_part).*corr;      %Collision rate between molecules and particles (m^3/s)

    
    
    units='m3/s';
else
    error('Function requires input of scalar Temp, Pressure and particle diameter')
end




	

